function [g3] = addg(g1,g2)
%  ADDG:   Add two POLPAC transfer function matrices
%
%  Usage:  g3 = addg(g1,g2)
%
%  This function operates on transfer function matrices stored in modes 4
%  thru 6.  G3 is the logical sum of G1 and G2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    g1     := Transfer function matrix in modes 4 thru 6
%    g2     := Transfer function matrix in modes 4 thru 6
%
%  Output Parameter:
%  -----------------
%
%    g3     := Transfer function matrix in modes 4 thru 6
%
push('ADDG')
global debg
%
%  Check for necessary conversions first
%
[gg1,gg2] = mixmod(g1,g2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(gg1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(gg2(1,1));
%
[n1,m1] = logdim(gg1);
[n2,m2] = logdim(gg2);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 4,
    disp('ADDG: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if mod1 > 6,
    disp('ADDG: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('ADDG: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if n1 ~= n2,
    disp('ADDG: Error - Incompatible number of rows'),
    abort,
  end,
  if m1 ~= m2,
    disp('ADDG: Error - Incompatible number of columns'),
    abort,
  end,
end
%
%  Create a shell to store the result.
%
logcol = max([lcol1,lcol2]);
dom1 = domain(gg1);
g3 = zerop(n1,m1,mod1,logcol,styp1,dom1);
%
%  Perform the actual summation now.
%  Loop over all transfer functions.
%
for i=1:n1,
  for k=1:m1,
    g1ik = gget(gg1,i,k);
    g2ik = gget(gg2,i,k);
    g3ik = addg1(g1ik,g2ik);
    g3 = put(g3,g3ik,i,k);
  end,
end
%
%  The highest orders can annihilate in summation.
%  Perform a SQUEEZE for the sake of it.
%
g3 = squeeze(g3);
%
pull
return
