function [g3] = addg1(g1,g2)
%  ADDG1:  Add two POLPAC transfer functions
%
%  Usage:  g3 = addg1(g1,g2)
%
%  This function operates on single transfer functions stored in modes 4
%  thru 6.  G3 is the logical sum of G1 and G2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    g1     := Single transfer function in modes 4 thru 6
%    g2     := Single transfer function in modes 4 thru 6
%
%  Output Parameter:
%  -----------------
%
%    g3     := Single transfer function in modes 4 thru 6
%
push('ADDG1')
global debg
%
%  Check for necessary conversions first
%
[gg1,gg2] = mixmod(g1,g2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(gg1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(gg2(1,1));
%
[n1,m1] = logdim(gg1);
[n2,m2] = logdim(gg2);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 4,
    disp('ADDG1: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if mod1 > 6,
    disp('ADDG1: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('ADDG1: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if n1*m1 ~= 1,
    disp('ADDG1: Error - Operates on scalar transfer functions only'),
    abort,
  end,
  if n2*m2 ~= 1,
    disp('ADDG1: Error - Operates on scalar transfer functions only'),
    abort,
  end,
end
%
%  Decompose the transfer functions into their numerator and
%  denominator polynomials
%
[p1,q1] = tfm2plm(gg1);
[p2,q2] = tfm2plm(gg2);
%
%  Check for common denominator
%
[qr1,qr2,f] = lcd1(q1,q2);
%
%  Perform the actual summation now.
%
h1 = multp1(p1,qr2);
h2 = multp1(qr1,p2);
p3 = addp1(h1,h2);
q3 = multp1(qr1,qr2);
%
%  Multiply the common factor back in
%
if ~isempty(f),
  q3 = multp1(f,q3);
end
%
%  Concatenate them back together
%
g3 = plm2tfm(p3,q3);
g3 = reduceg(g3);
%
pull
return
