function [p3] = addp(p1,p2)
%  ADDP:   Add two POLPAC polynomial matrices
%
%  Usage:  p3 = addp(p1,p2)
%
%  This function operates on polynomial matrices stored in modes 1 thru 3.
%  P3 is the logical sum of P1 and P2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    p1     := Polynomial matrix in modes 1 thru 3
%    p2     := Polynomial matrix in modes 1 thru 3
%
%  Output Parameter:
%  -----------------
%
%    p3     := Polynomial matrix in modes 1 thru 3
%
push('ADDP')
global debg
%
%  Check for necessary conversions first
%
[pp1,pp2] = mixmod(p1,p2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(pp1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(pp2(1,1));
%
[n1,m1] = logdim(pp1);
[n2,m2] = logdim(pp2);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 1,
    disp('ADDP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if mod1 > 3,
    disp('ADDP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('ADDP: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if n1 ~= n2,
    disp('ADDP: Error - Incompatible number of rows'),
    abort,
  end,
  if m1 ~= m2,
    disp('ADDP: Error - Incompatible number of columns'),
    abort,
  end,
end
%
%  Create a shell to store the result.
%
logcol = max([lcol1,lcol2]);
dom1 = domain(pp1);
p3 = zerop(n1,m1,mod1,logcol,styp1,dom1);
%
%  Perform the actual summation now.
%  Loop over all polynomials.
%
for i=1:n1,
  for k=1:m1,
    p1ik = gget(pp1,i,k);
    p2ik = gget(pp2,i,k);
    p3ik = addp1(p1ik,p2ik);
    p3 = put(p3,p3ik,i,k);
  end,
end
%
%  The highest orders can annihilate in summation.
%  Perform a SQUEEZE for the sake of it.
%
p3 = squeeze(p3);
%
pull
return
