function [p3] = addp1(p1,p2)
%  ADDP1:  Add two POLPAC polynomials
%
%  Usage:  p3 = addp1(p1,p2)
%
%  This function operates on single polynomials stored in modes 1 thru 3.
%  P3 is the logical sum of P1 and P2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    p1     := Single polynomial in modes 1 thru 3
%    p2     := Single polynomial in modes 1 thru 3
%
%  Output Parameter:
%  -----------------
%
%    p3     := Single polynomial in modes 1 thru 3
%
push('ADDP1')
global debg
%
%  Check for necessary conversions first
%
[pp1,pp2] = mixmod(p1,p2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(pp1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(pp2(1,1));
%
[n1,m1] = logdim(pp1);
[n2,m2] = logdim(pp2);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 1,
    disp('ADDP1: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if mod1 > 3,
    disp('ADDP1: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('ADDP1: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if n1*m1 ~= 1,
    disp('ADDP1: Error - Operates on scalar polynomials only'),
    abort,
  end,
  if n2*m2 ~= 1,
    disp('ADDP1: Error - Operates on scalar polynomials only'),
    abort,
  end,
end
%
%  Perform the actual summation now.
%  In storage mode 2, start with a defactorization, then
%  treat it as if it were storage mode 1.
%
if mod1 == 2,
  pp1 = c21(pp1);
  pp2 = c21(pp2);
  [mod1,lcol1,dtyp1,styp1] = unpack(pp1(1,1));
  [modd,lcol2,dtyp2,styp2] = unpack(pp2(1,1));
end
%
%  Storage mode = 1
%
if mod1 == 1,
  %
  %  Expand the smaller of the two polynomials to the size of
  %  the larger.
  %
  logcol = max([lcol1,lcol2]);
  if lcol1 < logcol,
    pp1 = expand(pp1,logcol);
  end,
  if lcol2 < logcol,
    pp2 = expand(pp2,logcol);
  end,
  %
  %  Now, simply add up the polynomial coefficients
  %
  pp3 = pp1(2,1:logcol) + pp2(2,1:logcol);
  %
  %  Compute the new logcol and the physical destination
  %
  prow = 2;
  pcol = 1 : logcol;
end
%
%  Storage mode = 3
%
if mod1 == 3,
  %
  %  Add all trajectory values
  %
  ppp1 = pp1(3,1:lcol1);
  ppp2 = pp2(3,1:lcol2);
  pp3 = ppp1 + ppp2;
  %
  %  Compute the new logcol and the physical destination
  %
  logcol = lcol1;
  prow = 3;
  pcol = 1 : logcol;
end
%
%  Compute the new structural information.
%  Same for all three modes.
%
dtype = datatype(pp3);
%
%  Create the new data structure
%
dom1 = domain(pp1);
p3 = zerop(1,1,mod1,logcol,styp1,dom1);
%
%  Copy the convolved polynomial into the new data structure
%
p3(prow,pcol) = pp3;
%
%  Copy the structural information back into the new structure
%
nbr = ppack(mod1,logcol,dtype,styp1);
p3(1,1) = nbr;
%
%  In storage mode 2, perform now a refactorization
%
if mod2 == 2,
  p3 = c12(p3);
end
%
%  The highest orders can annihilate in summation.
%  Perform a SQUEEZE for the sake of it.
%
p3 = squeeze(p3);
%
pull
return
