function [s3] = addps(s1,s2)
%  ADDPS:  Add two POLPAC matrix polynomial systems (parallel connect.)
%
%  Usage:  s3 = addps(s1,s2)
%
%  This function connects two systems s1 and s2 specified through
%  polynomial system descriptions (modes 7 thru 9) in parallel.
%
%  Input Parameters:
%  -----------------
%
%    s1     := POLPAC polynomial matrix system description (modes 7 thru 9)
%    s2     := POLPAC polynomial matrix system description (modes 7 thru 9)
%
%  Output Parameters:
%  ------------------
%
%    s3     := POLPAC polynomial matrix system description (modes 7 thru 9)
%
push('ADDPS')
global debg
%
%  Check for necessary conversions first
%
[ss1,ss2] = mixmod(s1,s2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(ss1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(ss2(1,1));
%
[o1,i1] = logdim(ss1);
[o2,i2] = logdim(ss2);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 7,
    disp('ADDPS: Error - Operates only on modes 7 thru 9'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('ADDPS: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if i1 ~= i2,
    disp('ADDPS: Error - Incompatible number of inputs'),
    abort,
  end,
  if o1 ~= o2,
    disp('ADDPS: Error - Incompatible number of outputs'),
    abort,
  end,
end
%
%  Unpack the systems
%
[p1,q1,r1,w1] = pms2plm(ss1);
[p2,q2,r2,w2] = pms2plm(ss2);
[ns,ms] = size(ss1);
if ms == 1,
  n1 = 0;
  else
  n1 = ss1(1,2);
end
[ns,ms] = size(ss2);
if ms == 1,
  n2 = 0;
  else
  n2 = ss2(1,2);
end
%
%  Check for special cases
%
if n1 == 0,
  if n2 == 0,
    %
    %  Both systems are constant
    %
    p3 = [];
    q3 = [];
    r3 = [];
  end,
  %
  %  System 1 is constant, whereas system 2 is not
  %
  p3 = p2;
  q3 = q2;
  r3 = r2;
  else
  if n2 == 0,
    %
    %  System 2 is constant, whereas system 1 is not
    %
    p3 = p1;
    q3 = q1;
    r3 = r1;
    else
    %
    %  Parallel connect the subsystems
    %
    p3 = dconc(p1,p2);
    q3 = vconc(q1,q2);
    r3 = hconc(r1,r2);
  end,
end
w3 = addp(w1,w2);
%
%  Repack the system
%
s3 = plm2pms(p3,q3,r3,w3);
%
pull
return
