function [s3] = adds(s1,s2)
%  ADDS:   Add two POLPAC systems (parallel connection)
%
%  Usage:  s3 = adds(s1,s2)
%
%  This function connects two systems s1 and s2 specified in
%  the time domain (mode = 0) in parallel.
%
%  Input Parameters:
%  -----------------
%
%    s1     := POLPAC system description (mode = 0)
%    s2     := POLPAC system description (mode = 0)
%
%  Output Parameters:
%  ------------------
%
%    s3     := POLPAC system description (mode = 0)
%
push('ADDS')
global debg
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(s1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(s2(1,1));
%
[o1,i1] = logdim(s1);
[o2,i2] = logdim(s2);
%
%  Check for consistency
%
if debg == 3,
  if styp1 ~= styp2,
    disp('ADDS: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if i1 ~= i2,
    disp('ADDS: Error - Incompatible number of inputs'),
    abort,
  end,
  if o1 ~= o2,
    disp('ADDS: Error - Incompatible number of outputs'),
    abort,
  end,
end
%
%  Unpack the systems
%
[a1,b1,c1,d1] = oldsyst(s1);
[a2,b2,c2,d2] = oldsyst(s2);
[ns,ms] = size(s1);
if ms == 1,
  n1 = 0;
  else
  n1 = s1(1,2);
end
[ns,ms] = size(s2);
if ms == 1,
  n2 = 0;
  else
  n2 = s2(1,2);
end
%
%  Check for special cases
%
if n1 == 0,
  if n2 == 0,
    %
    %  Both systems are constant
    %
    a3 = [];
    b3 = [];
    c3 = [];
  end,
  %
  %  System 1 is constant, whereas system 2 is not
  %
  a3 = a2;
  b3 = b2;
  c3 = c2;
  else
  if n2 == 0,
    %
    %  System 2 is constant, whereas system 1 is not
    %
    a3 = a1;
    b3 = b1;
    c3 = c1;
    else
    %
    %  Neither system is constant.  Parallel connect the subsystems.
    %
    a3 = [ a1 , zeros(n1,n2) ; zeros(n2,n1) , a2 ];
    b3 = [ b1 ; b2 ];
    c3 = [ c1 , c2 ];
  end,
end
d3 = d1 + d2;
%
%  Repack the system
%
s3 = newsyst(a3,b3,c3,d3);
%
pull
return
