function [s2,gco,t] = balance(s1)
%  BALANCE:  POLPAC function to balance state-space description
%
%  Usage:  [s2,gco,t] = balance(s1)
%
%  This function transforms an either continuous or discrete
%  stable system representation in the time domain (POLPAC mode = 0)
%  into a balanced system representation.
%
%  Input Parameter:
%  -----------------
%
%    s1   := POLPAC stable system representation in time domain (mode 0).
%
%  Output Parameters:
%  -----------------
%
%    s2   := Transformed POLPAC representation in the time domain (mode 0).
%    gco  := Common diagonal controllability/observability Gramian
%    t    := The transformation matrix to go into balanced form
%            (a_balanced = t*a_original/t, etc.)
%
push('BALANCE')
global debg
%
%  Start by unpacking the structural information of the input matrix s1.
%
[mode,logcol,dtype,stype] = unpack(s1(1,1));
%
%  Check for consistency.
%
if debg == 3,
  if mode ~= 0,
    disp('BALANCE: Error - The algorithm works only in MODE = 0')
    abort,
    return,
  end,
end
%
%  Extract the A, B, C and D matrices from S1.
%
[A,B,C,D] = oldsyst(s1);
[n,n] = size(A);
[p,m] = size(D);
%
%  We first need to check whether the system is stable.
%
if debg >= 2,
  if max(real(eig(A))) >= 0,
    disp('BALANCE: Error - Balancing works only on stable systems.')
    abort,
    return,
  end,
end
%
%  We need to check whether the system is fully contollable.  If not,
%  augment the B-vector by an additional column vector of small numbers,
%  until the system is fully controllable.  If this doesn't work, add
%  yet one more vector.
%
if stype == 0,
  Gc = gram(A,B);
  else
  Gc = dgram(A,B);
end
Bnorm = norm(B,'inf');
%
kB = 0;
while rank(Gc) < n,
  kB = kB + 1;
  i = 1;
  e = eps;
  while rank(Gc) < n,
    e = 2*e;
    Baux = 100*e*rand(n,kB)*Bnorm;
    Bmod = [ B , Baux ];
    if stype == 0,
      Gc = gram(A,Bmod);
      else
      Gc = dgram(A,Bmod);
    end,
    i = i + 1;
    if i > 20, break, end,
  end,
end
%
%  We need to check whether the system is fully observable.  If not,
%  augment the C-vector by an additional row vector of small numbers,
%  until the system is fully observable.  If this doesn't work, add
%  yet one more vector.
%
if stype == 0,
  Go = gram(A',C');
  else
  Go = dgram(A',C');
end
Cnorm = norm(C,'inf');
%
kC = 0;
while rank(Go) < n,
  kC = kC + 1;
  i = 1;
  e = eps;
  while rank(Go) < n,
    e = 2*e;
    Caux = 100*e*rand(kC,n)*Cnorm;
    Cmod = [ C ; Caux ];
    if stype == 0,
      Go = gram(A',Cmod');
      else
      Go = dgram(A',Cmod');
    end,
    i = i + 1;
    if i > 20, break, end,
  end,
end
%
%  Now use BALREAL or DBALREAL to balance the system
%
[Uc,Sc,Vc] = svd(Gc);
[Uo,So,Vc] = svd(Go);
Sc = diag(diag(Sc) .^ 0.5);
So = diag(diag(So) .^ 0.5);
H = So*Uo'*Uc*Sc;
[Uh,Sh,Vh] = svd(H);
t = diag(diag(Sh) .^ 0.5)\Uh'*So*Uo';
gco = diag(Sh);
%
%  Perform the similarity transformation on the transformed system
%
Abal = t*A/t;
Bbal = t*B;
Cbal = C/t;
%
%  Now get rid of the added input/output vectors again
%
if kB > 0,
  Bbal = Bbal(1:n,1:m);
end
if kC > 0,
  Cbal = Cbal(1:p,1:n);
end
s2 = newsyst(Abal,Bbal,Cbal,D,stype);
%
pull
return
