function [s2] = c00(s1,t,ts)
%  C00:    POLPAC conversions between state-space descriptions
%
%  Usage:  s2 = c00(s1,t,ts)
%
%  This function transforms a system representation in the time domain
%  (POLPAC mode = 0) into another system representation in the time
%  domain (mode = 0).  If t is a transformation matrix and not text,
%  the transformation is made with t.  If t is text (either ccf, ocf, 
%  jcf, gw10, schr, astc, hsbg, lctod, mzctod, nfctod, dtoc, zdtoc, 
%  or fdtoc) then the system is transformed into the specified form.
%
%  Input Parameters:
%  -----------------
%
%    s1   := Polpac system representation in time domain (mode 0).
%    ts   := Sampling time.
%    t    := Either a transformation matrix or text describing new form.
%	     If t is text, it can be transformed into one of the following
%            forms:
%                   ccf = controller canonical form
%                   ocf = observer canonical form
%                   jcf = jordan canonical form
%                  gw10 = Golub-Wilkinson section 10
%                  schr = Schur decomposition
%                  astc = staircase transformation
%                  hsbg = Hessenberg transformation
%                 lctod = continuous to discrete time transformation
%                mzctod = continuous to discrete transform (zero order hold)
%                nfctod = continuous to discrete (first order hold)
%                  dtoc = Discrete to continuous time transformation
%                 zdtoc = Discrete to continuous (zero order hold)
%                 fdtoc = Discrete to continuous (first order hold)
%
%  Output Parameter:
%  -----------------
%
%    s2   := Transformed polpac representation in the time domain (mode 0).
%
push('C00')
global debg
%
%  Start by unpacking the structural information of the input matrix s1.
%
[mode,logcol,dtype,stype] = unpack(s1(1,1));
%
%  Extract the A, B, C and D matrices from s1.
%
[a,b,c,d] = oldsyst(s1);
%
%  If this is a constant system, don't do anything
%
if isempty(a),
  s2 = s1;
  pull,
  return,
end
%
%  Make an empty s2, but leave the first row the same as s1.
%
[row,col] = size(s1);
s2 = s1;
for i = 2:row,
  for k = 1:col,
    s2(i,k) = 0;
  end,
end
[rowt,colt] = size(t);
[rowa,cola] = size(a);
[rowb,colb] = size(b);
[rowc,colc] = size(c); 
[rowd,cold] = size(d); 
if rowa ~= cola,
  disp('C00 Error: The A matrix is not square'),
  abort,
end 
%
%  Determine if t is text for the transformation form or if t
%  is a transformation matrix.
%
if rowt > 1,
  if rowt ~= colt,
    disp('C00 Error: The T matrix is not square'),
    abort,
  end 
  if rowa ~= rowt,
    disp('C00 Error:  Size of T is not the same as size of A.'),
    abort,
  end,
  %
  %  Size of A and T are the same so perform transformation.
  %
  an = t*a/t;
  bn = t*b;
  cn = c/t;
  dn = d;
  s2 = newsyst(an,bn,cn,dn,stype);
  pull,
  return,
end
%
if rowt > 1,
  disp('C00 Error:  T is not a transformation matrix or text'),
  abort,
end
%
%
%  Transform into controller canonical form.
%
if strcmp(t,'ccf'),
  gmtemp = c04(s1);
  s2 = c40(gmtemp);
  pull,
  return,
end
%
%  Transform into observer canonical form.
%
if strcmp(t,'ocf'),
  s1t = transpose(s1);
  gmtemp = c04(s1t);
  s2t = c40(gmtemp);
  s2 = transpose(s2t);
  pull,
  return,
end
%
% Determine the modal matrix for multiple eigenvalues
% and then transform into Jordan canonical form.
%
if strcmp(t,'jcf'),
  [l,m] = neig2(a);
  if debg == 3,
    [l1,m1] = neig(a);
    k = 0;
    while length(l) ~= length(l1),
       chop(k);
       k = k+1;
       [l,m] = neig2(a);
       [l1,m1] = neig(a);
    end,
    chop(-1);
  end,
  [v] = modal(a,l,m);
  tr = inv(v);
  an = tr*a/tr;
  bn = tr*b;
  cn = c/tr;
  dn = d;
  s2 = newsyst(an,bn,cn,dn,stype);  
  pull,
  return,
end
%
% Algorithm from chapter 10 of Golub/Wilkinson.
%
if strcmp(t,'gw10'),
  [l,m] = neig2(a);
  [rown,coln] = size(l);
  atemp = a;
  v = eye(rowa);
  tol = 100*eps;
  ms = 0;
  %
  % Loop over all distinct eigenvalues.
  %
  for i = 1:rown,
    [rowat,colat] = size(atemp);
    all = atemp - l(i)*eye(rowat);
    if rowat > 1,
      mm = 0;
      ms = ms + m(i);
      while mm < ms,
        %
        % Iterate over all necessary steps for this eigenvalue.
        %
        [rowat,cola] = size(all);
        [ua,sa,va] = svd(all);
        vlt = eye(rowa);
        vlt(1:rowat,1:rowat) = va;
        v = v*vlt;
        m1 = rank(all,tol);
        mm = rowa - m1;
        all = va'*all*va;
        all = all(1:m1,1:m1);
      end,
      atemp = all + l(i)*eye(m1);
    end,
  end,
  tr = inv(v);
  an = tr*a/tr;
  bn = tr*b;
  cn = c/tr;
  dn = d;
  s2 = newsyst(an,bn,cn,dn,stype);  
  pull,
  return,
end
%
% Schur Decomposition.
%
if strcmp(t,'schr'),
  [u,ut] = schur(a);
  [u,ut] = rsf2csf(u,ut);
  tr = u;
  an = tr'*a*tr;
  bn = tr*b;
  cn = c/tr;
  dn = d;
  s2 = newsyst(an,bn,cn,dn,stype);  
  pull,
  return,
end
%
% Hessenberg form.
%
if strcmp(t,'hsbg'),
  [p,h] = hess(a);
  tr = p;
  an = inv(tr)*a*inv(tr');
  bn = tr*b;
  cn = c/tr;
  dn = d;
  s2 = newsyst(an,bn,cn,dn,stype);
  pull,
  return,
end
%
% Staircase form.
%
if strcmp(t,'astc'),
  [abar,bbar,cbar,tr] = ctrbf(a,b,c);
  an = tr*a*tr';
  bn = tr*b;
  cn = c*tr';
  dn = d;
  s2 = newsyst(an,bn,cn,dn,stype);
  pull,
  return,
end
%
% Continuous to discrete transformation.
%
if strcmp(t,'lctod'),
  [f,g] = c2d(a,b,ts);
  g = f * b;
  h = c;
  k = d;
  stype = 1;
  s2 = newsyst(f,g,h,k,stype);
  pull,
  return,
end
%
% Continuous to discrete transformation with
% zero order hold.
%
if strcmp(t,'mzctod'),
  [f,g] = c2d(a,b,ts);
  h = c;
  k = d;
  stype = 1;
  s2 = newsyst(f,g,h,k,stype);
  pull,
  return,
end
%
% Continuous to discrete transformation with
% first order hold.
%
if strcmp(t,'nfctod'),
  [m,n] = eig(a);
  dn = diag(n);
  [nrow, ncol] = size(dn);
  for ni = 1:nrow,
    if real(dn(ni,1)) < 100*eps,
      disp('C00 Error: Poles at the origin in the z-domain');
      s2 = s1;
      abort,
    end,
  end,
  %
  % Poles are not at the origin, transformation possible.
  %
  [ad,bd] = c2d(a,b,ts);
  b2 = inv(a*a)*((expm(ts*a))/ts)*((eye(a)-expm(-ts*a)*(eye(a)+ts*a))*b);
  ad = [ad,-b2];
  [n,m] = size(ad);
  ad = [ad;zeros(m-n,m)];
  bd = [bd+b2;eye(m-n)];
  [n1,m1] = size(c);
  cd = [c,zeros(n1,m-n)];
  dd = d;
  stype = 1;
  s2 = newsyst(ad,bd,cd,dd,stype);
  pull,
  return,
end
%
% Discrete to continuous transformation.
%
if strcmp(t,'dtoc'),
  [m,n] = eig(a);
  dn = diag(n);
  [nrow, ncol] = size(dn);
  for ni = 1:nrow,
    if real(dn(ni,1)) < 100*eps,
      disp('C00 Error: Poles at the origin in the z-domain');
      s2 = s1;
      abort,
    end,
  end,
  %
  % Poles are not at the origin, transformation possible.
  %
  f = logm(a) / ts;
  g = inv(expm(f*ts) - eye(f)) * f * b;
  h = c;
  k = d;
  stype = 0;
  s2 = newsyst(f,g,h,k,stype);
  pull,
  return,
end
%
% Discrete to continuous transformation with
% zero order hold.
%
if strcmp(t,'zdtoc'),
  [m,n] = eig(a);
  dn = diag(n);
  [nrow, ncol] = size(dn);
  for ni = 1:nrow,
    if real(dn(ni,1)) < 100*eps,
      disp('C00 Error: Poles at the origin in the z-domain');
      s2 = s1;
      abort,
    end,
  end,
  %
  % Poles are not at the origin, transformation possible.
  %
  [f,g] = d2c(a,b,ts);
  h = c;
  k = d;
  stype = 0;
  s2 = newsyst(f,g,h,k,stype);
  abort,
end
%
% Discrete to continuous transformation with
% first order hold.
%
if strcmp(t,'fdtoc'),
  temp = a(rowa-colb+1:rowa,:);
  if temp ~= zeros(colb,cola),
    disp('C00 Error: Discrete system is not represented in FOH form');
    s2 = s1;
    abort,
  end,
  an = a(1:rowa-colb,:);
  b1 = an(:,cola-colb+1:cola);
  az = an(:,1:cola-colb);
  temp = b(rowb-colb+1:rowb,:);
  if temp ~= eye(colb),
    disp('C00 Error: Discrete system is not represented in FOH form');
    s2 = s1;
    abort,
  end,
  b3 = b(1:rowb-colb,:);
  bz = b1 + b3;
  [ac,bc] = d2c(az,bz,ts);
  cc = c(:,1:cola-colb);
  dc = d;
  stype = 0;
  s2 = newsyst(ac,bc,cc,dc,stype);
  pull,
  return,
end
%
disp('C00 Error: No transformation possible')
s2
%
pull
return
