function [gm] = c04(s)
%  C04:   POLPAC conversion state-space -> transfer function matrix (coefs.)
%
%  Usage:  gm = c04(s)
%
%  This function converts a system representation in the time domain
%  (POLPAC mode = 0) into an unfactorized transfer function matrix
%  (POLPAC mode = 4).
%
%  Input Parameters:
%  -----------------
%
%    s      := Polpac system representation in time domain (mode 0).
%
%  Output Parameter:
%  -----------------
%
%    gm     := Polpac unfactorized transfer function matrix (mode 4).
%
push('C04')
%
%  Start by unpacking the structural information of the input s.
%
[mode,logcol,dtype,stype] = unpack(s(1,1));
%
%  Extract the A, B, C and D matrices from S.
%
[a,b,c,d] = oldsyst(s);
%
%  Get number of inputs (LCOL) and outputs (LROW)
%  The logical size of GM is LROWxLCOL.
%
[lrow,lcol] = size(d);
%
%  Create an empty GM.
%
mode = 4;
[ns,ms] = size(s);
if ms == 1,
  ord = 0;
  else
  ord = s(1,2);
end
%
%  First take care of the special case of a constant system
%
if ord == 0,
  pm = newmat(d,1,stype,0);
  qm = onesp(lrow,lcol,1,1,stype,0);
  gm = c14(pm,qm);
  pull,
  return,
end
%
%  Now take care of the general case
%
logcol = ord + 1;
[gm] = zerop(lrow,lcol,mode,logcol,stype,0);
%
%  Assign values to GM.
%  Notice that the order of the polynomial coefficients is reversed.
%
[p,q] = ss2tf(a,b,c,d,1);
q = q(logcol:-1:1);
for i=1:lcol,
  if i > 1,
     p = ss2tf(a,b,c,d,i);
  end,
  p = p(:,logcol:-1:1);
  i1 = (i-1)*logcol+1 : i*logcol;
  gm(2:lrow+1,i1) = p;
  for k = 1:lrow,
    gm(lrow+k+1,i1) = q;
  end,
end
%
pull
return
