function [pm2] = c12(pm1)
%  C12:    POLPAC conversion polynomial matrices: coeficients -> roots
%
%  Usage: pm2 = c12(pm1)
%
%  This routine converts a polynomial POLPAC matrix in mode 1 into a
%  similar POLPAC matrix stored in mode 2.
%
%  Input Parameter:
%  ----------------
%
%    pm1    := Input polynomial matrix in mode 1
%
%  Output Parameter:
%  -----------------
%
%    pm2    := Output polynomial matrix in mode 2
%
push('C12')
global debg
global warn
global neigflag
%
%  Start by unpacking the structural information of the input matrix.
%
[mode,logcol,dtype,stype] = unpack(pm1(1,1));
[nrow,ncol] = logdim(pm1);
% 
%  Create the mode 2 matrix. The LOGCOL that we will need is either smaller
%  or equal to the one in mode 1. Thus, we'll use this one, and at the end,
%  we'll perform a SQUEEZE.
%
pm2 = zerop(nrow,ncol,2,logcol,stype,0);
%
%  Check for consistency
%
if debg == 3,
  if mode ~= 1,
    disp('C12: Error - Input matrix must be in mode 1'),
    abort,
  end,
end
%
%  We will work on a single polynomial at any one time.
%  Dual loop to extract all of them one after the other.
%
for irow=1:nrow,
  for icol=1:ncol,
    prc = gget(pm1,irow,icol);
    [n,m] = size(prc);
    p0 = prc(2,m);
    %
    %  Check whether this is a constant polynomial
    %
    if m == 1,
      %
      %  This is a constant polynomial.  Requires special treatment
      %
      dtype = datatype(p0);
      nbr = ppack(2,1,dtype,stype);
      p2 = [ nbr ; p0 ; 0 ];
    end,
    if m > 1,
      %
      %  This is not a constant polynomial.  Get the companion matrix
      %
      cmp = companion(prc);
      %
      %  Get the eigenvalues and their multiplicities
      %
      if warn == 1,
        %
        %  if WARN=1, use both algorithms for comparison
        %
        [lamb1,mlt1] = neig1(cmp);
        [lamb2,mlt2] = neig2(cmp);
        %
        %  Check if they agree.  Since the eigenvalues have been sorted,
        %  the MLT matrices should be identical (integer numbers) and the
        %  LAMB matrices should differ only by roundoff errors.
        %
        flag1 = normcomp(mlt1,mlt2,10*eps);
        flag2 = normcomp(lamb1,lamb2,10000*eps);
        flag1 = flag1*flag2;
        if flag1 ~= 1,
          disp('C12: Warning - The two NEIG algorithms disagree'),
          lamb1,
          mlt1,
          lamb2,
          mlt2,
        end,
        %
        %  If the two algorithms indeed agreed, the solution is supposedly
        %  correct and we can store the eigenvalues and their
        %  multiplicities in a matrix in mode 2.  
        %
        if neigflag == 1,
          lamb = conj(lamb1');
          mlt = mlt1';
          else
          lamb = conj(lamb2');
          mlt = mlt2';
        end,
        else
        %
        %  If WARN=0, use the algorithm indicated by NEIGFLAG
        %
        if neigflag == 1,
          [lamb,mlt] = neig1(cmp);
          else
          [lamb,mlt] = neig2(cmp);
        end,
        lamb = conj(lamb');
        mlt = mlt';
      end,
      %
      %  Pack the eigenvalues into the new data structure
      %
      [nee,ne] = size(mlt);
      logcol = ne + 1;
      p2 = zeros(3,logcol);
      dtype = datatype(lamb);
      nbr = ppack(2,logcol,dtype,stype);
      p2(1,1) = nbr;
      p2(2,1) = p0;
      p2(3,1) = sum(mlt);
      p2(2,2:logcol) = lamb;
      p2(3,2:logcol) = mlt;
    end,
    %
    %  The single polynomial we were working on has now been converted
    %  from mode 1 to mode 2. We can put it in the final pm2 matrix.
    %
    pm2 = put(pm2,p2,irow,icol);
  end,
end
%
pull
return
