function [p2] = c13(p1,dom)
%  C13:    POLPAC conversion polynomial matrices: coeficients -> trajectories
%
%  p2 = c13(p1,dom)
%
%  This function converts the polynomial matrix P1 represented by 
%  its coefficients (mode = 1) to a polynomial matrix P2 represented 
%  by trajectories over the domain DOM (mode = 3).
%
%  Notice that P1 and P2 have the same logical dimensions, but their
%  LOGCOL value will usually be different.  P2's LOGCOL value is given
%  by the size of DOM.
%
%  Also notice that the DTYPE of P1 and P2 may be different.
%     
%  Input Parameters:
%  -----------------
%
%    p1     := Polynomial matrix (mode 1).
%    dom    := Domain row vector.
%
%  Output Parameter:
%  -----------------
%
%    p2     := Polynomial matrix (mode 3).
%
push('C13')
global debg
global warn
%
%  Start by unpacking the structural information.
%
[mode,logcol,dtype,stype] = unpack(p1(1,1));
%
%  Get the size of the domain row-vector DOM.
%
[n,dsize] = size(dom);
%
%  In debg = 3 check that P1 is in mode 1, and check whether
%  DOM is really a row vector.
%
if debg == 3,
  if mode ~= 1,
    disp('C13: Error - MODE of P1 must equal 1.'),
    mode,
    abort,
  end,
  if n ~= 1,
    disp('C13: Error - DOM must be a row vector'),
    abort,
  end,
end
%
%  Warn the user if the size of DOM is less than LOGCOL.
%
if warn == 1,
  if dsize < logcol,
    t1 = 'C13: Warning - Domain size smaller than LOGCOL.';
    t2 = '               Precision is lost in operation. ';
    txt = [ t1 ; t2 ] ;
    disp(txt),
  end,
end
%
%  Get the logical dimensions of P1
%
[lrow,lcol] = logdim(p1);
%
%  Now create a zero polynomial matrix P2 in mode 3 having domain dom.
%
mode = 3;
p2 = zerop(lrow,lcol,mode,dsize,stype,dom);
%
%  Next, compute new polynomial matrix elements and store them in P2.
%  Such elements define the trajectory over the domain DOM.
%
[prow1,pcol1] = size(p1);
[prow2,pcol2] = size(p2);
%
%  Loop over all domain values
%
for i=1:dsize,
  %
  %  Apply Horner's rule for polynomial evaluations applied
  %  to all coefficients of same power simultaneously
  %
  cof = p1(2:prow1,logcol:logcol:pcol1);
  domi = dom(i);
  for k=logcol-1:-1:1,
    cof1 = p1(2:prow1,k:logcol:pcol1-logcol+k);
    cof = domi*cof + cof1;
  end,
  %
  %  Store the result in appropriate place in P2
  %
  p2(3:prow2,i:dsize:pcol2-dsize+i) = cof;
end
%
%  Now find the DTYPE of P2.
%
dtype = datatype(p2);
%
%  Pack structural information into P2
%
mode = 3;
p2(1,1) = ppack(mode,dsize,dtype,stype);
%
pull
return
