function [gm] = c14(pm,qm)
%  C14:    POLPAC conversion polynomial matrices -> transfer function matrix
%
%  Usage: gm = c14(pm,qm)
%
%  This function converts two nonfactorized polynomial matrices
%  (stored in storage mode 1) into one nonfactorized transfer
%  function matrix (stored in storage mode 4).
%
%  Input Parameters:
%  -----------------
%
%    pm     := Numerator polynomial matrix (mode = 1)
%    qm     := Denominator polynomial matrix (mode = 1)
%
%  Output Parameter:
%  -----------------
%
%    gm     := Transfer function matrix (mode = 4)
%
push('C14')
global debg
global warn
%
%  Start by unpacking the structural information
%
[mod1,logco1,dtyp1,styp1] = unpack(pm(1,1));
[mod2,logco2,dtyp2,styp2] = unpack(qm(1,1));
%
%  Check for consistency
%
if debg == 3,
  if mod1 ~= 1,
    disp('C14: Error - Mode of PM must be 1'),
    mod1,
    abort,
  end,
  if mod2 ~= 1,
    disp('C14: Error - Mode of QM must be 1'),
    mod2,
    abort,
  end,
  if styp1 ~= styp2,
    disp('C14: Error - STYPE of PM ~= STYPE of QM'),
    abort,
  end,
end
if debg >= 2,
  [m1,n1] = logdim(pm);
  [m2,n2] = logdim(qm);
  if m1 ~= m2,
    disp('C14: Error - Logical rows are incompatible'),
    abort,
  end,
  if n1 ~= n2,
    disp('C14: Error - Logical columns are incompatible'),
    abort,
  end,
end
%
%  Expand PM or QM if LOGCOL's are not equal
%
pmn = pm;
qmn = qm;
logcol = logco1;
if logco1 ~= logco2,
  dff = logco1 - logco2;
  if dff > 100*eps,
    qmn = expand(qm,logco1);
    else
    pmn = expand(pm,logco2);
    logcol = logco2;
  end,
end
%
%  if any denominator polynomial is zero (i.e. has all zero
%  coefficients), issue a warning message.  Extract one
%  coefficient vector after the other (TST), and check whether
%  any of them is all zero.
%
if warn == 1,
  [nq,mq] = logdim(qmn);
  check = 0;
  for i=1:nq,
    for k=1:mq,
      [pi,pk] = pmap(qmn,i,k);
      tst = qmn(pi,pk);
      if norm(tst,'inf') < 100*eps,
        check = 1;
      end,
    end,
  end,
  if check == 1,
    disp('C14: Warning - At least one denominator is zero'),
  end,
end
%
%  Set the data type to the larger of the two
%
dtype = max([dtyp1,dtyp2]);
%
%  Now, convert to storage mode 4
%
mode = 4;
[nn,mm] = size(qmn);
gm = [pmn;qmn(2:nn,1:mm)];
gm(1,1) = ppack(mode,logcol,dtype,styp1);
%
pull
return
