function [p2] = c21(p1);
%  C21:    POLPAC conversion polynomial matrices: roots -> coefficients
%
%  Usage: p2 = c21(p1);
%
%  This function converts the factorized polynomial matrix p1
%  to its corresponding nonfactorized form.
%
%  The input matrix will be checked for correct dimensioning and valid 
%  multiplicity values.
%
%  Input Parameter:
%  ----------------
%
%    p1     := Polynomial matrix in mode 2
%
%  Output Parameter:
%  -----------------
%
%    p2     := Polynomial matrix in mode 1  
%
push('C21')
global debg
%
[mode,logcol,dtype,stype] = unpack(p1(1,1));
%
[m,n] = size(p1);
[lrow,lcol] = logdim(p1);
%
%  if debg == 3 check that P1 is in mode 2
%
if debg == 3,
  if mode ~= 2,
    disp('C21: Error - MODE of P1 must equal 2.'),
    mode,
    abort,
  end,
end
%
%  Construct P2 to acccept the largest polynomial
%       
ord = p1(3:2:m,1:logcol:n-logcol+1);
highord = max(max(ord));
p2 = zerop(lrow,lcol,1,highord+1,stype,0);
%
%  Construct gain vector
%
gain = p1(2:2:m-1,1:logcol:n-logcol+1);
% 
%  Loop over all logical rows and all logical columns
%
for l=1:lrow,
  for k=1:lcol,
    gainlk = gain(l,k);    
    ordlk = ord(l,k);
    rts = 0;
    if gainlk ~= 0,
      coefs = gainlk;
      if ordlk > 0,
        %
        %  Loop over all roots of polynomial (l,k)
        %  Compute a complete set of roots
        %
        for i=(k-1)*logcol+2:k*logcol,
          rti = p1(2*l,i);
          mlti = p1(2*l+1,i);
          if mlti ~= 0,
            rts = [ rts ; rti*ones(mlti,1) ];
          end,
        end,
        rts = rts(2:ordlk+1,1);
        %
        %  Compute now the corresponding coefficients
        %  and reverse them at once
        %
        coefs = poly(rts);
        coefs = gainlk*conj(coefs');
        coefs = coefs(ordlk+1:-1:1);
      end,
      %
      %  Place it in the new matrix at the right place.
      %
      k1 = (k - 1)*(highord + 1) + 1;
      k2 = k1 + ordlk;
      p2(l+1,k1:k2) = coefs';
    end,
  end,
end
%
%  Now find the DTYPE of P2
%
dtype = datatype(p2);
%
% Pack the structural information.
%
mode = 1;
p2(1,1) = ppack(mode,highord+1,dtype,stype);
%
pull
return
