function [p2] = c23(p1,dom)
%  C23:    POLPAC conversion polynomial matrices: roots -> trajectories
%
%  Usage:  p2 = c23(p1,dom)
%
%  This function converts the polynomial matrix P1 represented by 
%  its roots (mode = 2) to a polynomial matrix P2 represented 
%  by trajectories over the domain DOM (mode = 3).
%
%  P2's LOGCOL value is given by the size of DOM (called dsize).  
%  P1's LOGCOL value has no relationship to P2's.
%  Also, notice that the DTYPE of P1 and P2 may be different.
%
%  The program starts by finding the polynomial in the p1 matrix with 
%  the most roots and thus will have the largest order (maxord).  The 
%  size of domain must be at least 1 greater than this maxord to 
%  properly represent the polynomial with trajectories.  If not, a 
%  warning message is issued.
%
%  The top left polynomial in the p1 matrix is accessed first and then 
%  subsequent polynomials on the same row followed by the next rows 
%  ending at the bottom right corner of the matrix.  for each 
%  polynomial, the roots are placed in a column vector (vec) the number 
%  of times corresponding to its multiplicity.  A single domain value 
%  is placed in a column vector (domvec) which is made to be the same 
%  size as vec.  This s value will be added to the negative of vec to 
%  yield the vector newvec.  Each term in this vector is multiplied 
%  together by the procedure PROD and this product is then multiplied 
%  by the gain factor of this polynomial to yield the trajectory for 
%  the specific domain value.  The result is then placed in p2
%  appropriately.
%
%  After the trajectories are all placed in p2, DTYPE is found and the 
%  new matrix is PACKed.
% 
%  Input Parameters:
%  -----------------
%    p1     := Polynomial matrix (mode 2).
%    dom    := Domain row vector.  Note that each polynomial in the new
%              POLPAC structure uses the same domain.
%
%  Output Parameter:
%  -----------------
%    p2     := Polynomial matrix (mode 3).
%
push('C23')
global debg
global warn
%
%  Start by UNPACKing the structural information.
%
[mode,logcol,dtype,stype] = unpack(p1(1,1));
%
%  Get the size of the domain row-vector DOM.
%
[n,dsize] = size(dom);
%
%  If debg == 3 check that P1 is in mode 2, and check whether
%  DOM is really a row vector.
%
if debg == 3,
  if mode ~= 2,
    disp('C23: Error - MODE of P1 must equal 2.'),
    mode,
    abort,
  end,
  if n ~= 1,
    disp('C23: Error - DOM must be a row vector.'),
    abort,
  end,
end
%
%  Get the logical dimensions of P1.
%
[lrow,lcol] = logdim(p1);
%
%  Find the largest polynomial term order, make sure the domain
%  DOM will be large enough, and warn the user if the size of DOM is 
%  less than the order plus one.
%
[row1,col1] = size(p1);
ord = p1(3:2:row1,1:logcol:col1-logcol+1);
maxord = max(max(abs(ord)));
%
if warn == 1,
  if dsize < maxord+1,
    t1 = 'C23: Warning - Domain size is smaller than the order';
    t2 = '               of the system plus one.              ';
    t3 = '               Precision is lost in operation.      ';
    txt = [ t1 ; t2 ; t3 ];
    disp(txt),
  end,
end
%  
%  Create a zero polynomial matrix P2 in mode 3 having domain DOM.
%
mode = 3;
p2 = zerop(lrow,lcol,mode,dsize,stype,dom);
%
%  Next, compute new polynomial matrix elements and store them in P2.
%  Such elements define the trajectory over the domain DOM.  It will 
%  start placing trajectories in row 3 of p2.
%
nrow = 2;
for row=2:2:row1-1,
  nrow = nrow + 1;
  %
  %  k is used to span across logical blocks.
  %
  for k=0:lcol-1,
    vec = 0;
    %
    %  col is used to span across roots in each polynomial.
    %
    for col=2:logcol,
      %
      %  if the multiplicity equal 0, disregard root.
      %
      if p1(row+1,col+(k*logcol)) ~= 0,
        %
        %  Place root in vec multiplicity number of times.
        %
        vec = [vec;p1(row,col+(k*logcol))*ones(p1(row+1,col+(k*logcol)),1)];
      end,
    end,
    [n,m] = size(vec);
    %
    %  If the size of vec equal 1, vec only has the initial 0 placed
    %  in it.  This means that the polynomial is constant, so use the
    %  gain factor.
    %
    if n ~= 1,
      vec = vec(2:n);
      [n,m] = size(vec);
      for i=1:dsize,
        %
        %  Build domvec and add it to the negative of vec.
        %
        domvec = dom(i)*ones(n,1);
        newvec = domvec - vec;
        %
        %  Do multiplication and place the trajectory value in p2.
        %
        p2(nrow,i+(k*dsize)) = prod(newvec) * p1(row,1+(k*logcol));
      end,
      else
      %
      % There are no roots, so trajectory equals the gain value.
      %
      for i=1:dsize,
        p2(nrow,i+(k*dsize)) = p1(row,1+(k*logcol));
      end,
    end,
  end,
end
%
%  Now find the DTYPE of P2.
%
dtype = datatype(p2);
%
%  PACK structural information into p2.
%
mode = 3;
p2(1,1) = ppack(mode,dsize,dtype,stype);
%
pull
return
