function [gm] = c25(pm,qm)
%  C25:    POLPAC conversion polynomial matrices -> transfer function matrix
%
%  Usage:  gm = c25(pm,qm)
%
%  This function converts two factorized polynomial matrices
%  (stored in storage mode 2) into one factorized transfer
%  function matrix (stored in storage mode 5).
%
%  Input Parameters:
%  -----------------
%
%    pm     := Numerator polynomial matrix (mode = 2)
%    qm     := Denominator polynomial matrix (mode = 2)
%
%  Output Parameter:
%  -----------------
%
%    gm     := Transfer function matrix (mode = 5)
%
push('C25')
global debg
global warn
%
%  Start by unpacking the structural information
%
[mod1,logco1,dtyp1,styp1] = unpack(pm(1,1));
[mod2,logco2,dtyp2,styp2] = unpack(qm(1,1));
%
%  Check for consistency
%
if debg == 3,
  if mod1 ~= 2,
    disp('C25: Error - Mode of PM must be 2'),
    mod1,
    abort,
  end,
  if mod2 ~= 2,
    disp('C25: Error - Mode of QM must be 2'),
    mod2,
    abort,
  end,
  if styp1 ~= styp2,
    disp('C25: Error - STYPE of PM ~= STYPE of QM'),
    abort,
  end,
end
if debg >= 2,
  [m1,n1] = logdim(pm);
  [m2,n2] = logdim(qm);
  if m1 ~= m2,
    disp('C25: Error - Logical rows are incompatible'),
    abort,
  end,
  if n1 ~= n2,
    disp('C25: Error - Logical columns are incompatible'),
    abort,
  end,
end
%
%  Expand PM or QM if LOGCOL's are not equal
%
pmn = pm;
qmn = qm;
logcol = logco1;
if logco1 ~= logco2,
  dff = logco1 - logco2;
  if dff > 100*eps,
    qmn = expand(qm,logco1);
    else
    pmn = expand(pm,logco2);
    logcol = logco2;
  end,
end
%
%  if any denominator polynomial is zero (i.e. has a zero
%  gain factor), issue a warning message.  Extract the
%  gain factors into an auxillary matrix (GFCTR), and check for
%  absolute minimum value (TST).
%
if warn == 1,
  [nq,mq] = size(qmn);
  gfctr = qmn(2:2:nq-1,1:logcol:mq-logcol+1);
  tst = min(min(abs(gfctr)));
  if tst < 100*eps,
    disp('C25: Warning - At least one denominator is zero'),
  end,
end
%
%  Set the data type to the larger of the two
%
dtype = max([dtyp1,dtyp2]);
%
%  Now, convert to storage mode 5
%
mode = 5;
[nn,mm] = size(qmn);
gm = [pmn;qmn(2:nn,1:mm)];
gm(1,1) = ppack(mode,logcol,dtype,styp1);
%
pull
return
