function [ps] = c28(p,q,r,w)
%  C28:    POLPAC conversion polynomial matrices -> polynomial matrix system
%
%  Usage:  ps = c28(p,q,r,w)
%
%  This function concatenates four factorized polynomial
%  matrices P,Q,R and W stored in POLPAC storage mode 2 into a
%  factorized polynomial matrix system representation
%  stored in POLPAC storage mode 8.
%
%  Input Parameters:
%  -----------------
%
%    p      :=  Partial State polynomial matrix (POLPAC mode = 2)
%    q      :=  Input polynomial matrix (POLPAC mode = 2)
%    r      :=  Output polynomial matrix (POLPAC mode = 2)
%    w      :=  Input-output polynomial matrix (POLPAC mode = 2)
%
%  Output Parameter:
%  -----------------
%
%    ps     :=  System polynomial matrix (POLPAC mode = 8)
%
%  The function first checks whether MODE and STYPE are legal.
%  Since the highest system order of all four polynomial matrices is needed,
%  this function will unpack each matrix regardless of the debug variable 
%  value. Mode and stype compatiblity will be checked in a debg = 3 setting.
%
push('C28')
global debg
%
%  Start with the special case of a constant system.
%
if isempty(p),
  [mm,ll,dd,ss] = unpack(w(1,1));
  ps = w;
  ps(1,1) = ppack(mm+6,ll,dd,ss);
  pull,
  return,
end
%
%  Start with unpacking the structural information
%
[mm,ll,dd,ss] = unpack(p(1,1));
m(1) = mm; l(1) = ll; d(1) = dd; s(1) = ss;
[mm,ll,dd,ss] = unpack(q(1,1));
m(2) = mm; l(2) = ll; d(2) = dd; s(2) = ss;
[mm,ll,dd,ss] = unpack(r(1,1));
m(3) = mm; l(3) = ll; d(3) = dd; s(3) = ss;
[mm,ll,dd,ss] = unpack(w(1,1));
m(4) = mm; l(4) = ll; d(4) = dd; s(4) = ss;
%
%  Check for MODE and STYPE consistency
%
if debg == 3,
  for i=1:4,
    if m(i) ~= 2,
      disp('C28: Error - MODE must equal 2.'),
      abort,
    end,
  end,
  for i=2:4,
    if s(i) ~= s(1),
      t1 = 'C28: Error - The STYPE of each polynomial matrix';
      t2 = '             must be the same.                  ';
      txt = [ t1 ; t2 ];
      disp(txt),
      abort,
    end,
  end,
end
%
%  Check for correct number of logical rows and columns
%
if debg >= 2,
  [q1,q2] = logdim(p);
  if q1 ~= q2,
    disp('C28: Error - The P polynomial matrix is not logically square'),
    abort,
  end,
  [q2,mx] = logdim(q);
  if q2 ~= q1,
    t1 = 'C28: Error - P and Q polynomial matrices have a';
    t2 = '             different number of logical rows. ';
    txt = [ t1 ; t2 ] ;
    disp(txt),
    abort,
  end,
  [p1,q2] = logdim(r);
  if q2 ~= q1,
    t1 = 'C28: Error - P and R polynomial matrices have a  ';
    t2 = '             different number of logical columns.';
    txt = [ t1 ; t2 ] ;
    disp(txt),
    abort,
  end,
  [p2,m1x] = logdim(w);
  if mx ~= m1x,
    t1 = 'C28: Error - Q and W polynomial matrices have a  ';
    t2 = '             different number of logical columns.';
    txt = [ t1 ; t2 ] ;
    disp(txt),
    abort,
  end,
  if p2 ~= p1,
    t1 = 'C28: Error - R and W polynomial matrices have a';
    t2 = '             different number of logical rows. ';
    txt = [ t1 ; t2 ] ;
    disp(txt),
    abort,
  end,
end
%
%  Find the largest DTYPE and LOGCOL among P,Q,R and W and assign
%  that value to the DTYPE and the LOGCOL of PS.
%
dtype = max(d);
logcol = max(l);
mode = 8;
%
%  Since STYPEs have been checked inside DEBG to be all equal,
%  just pick any for our STYPE.
%
stype = s(1);
%
%  In case the LOGCOL's are not equal, EXPAND all of them to the
%  largest size.
%
pp = p;
if l(1) < logcol,
  pp = expand(p,logcol);
end
qq = q;
if l(2) < logcol,
  qq = expand(q,logcol);
end
rr = minus(r);
if l(3) < logcol,
  rr = expand(rr,logcol);
end
ww = w;
if l(4) < logcol,
  ww = expand(w,logcol);
end
%
%  Then concatenate, first eliminating the first row of P,Q,R and W.
%
[n,m] = size(pp);
[n1,m] = size(rr);
ps = [ pp(2:n,:) , qq(2:n,:) ; rr(2:n1,:) , ww(2:n1,:) ];
%
%  Logical length of partial state vector q2 is equal to
%  the number of columns in P(s).
%
[q1,q2] = logdim(pp);
%
nbr = ppack(mode,logcol,dtype,stype);
%
%  Concatenate the top row
%
[n,m] = size(ps);
ps = [ nbr , q2 , zeros(1,m-2) ; ps ];
%
pull
return
