function [p2] = c31(p1,mcol)
%  C31:    POLPAC conversion polynomial matrices: trajectories -> coefficients
%
%  Usage:  p2 = c31(p1,mcol)
%
%  This function converts a POLPAC data structure in mode 3
%  to a POLPAC polynomial matrix in mode 1.
%
%  Input Parameters:
%  -----------------
%
%    p1     := POLPAC data structure in storage mode 3
%    mcol   := Non-negative integer which, if non-zero, equals
%              the largest allowed LOGCOL value of output matrix
%              default: mcol = 0
%
%  Output Parameters:
%  ------------------
%
%    p2     := POLPAC polynomial matrix, storage mode 1
%
push('C31')
global debg
global warn
%
%  Set default values
%
if nargin == 1,
  mcol = 0;
end
%
%  Unpack structural information
%
[mode,logcol,dtype,stype] = unpack(p1(1,1));
[lr,lc] = logdim(p1);
%
%  Check for consistency
%
if debg == 3,
  if mode ~= 3,
    disp('C31: Error - Operates on mode 3 only'),
    abort,
  end,
end
%
%  Extract the domain information
%
dom = domain(p1);
%
%  Issue warning if requested logcol value of output matrix
%  so large as to raise concerns over error accumulation
%
if warn == 1,
  if mcol > 10,
    disp('C31: Warning - mcol > 10, likely to accumulate errors'),
  end,
end
%
%  Check consistency
%
if debg >= 2,
  if mcol < -0.5,
    disp('C31: Error - mcol must be non-negative'),
    abort,
  end,
end
mmcol = mcol;
if (mcol - logcol) > 0.5,
  if warn == 1,
    disp('C31: Warning - mcol cannot exceed logcol of input matrix'),
  end,
  mmcol = logcol;
end
%
%  if the user left it up to the system which MCOL to use, use the
%  smaller of LOGCOL and 10.
%
if abs(mmcol) < 100*eps,
  mmcol = min([logcol,10]);
end
%
%  Get the required Van-der-Monde matrix
%
vander = vdm(dom,mmcol);
%
supval = zeros(logcol,lr*lc);
%
for i=1:lc,
  supval(:,(i-1)*lr+1:i*lr) = conj(p1(3:lr+2,(i-1)*logcol+1:i*logcol)');
end
%
coeff = vander\supval;
%
p2 = zeros(1+lr,lc*mmcol);
%
for i=1:lc,
  p2(2:lr+1,(i-1)*mmcol+1:i*mmcol) = conj(coeff(:,(i-1)*lr+1:i*lr)');
end
%
p2(1,1) = ppack(1,mmcol,datatype(p2),stype);
%
%  Here belongs a SQUEEZE call to reduce logcol value of
%  output file.  However, it is a little delicate to decide
%  which values are to be considered equal to zero.  We use
%  an experimental formulae which led to decent results in the
%  experiments that we carried out.
%
epssave = eps;
eps = eps*mmcol*(logcol - mmcol + 2);
p2 = squeeze(p2);
eps = epssave;
%
pull
return
