function [p2] = c33(p1,newdomain)
%  C33:    POLPAC conversion polynomial matrices: trajectories domain change
%
%  Usage:  p2 = c33(p1,newdomain)
%
%  This function converts a trajectory POLPAC polynomial matrix (p1) into
%  another trajectory POLPAC matrix (p2) with domain (newdomain). 
%
%  The input matrix has to be coded in mode 3.
%  The output matrix is generated also in mode 3.
%
%  Input Parameters:
%  -----------------
%
%    p1        :=  POLPAC data structure in mode 3.
%    newdomain :=  Desired new domain.
%
%  Output Parameter:
%  -----------------
%
%    p2        :=  New trajectory matrix in mode 3.
%
push('C33')
global debg
%
%  Someday, we may also use splines -- so we have a way
%  to choose between.  Set useaitken = 1 for AITKEN's method
%  and set useaitken = 0 to use splines
%
useaitken = 1;  % now a local var -- someday a GLOB()
%
%  We start by UNPACKing the structural information
%
[mode,logcol,dtype,stype] = unpack(p1(1,1));
%
%  Check for consistency
%
if debg == 3,
  if mode ~= 3,
    disp('C31: Error - Operates on mode3 only'),
    abort,
  end,
end
%
%  Extract the original domain
%
olddomain = domain(p1);
%
%  Determine whether either domain is complex
%
xtype = datatype(olddomain);
ytype = datatype(newdomain);
complex = 0;
if xtype == 3, complex = 1; end
if ytype == 3, complex = 1; end
%
%  Find out whether the new domain is entirely contained
%  inside the old domain (inside=1) or not.  But watch,
%  MIN and MAX only work on real data.
%
inside = 1;
if complex == 0,
  if max(newdomain) > max(olddomain), inside = 0; end,
  if min(newdomain) < min(olddomain), inside = 0; end,
end
%
%   Here goes the inside/outside function for complex numbers:
%   What follows is a bounding rectangle version to quickly
%   evaluate if new domain is outside old domain
%
if complex == 1,  
  if max(real(newdomain)) > max(real(olddomain)), inside = 0; end,
  if min(real(newdomain)) < min(real(olddomain)), inside = 0; end,
  if max(imag(newdomain)) > max(imag(olddomain)), inside = 0; end,
  if min(imag(newdomain)) < min(imag(olddomain)), inside = 0; end,
end
%
%  If we still think the new domain is inside the old
%  domain, we check using a more exact method.  Note:
%  The bounding rectangle check was quick and easy - now
%  we spend more time to get a little better answer
%
%  Rotate both domains 45 degrees and do another bounding
%  rectangle check.
%
if complex == 1,
  if inside == 1,
    %
    %  Fast and dirty 45 degree rotation
    %
    x = cos(pi/4);
    y = sin(pi/4);
    rnew1 = real(newdomain);
    inew1 = imag(newdomain);
    rold1 = real(olddomain);
    iold1 = imag(olddomain);
    new1 = (x*rnew1-y*inew1) + j*(y*rnew1+x*inew1);
    old1 = (x*rold1-y*iold1) + j*(y*rold1+x*iold1);
    if max(real(new1)) > max(real(old1)), inside = 0; end,
    if min(real(new1)) < min(real(old1)), inside = 0; end,
    if max(imag(new1)) > max(imag(old1)), inside = 0; end,
    if min(imag(new1)) < min(imag(old1)), inside = 0; end,
  end,
end
%
%   Use AITKEN or SPLINES methods if the newdomain is inside the olddomain
%
if inside == 1,
  [m,n] = size(p1);
  [m1,n1] = size(newdomain);
  col = round(n/logcol+.25);
  p2 = zeros(m,col*n1);
  p2(2,1:n1) = newdomain;
  for i = 3:m,
    for k = 1:col,
      if useaitken == 1,
        k1 = (k-1)*n1+1 : k*n1;
        k2 = (k-1)*logcol+1 : k*logcol;
        p2(i,k1) = aitken(olddomain,p1(i,k2),newdomain);
      end,
      if useaitken ~= 1,
        disp('Error: C33 - splines not currently supported'),
        abort,
      end,
    end,
  end,
  dtype = datatype(p2);
  p2(1,1) = ppack(mode,n1,dtype,stype);
  p2 = squeeze(p2);
end
%
%   Use this method if the newdomain is outside the olddomain
%
if inside == 0,
  p3 = c31(p1,logcol);
  p2 = c13(p3,newdomain);
end
%
pull
return
