function [gm] = c36(pm,qm)
%  C36:    POLPAC conversion polynomial matrices -> transfer function matrix
%
%  Usage:  gm = c36(pm,qm)
%
%  This function converts two trajectory POLPAC polynomial matrices
%  (stored in storage mode 3) into one trajectory POLPAC transfer
%  function matrix (stored in storage mode 6).
%
%  Input Parameters:
%  -----------------
%
%    pm     := Numerator polynomial matrix (mode = 3)
%    qm     := Denominator polynomial matrix (mode = 3)
%
%  Output Parameter:
%  -----------------
%
%    gm     := Transfer function matrix (mode = 6)
%
push('C36')
global debg
global warn
%
%  Start by unpacking the structural information
%
[mod1,logco1,dtyp1,styp1] = unpack(pm(1,1));
[mod2,logco2,dtyp2,styp2] = unpack(qm(1,1));
%
%  Check for consistency
%
if debg == 3,
  if mod1 ~= 3,
    disp('C36: Error - Mode of PM must be 3'),
    mod1,
    abort,
  end,
  if mod2 ~= 3,
    disp('C36: Error - Mode of QM must be 3'),
    mod2,
    abort,
  end,
  if styp1 ~= styp2,
    disp('C36: Error - STYPE of PM ~= STYPE of QM'),
    styp1,
    styp2,
    abort,
  end,
end
if debg >= 2,
  [m1,n1] = logdim(pm);
  [m2,n2] = logdim(qm);
  if m1 ~= m2,
    disp('C36: Error - Logical rows are incompatible'),
    m1,
    m2,
    abort,
  end,
  if n1 ~= n2,
    disp('C36: Error - Logical columns are incompatible'),
    n1,
    n2,
    abort,
  end,
  %
  % check for identity of domains
  %
  if logco1 ~= logco2,
    t1 = 'C36: Error - LOGCOL of PM ~= LOGCOL of QM';
    t2 = '         ==> Domain of PM ~= Domain of QM';
    txt = [ t1 ; t2 ];
    disp(txt),
    logco1,
    logco2,
    abort,
  end,
  dom1 = domain(pm);
  dom2 = domain(qm);
  if normcomp(dom1,dom2,100*eps) == 0,
    disp('C36: Error - Domain of PM ~= Domain of QM'),
    abort,
  end,
end
%
%  If any denominator trajectory vector is zero (i.e. has all zero
%  trajectories associated with the domain), issue a warning message.
%  Extract one trajectory vector after the other (TST), and check whether
%  any of them is all zero.
%
if warn == 1,
  [nq,mq] = logdim(qm);
  check = 0;
  for i=1:nq,
    for k=1:mq,
      [pi,pk] = pmap(qm,i,k);
      tst = qm(pi,pk);
      if norm(tst,'inf') < 100*eps,
        check = 1;
      end,
    end,
  end,
  if check == 1,
    disp('C36: Warning - At least one denominator is zero'),
  end,
end
%
%  Set the data type to the larger of the two
%
dtype = max([dtyp1,dtyp2]);
%
%  Now, convert to storage mode 6
%
mode = 6;
[nn,mm] = size(qm);
gm = [pm;qm(3:nn,1:mm)];
gm(1,1) = ppack(mode,logco1,dtype,styp1);
%
pull
return
