function [ps] = c39(p,q,r,w)
%  C39:    POLPAC conversion polynomial matrices -> polynomial matrix system
%
%  Usage:  ps = c39(p,q,r,w)
%
%  This function concatenates four trajectory polynomial matrices P,Q,R and W
%  stored in POLPAC storage mode 3 into a trajectory polynomial matrix system
%  representation stored in POLPAC storage mode 9.
%
%  Input Parameters:
%  -----------------
%
%    p      :=  Partial State polynomial matrix (POLPAC mode = 3)
%    q      :=  Input polynomial matrix (POLPAC mode = 3)
%    r      :=  Output polynomial matrix (POLPAC mode = 3)
%    w      :=  Input-output polynomial matrix (POLPAC mode = 3)
%
%  Output Parameter:
%  -----------------
%
%    ps     :=  System polynomial matrix (POLPAC mode = 9)
%
%  The function first checks whether MODE and STYPE are legal.
%  Since the highest system order of all four polynomial matrices is needed,
%  this function will unpack each matrix regardless of the debug variable 
%  value. Mode and stype compatiblity will be checked in a debg == 3 setting.
%
push('C39')
global debg
%
%  Start with the special case of a constant system.
%
if isempty(p),
  [mm,ll,dd,ss] = unpack(w(1,1));
  ps = w;
  ps(1,1) = ppack(mm+6,ll,dd,ss);
  pull,
  return,
end
%
%  Start with unpacking the structural information
%
[mm,ll,dd,ss] = unpack(p(1,1));
m(1) = mm; l(1) = ll; d(1) = dd; s(1) = ss;
[mm,ll,dd,ss] = unpack(q(1,1));
m(2) = mm; l(2) = ll; d(2) = dd; s(2) = ss;
[mm,ll,dd,ss] = unpack(r(1,1));
m(3) = mm; l(3) = ll; d(3) = dd; s(3) = ss;
[mm,ll,dd,ss] = unpack(w(1,1));
m(4) = mm; l(4) = ll; d(4) = dd; s(4) = ss;
%
%  Check for MODE and STYPE consistency
%
if debg == 3,
  for i=1:4,
    if m(i) ~= 3,
      disp('C39: Error - MODE must equal 3.'),
      abort,
    end,
  end,
  for i=2:4,
    if s(i) ~= s(1),
      t1 = 'C39: Error - The STYPE of each polynomial matrix';
      t2 = '             must be the same.                  ';
      txt = [ t1 ; t2 ];
      disp(txt),
      abort,
    end,
  end,
end
%
%  Check for identity of domains
%
logcol = l(1);
dom = domain(p);
if debg >= 2,
  for i=2:4,
    if l(i) ~= logcol,
      t1 = 'C39: Error - The LOGCOL of each trajectory polynomial'; 
      t2 = '             must be the same since domain must be   ';
      t3 = '             identical for C39.                      ';
      txt = [ t1 ; t2 ; t3 ];
      disp(txt),
      logcolumn1 = logcol,
      logcolumnk = l(i),
      abort,
    end,
  end,
  dom2 = domain(q);
  if normcomp(dom,dom2,100*eps) == 0,  
    disp('C39: Error - The domains of P and Q are not identical') ,
    abort,
  end,     
  dom2 = domain(r);
  if normcomp(dom,dom2,100*eps) == 0,  
    disp('C39: Error - The domains of P and R are not identical') ,
    abort,
  end,     
  dom2 = domain(w);
  if normcomp(dom,dom2,100*eps) == 0,  
    disp('C39: Error - The domains of P and W are not identical') ,
    abort,
  end,
  %
  %  Check for correct number of logical rows and columns
  %
  [q1,q2] = logdim(p);
  if q1 ~= q2,
    disp('C39: Error - The P polynomial matrix is not logically square'),
    q1,
    q2,
    abort,
  end,
  [q2,mx] = logdim(q);
  if q2 ~= q1,
    t1 = 'C39: Error - P and Q polynomial matrices have a';
    t2 = '             different number of logical rows. ';
    txt = [ t1 ; t2 ];
    disp(txt),
    q1,
    q2,
    abort,
  end,
  [p1,q2] = logdim(r);
  if q2 ~= q1,
    t1 = 'C39: Error - P and R polynomial matrices have a  ';
    t2 = '             different number of logical columns.';
    txt = [ t1 ; t2 ];
    disp(txt),
    q1,
    q2,
    abort,
  end,
  [p2,m1x] = logdim(w);
  if mx ~= m1x,
    t1 = 'C39: Error - Q and W polynomial matrices have a  ';
    t2 = '             different number of logical columns.';
    txt = [ t1 ; t2 ];
    disp(txt),
    mx,
    m1x,
    abort,
  end,
  if p2 ~= p1,
    t1 = 'C39: Error - R and W polynomial matrices have a';
    t2 = '             different number of logical rows. ';
    txt = [ t1 ; t2 ];
    disp(txt),
    p1,
    p2,
    abort,
  end,
end
%
%  Find the largest DTYPE among P,Q,R and W and assign
%  that value to the DTYPE of PS.
%
dtype = max(d);
mode = 9;
%
%  Since STYPEs have been checked inside DEBG to be all equal,
%  just pick any for our STYPE.
%
stype = s(1);
%
%  Negate the R(s)-polynomial matrix
%
rr = minus(r);
%
%  Then concatenate, first eliminating the first two rows of P,Q,R and W.
%
[n,m] = size(p);
[n1,m] = size(rr);
ps = [ p(3:n,:) , q(3:n,:) ; rr(3:n1,:) , w(3:n1,:) ];
%
%  Logical length of partial state vector q2 is equal to
%  the number of logicol columns of P.  
%
[q1,q2] = logdim(p);
%
nbr = ppack(mode,logcol,dtype,stype);
%
%  Concatenate the top two rows
%
[n,mn] = size(ps);
ps = [ nbr , q2 , zeros(1,mn-2) ; dom , zeros(1,mn-logcol) ; ps ];
%
pull
return
