function [s] = c40(g4)
%  C40:    POLPAC conversion transfer function matrix (coefs) -> state-space
%
%  Usage: s = c40(g4)
%
%  This function converts a transfer function matrix in POLPAC mode 4
%  into a state space system representation in POLPAC mode 0.
%  The mode 0 system S will be a controller canonical realization.
%
%  Input Parameter:
%  ----------------
%
%    g4     := Transfer function matrix represented through
%              coefficients (POLPAC mode 4).
%
%  Output Parameter:
%  -----------------
%
%    s      := System matrix (POLPAC mode 0).
%
push('C40')
global debg
%
%  First of all SQUEEZE G4.
%
gm = squeeze(g4);
%
%  UNPACK data structure.
%
[mode,logcol,dtype,stype] = unpack(gm(1,1));
%
%  Check that GM is in mode 4.
%
if debg == 3,
  if mode ~= 4,
    disp('C40: Error - Input must be in mode 4'),
    mode,
    abort,
  end,
end
%
%  Find the logical dimensions of GM.
%
[lr,lc] = logdim(gm);
%
%  Check to see whether GM is proper or strictly proper.
%  If GM is not proper, issue an error message and abort.
%
prp = isproper(gm);
if prp == 0,
  t1 = 'C40: Error - Transfer function is not proper     ';
  t2 = '             No state space representation exists';
  disp([t1;t2]),
  abort,
end
if prp == 1,
  [gms,ps] = strictprop(gm);
  d = oldmat(ps);
end
if prp == 2,
  gms = gm;
  d = zeros(lr,lc);
end
%
%  UNPACK GMS
%
[modes,logcols,dtypes,stype] = unpack(gms(1,1));
%
%  If logcols == 1, this is a static system.
%
if logcols == 1,
  a = [];
  b = [];
  c = [];
  s = newsyst(a,b,c,d,stype);
  pull,
  return,
end
%
%  Build the  A, B, and C matrices.
%
acount = 1;
a = 0;
b = zeros(1,lc);
c = zeros(lr,1);
for i=1:lc,
  gmk = gget(gms,[1:lr],i);
  [mdec,lgcolc,dtpe,stpe] = unpack(gmk(1,1));
  if lgcolc > 1,
    [pm,cdp] = commonden(gmk);
    [mdec,lgcolc,dtpe,stpe] = unpack(cdp(1,1));
    if lgcolc > 1,
      lg1 = lgcolc - 1;
      cp = zeros(1,lg1);
      for jj=1:lr,
        pmj = gget(pm,jj,1);
        cm = oldpoly(pmj);
        [r,cl] = size(cm);
        cp = [ cp ; zeros(1,lg1-cl) , cm ];
      end,
      cp = cp(2:lr+1,lg1:-1:1);
      c = [ c , cp ];
      dm = oldpoly(cdp);
      if lgcolc > 2,
        am = diag(ones(lgcolc-2,1),1);
        am(lg1,:) = -dm(lgcolc:-1:2);
        else
        am = -dm(2);
      end,
      a = [ a , zeros(acount,lg1) ; zeros(lg1,acount) , am ];
      bm = zeros(lg1,lc);
      bm(lg1,i) = 1;
      b = [ b ; bm ];
      acount = acount + lg1;
    end,
  end,
end
a = a(2:acount,2:acount);
b = b(2:acount,:);
c = c(:,2:acount);
%
%  Form the S matrix.
%
s = newsyst(a,b,c,d,stype);
%
pull
return
