function [pm,qm] = c41(gm)
%  C41:    POLPAC conversion transfer function matrix -> polynomial matrices
%
%  Usage:  [pm,qm] = c41(gm)
%
%  This function converts an unfactorized POLPAC transfer function matrix
%  into two unfactorized polynomial POLPAC matrices. One is the numerator
%  matrix, the other one is the denominator matrix.
%
%  The input matrix has to be coded in mode 4.
%  The two output matrices are being generated in mode 1.
%
%  Input Parameters:
%  -----------------
%
%    gm     := POLPAC data structure in mode 4.
%
%  Output Parameter:
%  -----------------
%
%    pm     := Numerator polynomial matrix, POLPAC mode = 1
%    qm     := Denominator polynomial matrix, POLPAC mode = 1
%
push('C41')
global debg
%
%  We start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(gm(1,1));
[n,m] = logdim(gm);
[n1,m1] = size(gm);
%
%  Check for consistency
%
if debg == 3,
  if mode ~= 4,
    disp('C41: Error - MODE must be 4'),
    mode,
    abort,
  end,
end
%
%  Now start to create the new data structures
%
mode = 1;
%
%  Extract the numerator polynomial
%
pm = gm(2:n+1,:);
%
%  Concatenate with blank top row
%
pm = [ zeros(1,m1) ; pm ];
%
%  Fill in the correct structural information
%  It may be necessary to recompute the data type
%
dtype = datatype(pm);
pm(1,1) = ppack(mode,logcol,dtype,stype);
%
%  Squeeze to minimal LOGCOL value.
%
pm = squeeze(pm);
%
%  Now, repeat the same steps with denominator polynomial
%
qm = gm(n+2:n1,:);
qm = [ zeros(1,m1) ; qm ];
dtype = datatype(qm);
qm(1,1) = ppack(mode,logcol,dtype,stype);
qm = squeeze(qm);
%
pull
return
