function [pm,qm] = c52(gm)
%  C52:    POLPAC conversion transfer function matrix -> polynomial matrices
%
%  Usage:  [pm,qm] = c52(gm)
%
%  This function  converts a factorized POLPAC transfer function matrix
%  into two factorized polynomial POLPAC matrices. One is the numerator
%  matrix, the other one is the denominator matrix.
%
%  The input matrix has to be coded in mode 5.
%  The two output matrices are being generated in mode 2.
%
%  Input Parameters:
%  -----------------
%
%    gm     := POLPAC data structure in mode 5.
%
%  Output Parameter:
%  -----------------
%
%    pm     := Numerator polynomial matrix, POLPAC mode = 2
%    qm     := Denominator polynomial matrix, POLPAC mode = 2
%
push('C52')
global debg
%
%  We start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(gm(1,1));
[n,m] = logdim(gm);
[n1,m1] = size(gm);
%
%  Check for consistency
%
if debg == 3,
  if mode ~= 5,
    disp('C52: Error - MODE must be 5'),
    mode,
    abort,
  end,
end
%
%  Now start to create the new data structures
%
mode = 2;
%
%  Extract the numerator polynomial roots and multiplicities
%
pm = gm(2:2*n+1,:);
%
%  Concatenate with blank top row
%
pm = [ zeros(1,m1) ; pm ];
%
%  Fill in the correct structural information
%  It may be necessary to recompute the data type
%
dtype = datatype(pm);
pm(1,1) = ppack(mode,logcol,dtype,stype);
%
%  Squeeze to minimal LOGCOL value.
%
pm = squeeze(pm);
%
%  Now, repeat the same steps with denominator polynomial roots
%
qm = gm(2*n+2:n1,:);
qm = [ zeros(1,m1) ; qm ];
dtype = datatype(qm);
qm(1,1) = ppack(mode,logcol,dtype,stype);
qm = squeeze(qm);
%
pull
return
