function [pm,qm] = c63(gm)
%  C63:    POLPAC conversion transfer function matrix -> polynomial matrices
%
%  Usage:  [pm,qm] = c63(gm)
%
%  This function  converts a trajectory POLPAC transfer function matrix
%  into two trajectory polynomial POLPAC matrices. One is the numerator
%  matrix, the other one is the denominator matrix. The user cannot
%  modify the domains. 
%
%  The input matrix has to be coded in mode 6.
%  The two output matrices are being generated in mode 3 with the same
%  supporting domain as the initial matrix.
%
%  Input Parameters:
%  -----------------
%
%    gm     := POLPAC data structure in mode 6.
%
%  Output Parameter:
%  -----------------
%
%    pm     := Numerator polynomial matrix, POLPAC mode = 3
%    qm     := Denominator polynomial matrix, POLPAC mode = 3
%
push('C63')
global debg
%
%  We start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(gm(1,1));
[n,m] = logdim(gm);
[n1,m1] = size(gm);
%
%  Check for consistency
%
if debg == 3,
  if mode ~= 6,
    disp('C63: Error - MODE must be 6'),
    mode,
    abort,
  end,
end
%
%  Now start to create the new data structures
%
mode = 3;
%
%  Extract the numerator trajectories and domain.
%
pm = gm(2:n+2,:);
%
%  Concatenate with blank top row
%
pm = [ zeros(1,m1) ; pm ];
%
%  Fill in the correct structural information
%  It may be necessary to recompute the data type
%
dtype = datatype(pm);
pm(1,1) = ppack(mode,logcol,dtype,stype);
%
%  Now, repeat the same steps with denominator polynomials.
%  Concatenate domain line with denominator trajectories.
%
qm = [ gm(2,:) ; gm(n+3:n1,:) ];
%
%  Concatenate with top blank line.
%
qm = [ zeros(1,m1) ; qm ];
dtype = datatype(qm);
qm(1,1) = ppack(mode,logcol,dtype,stype);
%
pull
return
