function [pms2] = c77(pms1,t,ts)
%  transformation, polynomial matrix system, coefficients, mode = 7
%
%  This function transforms a polynomial matrix system description, 
%  described by coefficients (POLPAC mode = 7), into another polynomial matrix 
%  system description, described by coefficients (POPLAC mode = 7). If t is
text 
%  (either ccf, ocf, jcf, gw10, schr, astc, hsbg, lctod, mzctod, nfctod, dtoc,
%  zdtoc, or fdtoc) then the system is transformed into the specified form.
%  If t is a transformation matrix(unimodular) and not text, ts has to be 
%  another transformation matrix(unimodular) the transformation is made with 
%  t as left unimodular transformation matrix and ts as right unimodular 
%  transformation matrix.  
%
%  Input Parameters:
%  -----------------
%
%    pms1 := Polpac polynomial matrix system description(coefficients, mode
= 7).
%    t    := Either text describing new form or first transformation matrix.
%    ts   := Either Sampling time (only in text case) or second transformation
%            matrix.
%
%	     If t is text, it can be transformed into one of the following
%            forms:
%                   ccf = controller canonical form
%                   ocf = observer canonical form
%                   jcf = jordan canonical form
%                 lctod = continuous to discrete time transformation
%                mzctod = continuous to discrete transform (zero order hold)
%                nfctod = continuous to discrete (first order hold)
%                  dtoc = Discrete to continuous time transformation
%                 zdtoc = Discrete to continuous (zero order hold)
%                 fdtoc = Discrete to continuous (first order hold)
%
%  Output Parameter:
%  -----------------
%
%    pms2   := Transformed Polpac polynomial matrix system description
%              (coefficients, mode = 7).
%
push('C77')
global debg

if nargin==2
% third argument is not defined; 
  ts=[];
end %if

if strcmp(t,'ccf')
% transformation into controller canonical form

%
% convert to mode 4
%
  tf1=c74(pms1);

  tf2 = tf1;   % same transfer function for similarity transformations

%
% convert back from 4 to 7 => in controller canonical form
%
  pms2 = c47(tf2);

elseif strcmp(t,'ocf')
% transformation into observer canonical form

%
% duality transformation
%

  pms1b = transpose(pms1);

%
% convert to mode 4
%
  tfm1=c74(pms1b);

  tfm2 = tfm1;   % same transfer function for similarity transformations

%
% convert back from 4 to 7 => in controller canonical form
%
  pms2b = c47(tfm2);

%
% duality transformation
%

  pms2 = transpose(pms2b);

elseif strcmp(t,'jcf') 
% transformation into jordan canonical form

%
% convert to mode 4
%
  tfm1=c74(pms1);

%
% do a Smith MacMillan diagonalization
%
  [pmu,tfms,pmv] = svdg(tfm1);

%
% determine P(s), Q(s), R(s), and W(s)
%
 
  pms2b = tfm2pms(tfms);
  [pm2b,qm2b,rm2b,wm2b] = pms2plm(pms2b);

%
% not necessary !!!! because above works !!!!
%  [s1,s2] = logdim(tfms);
%  [mode,logcol,dtype,stype] = unpack(tfms(1,1));
%% create zero structures
%  
%  pm2b = zerop(s1,s2,mode-3,logcol,stype,0);
%  qm2b = zerop(s1,s2,mode-3,logcol,stype,0);
%  rm2b = zerop(s1,s2,mode-3,logcol,stype,0);
%  wm2b = zerop(s1,s2,mode-3,logcol,stype,0);
%
%  if s1 > 1,
%    for k=1:s1,
%      tf = gget(tfms,k,k); %=> one transfer function on the diagonal
%      nom=tf(2,1:logcol);
%      den=tf(3,1:logcol);
%      pm2b = put(pm2b,den,k,k);
%      qm2b = put(qm2b,nom,k,k);
%      rm2b = put(rm2b,1,k,k); 
%      wm2b = put(wm2b,0,k,k);
%    end,
%  end

%
% influence of the unimodular transformation matrices pmu,pmv
%
  pm2 = pm2b;
  qm2 = multp(qm2b,pmv);
  rm2 = multp(pmu,rm2b);
  wm2 = multp(multp(pmu,wm2b),pmv);

%
% combine polynomial matrices into polynomial matrix system description
%
  [pms2] = plm2pms(pm2,qm2,rm2,wm2);
 


elseif strcmp(t,'lctod') | strcmp(t,'mzctod') | strcmp(t,'nfctod') ...
                    | strcmp(t,'dtoc') | strcmp(t,'zdtoc') | strcmp(t,'fdtoc') 
% transformation from discrete to continuous, or from continuous to discrete

%
% W(s), or combined R,Q is not zero and the system is not proper !!!!
%
  tf1=c74(pms1);
  pms1b=c47(tf1);  % => direct feed through matrix is combined into only W
  [pm1b,qm1b,rm1b,wm1b] = pms2plm(pms1b);
  wm1c = evalp(wm1b,0); % => constant part
  wmnotprop=sub(wm1b,wm1c)
  pms1c=plm2pms(pm1b,qm1b,rm1b,wm1c); %=> is a (not strictly) proper system
%  
%  not proper system wmnotprop is linearly superposed
%
  if max(max(order(wmnotprop))) > 0 % contains powers of s in case of not
proper system 
     disp('c77 Error: not proper system part is not transformed');
     show(wmnotprop);
  end 

disp('now break')
pause 

  s1=c70(pms1c);

%
% do transformation in time domain description
%
  s2=c00(s1,t,ts);

%
% convert back from 0 to 7 => in controller canonical form
%
  pms2 = c07(s2);

else  
% transformation by two unimodular matrices
  pml=t;
  pmr=ts;
  [model,logcoll,dtypel,stypel] = unpack(pml(1,1));
  [moder,logcolr,dtyper,styper] = unpack(pmr(1,1));

% check for two polynomial transformation matrices
  if (model < 1 | model > 3) | (moder < 1 | moder > 3)
    disp('c77 Error: Two polynomial matrices expected for transformation');
    model,
    moder, 
    abort;
  end % if

% check for unimodular left matrix
  pmldet = rdetp(pml);
  ordpmldet = order(pmldet);
  olmax = max(max(ordpmldet));
  if olmax>0 
     disp('c77 Error: Left transformation matrix is not unimodular');
     olmax,
     abort;
  end %if

% check for unimodular right matrix
  pmrdet = rdetp(pmr);  
  ordpmrdet = order(pmrdet);
  ormax = max(max(ordpmrdet));
  if ormax>0 
     disp('c77 Error: Right transformation matrix is not unimodular');
     ormax,
     abort;
  end %if

% probably not necessary
%% convert transformation matrices into same mode
%  if model==3
%    pml=c31(pml);
%  elseif model==2
%    pml=c21(pml);
%  else
%    %is already in mode=1
%  end %if
%
%  if moder==3
%    pmr=c31(pmr);
%  elseif moder==2
%    pmr=c21(pmr);
%  else
%    %is already in mode=1
%  end %if


% separate polynomial matrices out of polynomial matrix system description
  [pm1,qm1,rm1,wm1] = pms2plm(pms1);

%
% do the transformation, multiplications
%
  pm2 = multp(multp(pml,pm1),pmr);
  qm2 = multp(pml,qm1);
  rm2 = multp(rm1,pmr);
  wm2 = wm1;

% combine polynomial matrices into polynomial matrix system description
  [pms2] = plm2pms(pm2,qm2,rm2,wm2);

end %if

pull

return
