function [pm,qm,rm,wm] = c93(ps)
%  C93:    POLPAC conversion polynomial matrix system -> polynomial matrices
%
%  Usage:  [pm,qm,rm,wm] = c93(ps)
%
%  This function converts a POLPAC polynomial matrix system
%  description into four POLPAC polynomial matrices.
%
%  Input Parameters:
%  -----------------
%
%    ps     := POLPAC data structure in mode 9.
%
%  Output Parameters:
%  ------------------
%
%    pm     := Partial state coupling matrix (stored in mode 3)
%    qm     := Input coupling matrix (stored in mode 3)
%    rm     := Output coupling matrix (stored in mode 3)
%    wm     := Input/output coupling matrix (stored in mode 3)
%
push('C93')
global debg
%
%  We start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(ps(1,1));
%
%  Get the logical dimensions.  LOGDIM returns:
%    p := number of system outputs
%    m := number of system inputs
%
[p,m] = logdim(ps);
%
%  Get the length of the partial state vector
%
[ns,ms] = size(ps);
if ms == 1,
  q = 0;
  else
  q = ps(1,2);
end
%
%  Start with the special case of a constant system.
%
if q == 0,
  pm = [];
  qm = [];
  rm = [];
  wm = ps;
  wm(1,1) = ppack(mode-6,logcol,dtype,stype);
  pull,
  return,
end
%
%  Check for consistency
%
if debg == 3,
  if mode ~= 9,
    disp('C93: Error - PS must be stored in mode 9'),
    mode,
    abort,
  end,
  if m < 0.5,
    disp('C93: Error - System has no inputs, cannot handle'),
    abort,
  end,
  if p < 0.5,
    disp('C93: Error - System has no outputs, cannot handle'),
    abort,
  end,
end
%
%  Now start to create the new data structures
%
mode = 3;
%
%  Create empty shells to contain the four polynomial matrices
%
pm = zeros(q+2,q*logcol);
qm = zeros(q+2,m*logcol);
rm = zeros(p+2,q*logcol);
wm = zeros(p+2,m*logcol);
%
%  Move the DOMAINs
%
pm(2,1:logcol) = ps(2,1:logcol);
qm(2,1:logcol) = ps(2,1:logcol);
rm(2,1:logcol) = ps(2,1:logcol);
wm(2,1:logcol) = ps(2,1:logcol);
%
%  Move the polynomials
%
pm(3:q+2,1:q*logcol) = ps(3:q+2,1:q*logcol);
qm(3:q+2,1:m*logcol) = ps(3:q+2,q*logcol+1:(q+m)*logcol);
rm(3:p+2,1:q*logcol) = ps(q+3:q+p+2,1:q*logcol);
wm(3:p+2,1:m*logcol) = ps(q+3:q+p+2,q*logcol+1:(q+m)*logcol);
%
%  Check the new types, and copy the structural information back
%
dtype = datatype(pm);
pm(1,1) = ppack(mode,logcol,dtype,stype);
dtype = datatype(qm);
qm(1,1) = ppack(mode,logcol,dtype,stype);
dtype = datatype(rm);
rm(1,1) = ppack(mode,logcol,dtype,stype);
dtype = datatype(wm);
wm(1,1) = ppack(mode,logcol,dtype,stype);
%
%  Negate the R(s) polynomial matrix
%
rm = minus(rm);
%
%  Finally SQUEEZE for the sake of it
%
pm = squeeze(pm);
qm = squeeze(qm);
rm = squeeze(rm);
wm = squeeze(wm);
%
pull
return
