function [pm,cdp] = commonden(gm1)
%  COMMONDEN:  POLPAC function to find smallest common denominator
%
%  Usage:  [pm,cdp] = commonden(gm1)
%
%  This function does the following decomposition on GM1:
% 
%           GM1 = PM/CDP
%
%  where GM1 is a polynomial transfer function matrix in modes 4 thru 6,
%  PM is a polynomial matrix in modes 1 thru 3 having the same logical
%  dimensions as GM1, and CDP is the POLPAC common denominator polynomial
%  of GM1.
%
%  Input Parameter:
%  ----------------
%
%    gm1    := Transfer function matrix in mode 4, 5, or 6.
%
%  Output Parameters:
%  ------------------
%
%    pm     := Polynomial matrix containing the numerator polynomials
%              stored in the same mode as CDP
%    cdp    := common denominator polynomial of GM1 stored in the
%              input mode minus 3 (mode 1, 2, or 3).
%
push('COMMONDEN')
global debg
%
% UNPACK data structure.
%
[mode,logcol,dtype,stype] = unpack(gm1(1,1));
[lrow,lcol] = logdim(gm1);
dom = domain(gm1);
%
%  Check for consistency.
%
if debg == 3,
  if mode < 4,
    disp('COMMONDEN: Error - Operates only on modes 4 thru 6'),
    mode,
    abort,
  end,
  if mode > 6,
    disp('COMMONDEN: Error - Operates only on modes 4 thru 6'),
    mode,
    abort,
  end,
end
%
% If this is a single transfer function, we don't need to do anything
%
if lrow*lcol == 1,
  [pm,cdp] = tfm2plm(gm1);
  pull,
  return,
end
%
%  Now find the common denominator as the smallest common multiple of
%  all denominators.
%
[pm1,qm1] = tfm2plm(gm1);
cdp = onesp(1,1,mode-3,logcol,stype,dom);
for i=1:lrow,
  for k=1:lcol,
    qm1ik = gget(qm1,i,k);
    cdp = scm1(cdp,qm1ik);
  end,
end
%
%  Now equalize the numerators
%
pm = pm1;
for i=1:lrow,
  for k=1:lcol,
    pm1ik = gget(pm1,i,k);
    qm1ik = gget(qm1,i,k);
    gik = plm2tfm(cdp,qm1ik);
    [gik,fac] = strictprop(gik);
    pm1ik = multp1(fac,pm1ik);
    pm = put(pm,pm1ik,i,k);
  end,
end
%
pull
return
