function [dtype] = datatype(pm)
%  DATATYPE:  POLPAC function to determine data type (integer/real/complex)
%
%  Usage:  dtype = datatype(pm)
%
%  This function examines the elements of a POLPAC data structure
%  stored in any of the 10 storage modes (or an arbitrary Matlab
%  matrix, and returns a value of DTYPE which indicates whether
%  the matrix is classified as integer, real, or complex.
%
%  Input Parameters:
%  -----------------
%
%    pm     ::= matrix in any storage mode
%
%  Output Parameters:
%  ------------------
%
%    dtype  ::= type of data elements coded as follows:
%                 dtype = 1 : all elements are integer
%                 dtype = 2 : some elements are real; none are complex
%                 dtype = 3 : some elements are complex
%
push('DATATYPE')
%
%  Initially establish DTYPE as complex; alter DTYPE by comparing
%  magnitude of imaginary part to 10*EPS
%
dtype = 3;
%
%  Identify the largest imaginary coefficient.  Note that MAX(pm)
%  returns a row vector containing largest column values.
%  MAX(MAX(pm)) then selects the largest value from the
%  returned row vector: i.e., the matrix element of largest size
%
%  If no significant imaginary coefficients are found, alter
%  DTYPE from complex to real
%
if max(max(abs(imag(pm)))) > 10*eps, pull, return, end
%
%  Now establish DTYPE as real.
%
dtype = 2;
%
%  Compare the data structure with a ROUNDed version of itself.
%  If these are the same, DTYPE is integer.
%
if max(max(abs(pm-round(pm)))) > 10*eps, pull, return, end
%
%  Thus, DTYPE is integer
%
dtype = 1;
%
pull
return
