function [g3] = divg1(g1,g2)
%  DIVG1:  Divide two POLPAC transfer functions
%
%  Usage:  g3 = divg1(g1,g2)
%
%  This function operates on single rational functions stored in
%  modes 4 thru 6.  G3 is the division of G1 into G2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    g1     := Single rational function in modes 4 thru 6
%    g2     := Single rational function in modes 4 thru 6
%
%  Output Parameter:
%  -----------------
%
%    g3     := Single rational function
%
push('DIVG1')
global debg
%
%  Check for necessary conversions first
%
[gg1,gg2] = mixmod(g1,g2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(gg1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(gg2(1,1));
%
[n1,m1] = logdim(gg1);
[n2,m2] = logdim(gg2);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 4,
    disp('DIVG1: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if mod1 > 6,
    disp('DIVG1: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('DIVG1: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if n1*m1 ~= 1,
    disp('DIVG1: Error - Operates on scalar rational functions only'),
    abort,
  end,
  if n2*m2 ~= 1,
    disp('DIVG1: Error - Operates on scalar rational functions only'),
    abort,
  end,
end
%
%  Perform the actual division now.
%
[p1,q1] = tfm2plm(gg1);
[p2,q2] = tfm2plm(gg2);
pp1 = multp1(p1,q2);
pp2 = multp1(p2,q1);
g3 = plm2tfm(pp1,pp2);
%
%  Do a final SQUEEZE.
%
g3 = squeeze(g3);
%
pull
return
