function [s2] = eitelberg(s1,nred)
%
%  This function performs a model reduction on a stable either continuous
%  or discrete system using the algorithm of Eitelberg.
%  
%  References:  Eduard Eitelberg: Ph.D. Dissertation, University of Karlsruhe,
%               1979
%        also:  Eduard Eitelberg: Modellreduktion durch Minimierung des
%               Gleichungsfehlers, Regelungstechnik, 10, 1978.
%        also:  Essam Badreddin: A Time-Scale Method for Model Reduction of
%               Discrete-Time Systems, Ph.D. Dissertation, ETH Zurich, 1982.
%
%  Input Parameters:
%  ----------------
%
%    s1     := POLPAC stable system representation in time domain (mode 0).
%    nred   := Order of the reduced system.
%
%  Output Parameter:
%  ----------------
%
%    s2   := Reduced POLPAC representation in the time domain (mode 0).
%
push('EITELBERG')
global debg
%
%  Start by unpacking the structural information of the input matrix s1.
%
[mode,logcol,dtype,stype] = unpack(s1(1,1));
%
%  Extract the A, B, C and D matrices from S1.
%
[A,B,C,D] = oldsyst(s1);
[n,n] = size(A);
[p,m] = size(D);
%
%  Check for consistency.
%
if debg == 3,
  if mode ~= 0,
    disp('EITELBERG: Error - The algorithm works only in MODE = 0')
    abort,
    return,
  end,
  if n < nred,
    disp('EITELBERG: Warning - Nothing to be done (nred >= n)')
    s2 = s1;
    return,
  end,
end
%
%  We first need to check whether the system is stable.
%
if debg >= 2,
  if max(real(eig(A))) >= 0,
    disp('EITELBERG: Error - Balancing works only on stable systems.')
    abort,
    return,
  end,
end
%
%  Perform the model reduction now.  
%  We start by balancing the states.
%
[s2,Gco,t] = balance(s1);
%
%  Unpack the balanced system.
%
[Abal,Bbal,Cbal,D] = oldsyst(s2);
%
%  Determine the number of states that can be thrown away.
%
nred1 = n;
while Gco(nred1) < 1e-6*Gco(1),
  nred1 = nred1 - 1;
end
%
%  Don't reduce below the desired number of states.
%
nred1 = max([nred1,nred]);
%
%  Now reduce the system by throwing the nearly uncontrollable and
%  nearly unobservable states away.
%
Ared1 = Abal(1:nred1,1:nred1)
Bred1 = Bbal(1:nred1,:)
Cred1 = Cbal(:,1:nred1)
%
%  Check whether we are done.
%
if nred1 == nred,
  s2 = newsyst(Ared1,Bred1,Cred1,D,stype);
  pull,
  return,
end
%
%  Now reduce the system further by means of the Eitelberg approach.
%  The discrete version is due to Badreddin.
%
R = [ eye(nred) , zeros(nred,nred1-nred) ];
Cred = Cred1/R;
if stype == 0,
  Q = Ared1\Bred1;
  [U,S,V] = svd(Q);
  Ah = U'*Ared1*U;
  S2 = S*S';
  Sh = lyap(Ah,S2);
  S = U*Sh*U';
  Ared = R*Ared1*S*R'/(R*S*R');
  Bred = Ared*R*Q;
  else
  [U,S,V] = svd(Bred1);
  Ah = U'*Ared1*U;
  S2 = S*S';
  Sh = dlyap(Ah,S2);
  S = U*Sh*U';
  Ired1 = eye(nred1);
  Ired = eye(nred);
  Q = R/(Ired1 - Ared1);
  Ared = Ired - R*S*Q'/(Q*S*Q');
  Bred = (Ired - Ared)*R/(Ired1 - Ared1)*Bred1;
end
s2 = newsyst(Ared,Bred,Cred,D,stype);
%
pull
return
