function [pm2,e1] = elopc1(pm1,i,k)
%
% Input Parameters:
% ----------------
%
%   pm1   := the matrix in modes 1,2 or 3 to be operated upon
%   i     := the logical column to be exchanged with logical 
%            column k
%   k     := the logical column to be exchanged with logical
%            column i
%
% Output parameters:
% -----------------
%
%   pm2   := the output matrix of same mode as pm1 with logical
%            columns i and k exchanged
%   e1    := the elementary column transformation matrix that when
%            multiplied from the left with pm1 will give pm2
%
% This function will accept a polynomial matrix in modes one, two
% or three and exchange logical columns specified by input parameters
% i and k. This function will also generate an elementary column
% transformation matrix that when multiplied from the left with
% pm1 will give pm2.
%
push('ELOPC1')
global debg
%
% Start by unpacking the mode and logcol of pm1
%
[mode,l,dtype,stype] = unpack(pm1(1,1));
%
% Set domain for function NEWMAT
%
dom = 0;
%
% Get physical size of pm1
%
[a,b] = size(pm1);
[n,m] = logdim(pm1);
%
% Check for correct input parameters and matrix definition 
%
if debg == 3,
  %
  % Check whether the mode value is acceptable (must be either 1,2 or 3)
  %
  if mode < 1,
    disp('ELOPC1: Error - Well defined in modes 1,2,3 only'),
    abort,
  end,
  if mode > 3,
    disp('ELOPC1: Error - Well defined in modes 1,2,3 only'),
    abort,
  end,
  %
  % Check that columns i and k exist
  %
  if i < 1,
    disp('ELOPC1: Error - Incorrect column definition'),
    abort,
  end,
  if i > m,
    disp('ELOPC1: Error - Incorrect column definition'),
    abort,
  end,
  if k < 1,
    disp('ELOPC1: Error - Incorrect column definition'),
    abort,
  end,
  if k > m,
    disp('ELOPC1: Error - Incorrect column definition'),
    abort,
  end,
  %
  % Check that b is an integer multiple of l (logcol)
  % 
  nc = round(b/l);
  if abs(b/l-nc) > 10*eps,
    disp('ELOPC1: Error - Physical/logcol mismatch'),
    abort,
  end,
end
%
% Construct pm2
%
pm2 = pm1;
%
if mode ~= 3,
  pm2(2:a,(i-1)*logcol+1:i*logcol) = pm1(2:a,(k-1)*logcol+1:k*logcol);
  pm2(2:a,(k-1)*logcol+1:k*logcol) = pm1(2:a,(i-1)*logcol+1:i*logcol);
  else
  pm2(3:a,(i-1)*logcol+1:i*logcol) = pm1(3:a,(k-1)*logcol+1:k*logcol);
  pm2(3:a,(k-1)*logcol+1:k*logcol) = pm1(3:a,(i-1)*logcol+1:i*logcol);
end
%
% Construct POLPAC transformation matrix
%
e1 = eye(m);
e1(:,[i,k]) = e1(:,[k,i]);
e1 = newmat(e1,mode,stype,dom);
%
pull
return
