function [pm2,e2] = elopc2(pm1,i,c)
%
% Input Parameters:
% ----------------
%
%   pm1  := the matrix in modes 1,2 or 3 to be operated upon
%   i    := the logical column to be multiplied by constant c
%   c    := the constant to be multiplied
%
% Output Parameters:
% -----------------
%
%   pm2  := the output matrix of the same mode as pm1 with the 
%           logical column i multiplied by c
%   e2   := the elementary column transformation matrix that when
%           multiplied from the left with pm1 gives pm2
%
% This function will accept a polynomial matrix in modes 1,2 or 3
% and multiply the logical column i by the constant c. This function 
% will also generate an elementary column transformation matrix, e2,
% that when multiplied from the left with pm1 will give pm2.
% 
push('ELOPC2')
global debg
%
% Start by unpacking the mode and logcol of pm1
%
[mode,logcol,dtype,stype] = unpack(pm1(1,1));
%
% Initialize domain for function NEWMAT
%
dom = 0;
%
% Get physical size of pm1
%
[a,b] = size(pm1);
[n,m] = logdim(pm1);
%
% Check that input parameters and matrix definitions are valid 
%
if debg == 3,
  %
  % Check whether the mode value is acceptable
  %
  if mode < 1,
    disp('ELOPC2: Error - Well defined for modes 1,2,3 only'),
    abort,
  end,
  if mode > 3,
    disp('ELOPC2: Error - Well defined for modes 1,2,3 only'),
    abort,
  end,
  % 
  % Check that column i exists
  %
  if i < 1,
    disp('ELOPC2: Error - Incorrect column definition'),
    abort,
  end,
  if i > m,
    disp('ELOPC2: Error - Incorrect column definition'),
    abort,
  end,
  %
  % Check whether b is an integer multiple of logcol
  %
  nc = round(b/logcol);
  if abs(b/logcol-nc) > 10*eps,
    disp('ELOPC2: Error - physical/logcol mismatch'),
    abort,
  end,
end
%
pm2 = pm1;
%
if mode == 1,
  pm2(2:a,(i-1)*logcol+1:i*logcol) = c*pm2(2:a,(i-1)*logcol+1:i*logcol);
end
%
if mode == 2,
  pm2(2:2:a,(i-1)*logcol+1) = c*pm2(2:2:a,(i-1)*logcol+1);
end
%
if mode == 3,
  pm2(3:a,(i-1)*logcol+1:i*logcol) = c*pm2(3:a,(i-1)*logcol+1:i*logcol);
end
%
% Construct e2 matrix
%
e2 = eye(m);
e2(i,i) = c;
e2 = newmat(e2,mode,stype,dom);
%
pull
return  
