function [pm2,e3] = elopc3(pm1,p,i,k)
%
% Input Parameters:
% ----------------
%
%   pm1   := the matrix to be operated upon
%   p     := the polynomial to be multiplied with column k
%   k     := the column that will be multiplied by p
%   i     := the column the product of k and p will be added to
%
% Output Parameters:
% -----------------
%
%   pm2   := the matrix p1 after column k has been multiplied by polynomial
%            p and the result added to column i 
%   e3    := the transformation matrix that when used in conjunction with
%            the MULT function will yield pm2
%
% This function will multiply column k of the matrix pm1 with the polynomial
% p. After the multiplication has been done the result is added to row i.
%
push('ELOPC3')
global debg
% 
% Start by unpacking the mode and logcol of pm1
%
[mode,logcol,dtype,stype] = unpack(pm1(1,1));
%
% Check for correct input parameters and matrix definition
%
% Get dimensions of pm1
%
[lrow,lcol] = logdim(pm1);
[m,n] = size(pm1);
%
if debg == 3,
  if mode < 1,
    disp('ELOPC3: Error - Well defined in modes 1,2,3 only'),
    abort,
  end,
  if mode > 3,
    disp('ELOPC3: Error - Well defined in modes 1,2,3 only'),
    abort,
  end,
  %
  % Check that rows i and k exist
  %
  if i < 1,
    disp('ELOPC3: Error -Incorrect column definition'),
    abort,
  end,
  if i > lcol,
    disp('ELOPC3: Error - Incorrect column definition'),
    abort,
  end,
  if k < 1,
    disp('ELOPC3: Error - Incorrect column definition'),
    abort,
  end,
  if k > lcol,
    disp('ELOPC3: Error - Incorrect column definition'),
    abort,
  end,
  %
  % Check whether lcol is an integer multiple of logcol
  % 
  nc = round(n/logcol);
  if abs(n/logcol-nc) > 10*eps,  
    disp('ELOPC3: Error - physical/logical mismatch'),
    abort,
  end,
end
%
%Get column k
%
pk = gget(pm1,1:lrow,k);
%
% Put p in POLPAC form
%
[a,b] = size(p);
pa = p(1,b:-1:1);
p1 = zeros(1,b);
p1(1,1) = ppack(1,b,stype,dtype);
p1 = [p1;pa];
%
% Multiply pk with POLPAC p
%
pk = multp(pk,p1);
%
% Extract from pm1 row i
%
ps = gget(pm1,1:lrow,i);
%
% Add pk and ps
%
ps = add(pk,ps);
%
% Put the result of the addition in pm1 at row i
%
pm2 = put(pm1,ps,1:lrow,i);
%
% Generate the e3 matrix
%
e3 = eye(lcol);
%
dom = 0;
%
if mode == 3,
  dom = domain(pm2);
end 
e3 = newmat(e3,mode,stype,dom);
e3 = put(e3,p,k,i);
%
pull
return
