function [pm2,e1] = elopr1(pm1,i,k)
%
% Input Parameters:
% -----------------
%
%   pm1   := the input matrix to be operated upon
%   i     := the logical row to be exchanged with logical row k
%   k     := the logcial row to be exchanged with logical row i
%
% Output Parameters:
% -----------------
%
%   pm2   := the output matrix of same mode as pm1 with logical
%            rows i and k exchanged
%   e1    := the elementary row transformation matrix that when 
%            multiplied from the right with pm1 will give pm2
%
% This function will accept a polynomial matrix in modes one, two or three
% and exchange rows specified by the input parameters i and k. This function
% will also generate an elementary row transformation matrix, e1, that when 
% multiplied from the right with pm1 will give the desired pm2.
% 
push('ELOPR1')
global debg
%
% Start by unpacking the mode and logcol of pm1
%
[mode,logcol,dtype,stype] = unpack(pm1(1,1));
%
% Check for correct input parameters and matrix definition
%
% Get physical size of pm1
%
[a,b] = size(pm1);
[n,m] = logdim(pm1);
%
% Set domain for function NEWMAT
%
dom = 0;
%
%
% Check whether the mode value is acceptable (must be either 1,2 or 3)
%
if debg == 3,
  if mode < 1,
    disp('ELOPR1: Error - Well defined in modes 1,2,3 only'),
    abort,
  end,
  if mode > 3,
    disp('ELOPR1: Error - Well defined in modes 1,2,3 only'),
    abort,
  end,
  %
  % Check that rows i and k exist
  %
  if i < 1,
    disp('ELOPR1: Error - Incorrect row definition'),
    abort,
  end,
  if i > n,
    disp('ELOPR1: Error - Incorrect row definition'),
    abort,
  end,
  if k < 1,
    disp('ELOPR1: Error - Incorrect row definition'),
    abort,
  end,
  if k > n,
    disp('ELOPR1: Error - Incorrect row definition'),
    abort,
  end,
  %
  % Check whether b is an integer multiple of logcol
  %
  nc = round(b/logcol);
  if abs(b/logcol-nc) > 10*eps,
    disp('ELOPR1: Error - physical/logical mismatch'),
    abort,
  end,
end
%
% Construct pm2
%
pm2 = pm1;
%
if mode == 1,
  pm2(i+1,:) = pm1(k+1,:);
  pm2(k+1,:) = pm1(i+1,:);
  else
  if mode == 2,
    pm2(2*i:2*i+1,:) = pm1(2*k:2*k+1,:);
    pm2(2*k:2*k+1,:) = pm1(2*i:2*i+1,:);
    else
    pm2(i+2,:) = pm1(k+2,:);
    pm2(k+2,:) = pm1(i+2,:);
  end,
end
%
% Construct POLPAC transformation matrix
%
e1 = eye(n);
e1([i,k],:) = e1([k,i],:);
e1 = newmat(e1,mode,stype,dom);
%
pull
return
