function [pm2,e2] = elopr2(pm1,i,c)
%
% Input Parameters:
% -----------------
%
%   pm1   := the matrix in modes 1,2 or 3 to be operated upon 
%   i     := the logical column to be multiplied by the constant c
%   c     := the constant to be multplied
%
% Output Parameters:
% ------------------
%
%   pm2   := the output matrix of same mode as pm1 with the logical
%            column i multiplied by c
%   e2    := the elementary row transformation matrix that when multiplied
%            from the right with pm1 will give pm2
%
% This function will accept a polynomial matrix in modes 1,2 or 3 and
% multiply the logical column of pm1 indicated by i by the constant c.
% This function will also generate an elementary row transformation matrix,
% e2, that when multiplied from the right with pm1 will give pm2. 
%
push('ELOPR2')
global debg
%
% Start by unpacking the mode and logcol of pm1
%
[mode,logcol,stype,dtype] = unpack(pm1(1,1));
%
% Set domain for function NEWMAT
%
dom = 0;
%
% Get physical size of pm1
%
[a,b] = size(pm1);
[n,m] = logdim(pm1);
%
% Check that input parameters and matrix definitions are correct
%
if debg == 3,
  %
  % Check whether the mode value is acceptable
  %
  if mode < 1,
    disp('ELOPR2: Error - Well defined in modes 1,2,3 only'),
    abort,
  end,
  if mode > 3,
    disp('ELOPR2: Error - Well defined in modes 1,2,3 only'),
    abort, 
  end,
  %
  % Check that the column i exists
  %
  if i < 1,
    disp('ELOPR2: Error - Incorrect row definition'),
    abort,
  end,
  if i > n,
    disp('ELOPR2: Error - Incorrect row definition'),
    abort,
  end,
  if k < 1,
    disp('ELOPR2: Error - Incorrect row definition'),
    abort,
  end,
  if k > n,
    disp('ELOPR2: Error - Incorrect row definition'),
    abort,
  end,
  %
  % Check whether b is an integer multiple of logcol
  %
  nc = round(b/logcol);
  if abs(b/logcol-nc) > 10*eps,
    disp('ELOPR2: Error - physical/logcol mismatch'),
    abort,
  end,
end
%
% Construct pm2
%
pm2 = pm1;
%
if mode == 1,     
  pm2(i+1,:) = c*pm2(i+1,:);
end
%
if mode == 2,
  pm2(2*i,1:logcol:b) = c*pm2(2*i,1:logcol:b);
end
%
if mode == 3,
  pm2(i+2,:) = c*pm2(i+2,:);
end
%
% Construct POLPAC transformation matrix
%
e2 = eye(n);
e2(i,i) = c;
e2 = newmat(e2,mode,stype,dom);
%
pull
return
