function [pm2,e3] = elopr3(pm1,p,i,k)
%
% Input Parameters:
% ----------------
%
%   pm1   := the matrix to be operated upon
%   p     := the polynomial to be multiplied with row k
%   k     := the row p will be multiplied with
%   i     := the row the product of k and p will be added to
%
% Output Parameters:
% -----------------
%
%   pm2   := the matrix p1 after row k has been multiplied by polynomial
%            p and the result added to row i 
%   e3    := the transformation matrix that when used in conjunction with
%            the MULT function will yield pm2
%
% This function will multiply row k of the matrix pm1 with the polynomial
% p. After the multiplication has been done the result is added to row i.
% 
push('ELOPR3')
global debg
%
% Start by unpacking the mode and logcol of pm1
%
[mode,logcol,dtype,stype] = unpack(pm1(1,1));
%
% Get dimensions of pm1
%
[lrow,lcol] = logdim(pm1);
[m,n] = size(pm1);
%
% Check for correct input parameters and matrix definition
%
if debg == 3,
  if mode < 1,
    disp('ELOPR3: Error - Well defined in modes 1,2,3 only'),
    abort,
  end,
  if mode > 3,
    disp('ELOPR3: Error - Well defined in modes 1,2,3 only'),
    abort,
  end,
  %
  % Check that rows i and k exist
  %
  if i < 1,
    disp('ELOPR3: Error -Incorrect row definition'),
    abort,
  end,
  if i > lrow,
    disp('ELOPR3: Error - Incorrect row definition'),
    abort,
  end,
  if k < 1,
    disp('ELOPR3: Error - Incorrect row definition'),
    abort,
  end,
  if k > lrow,
    disp('ELOPR3: Error - Incorrect row definition'),
    abort,
  end,
  %
  % Check whether n is an integer multiple of logcol
  % 
  nc = round(n/logcol);
  if abs(n/logcol-nc) > 10*eps,
    disp('ELOPR3: Error - physical/logical mismatch'),
    abort,
  end,
end
%
% Get row k
%
pk = gget(pm1a,k,1:lcol);
%
% Multiply polynomial p with row i
%
% Put p in POLPAC form
%
[a,b] = size(p);
p1 = zeros(1,b);
p1(1,1) = ppack(1,b,dtype,stype);
p1 = [p1;p(1,b:-1:1)];
%
% Multiply pk with POLPAC p
%
pk = multp(pk,p1);
%
% Extract from pm1 row i
%
ps = gget(pm1,i,1:lcol);
%
% Add pk and ps
%
ps = add(pk,ps);
%
% Put the result of the addition in pm1 at row i
%
pm2 = put(pm1,ps,i,1:lcol);
%
% Generate the e3 matrix
%
dom = 0;
%
if mode == 3,
  dom = domain(pm2);
end
%
e3 = eye(lrow);
e3 = newmat(e3,mode,stype,dom);
e3 = put(e3,p,i,k);
%
pull 
return
