function [p2]  =  equalize(p1)
%
%  This POLPAC function takes a polynomial matrix P1 in mode 2 and
%  concatenates all roots (in P1) with their multiplicities. Then the
%  function uses MROOT to eliminate clusters of roots, and finally the
%  new roots are positioned back inside P1. The global flag EQUALFLAG
%  is used to pick which MROOT routine is to be used (MROOT0, MROOT1
%  or MROOT2).
%
%  Input Parameter:
%  ----------------
%
%    p1     := A polynomial matrix (mode 2 only)
%
%  Output Parameter:
%  -----------------
%
%    p2     := The same polynomial matrix (mode 2) with clusters
%              of nearby roots replaced by one value
%
push('EQUALIZE')
global debg
global equalflag
%
%  Squeeze the polynomial matrix.
%
p2 = squeeze(p1);
%
%  Start by unpacking the structural information.
%
[mode,logcol,dtype,stype] = unpack(p2(1,1));
%
%  Check for consistency. Must be in mode 2, else error.
%
if debg == 3,
  if mode ~= 2,
    disp('EQUALIZE: Error - Operates on mode 2 only'),
    mode,
    abort,
  end,
end
%
%  Get the logical dimensions of P2.
%
[lrow,lcol] = logdim(p2);
%
%  Now extract all roots from P2 with multiplicities > 0.
%  Loop over logical rows then logical columns.
%  However, only attempt this if logcol is > 1, otherwise there
%  are no roots to extract so it doesn't make any sense to try.
%
l = 0;
if logcol > 1,
  for i=1:lrow,
    ii = 2*i;
    for jj=1:lcol,
      jjj = (jj - 1)*logcol;
      for k=2:logcol,
        kk = jjj + k;
        if p2(ii+1,kk) > 0,
          l = l + 1;
          l1(l) = p2(ii,kk);
          m1(l) = p2(ii+1,kk);
          index(l,1) = ii;
          index(l,2) = kk;
        end,
      end,
    end,
  end,
end
l1 = conj(l1');
m1 = m1';
%
%  Now eliminate clusters using MROOT, but only if some roots
%  were found.
%
if l > 0,
  if equalflag == 0,  [l2,m2,indx] = mroot0(l1,m1);  end,
  if equalflag == 1,  [l2,m2,indx] = mroot1(l1,m1);  end,
  if equalflag == 2,  [l2,m2,indx] = mroot2(l1,m1);  end,
  %
  %  Now place these new roots back into P2 where the old roots were.
  %
  for i=1:l,
    p2(index(i,1),index(i,2)) = l2(indx(i));
  end,
  %  The multiplicities have not been messed with yet. Now
  %  GET each polynomial from P2 separately and use MROOT0
  %  on it to change equal roots with separate multiplicities
  %  to one root with a combined multiplicity within each 
  %  polynomial.
  for i=1:lrow,
    for jj=1:lcol,
      p3 = gget(p2,i,jj);
      [mde,lgcol,dtpe,stpe] = unpack(p3(1,1));
      if lgcol > 1,
        l1 = p3(2,2:lgcol);
        m1 = p3(3,2:lgcol);
        [l2,m2,indx] = mroot0(conj(l1'),m1');
        [n,m] = size(l2);
        p3(2,2:n+1) = conj(l2');
        p3(3,2:n+1) = m2';
        if n+1 < lgcol,
          p3(2:3,n+2:lgcol) = zeros(2,lgcol-n-1);
        end,
      end,
      p2 = put(p2,p3,i,jj);
    end,
  end,
end
%
%  SQUEEZE P2 in case the root equalizations have left extra empty columns.
%
p2 = squeeze(p2);
%
pull
return
