function [p2] = expand(p1,lognew)
%
%  Input Parameters:
%  -----------------
%
%    p1     := the input matrix to be expanded
%    lognew := the requested number of physical columns per
%              one logical column (logcol value)
%
%  Output Parameters:
%  ------------------
%
%    p2     := the expanded output matrix
%
%  The EXPAND function performs the opposite of the
%  SQUEEZE function.  EXPAND adds a requested number
%  of zero columns after each logcol block.  
%
push('EXPAND')
global debg
%
%  Start by unpacking the MODE, DTYPE, STYPE, and LOGCOL of the
%  given data structure.
%
[mode,logold,dtype,stype] = unpack(p1(1,1));
%
%  Check whether the mode value is acceptable (must be between 1 and 9)
%
if debg == 3,
  if mode == 0,
    disp('EXPAND: Error - Operates on modes 1,2,4,5,7,8 only'),
    mode,
    abort,
  end,
  m3 = mode/3;
  if abs(m3-round(m3)) < 10*eps,
    disp('EXPAND: Error - Operates on modes 1,2,4,5,7,8 only'),
    mode,
    abort,
  end,
end
%
%  Get the physical size of the input matrix to be expanded
%
[m,n] = size(p1);
%
%  Compute the number of zero columns to be added 
%
ncol = lognew - logold;
%
%  if the new logcol is smaller than the old logcol, don't do
%  anything.
%
if ncol <= 0,
  p2 = p1;
  pull,
  return,
end
%
%  Compute the number of logical columns.  Since EXPAND is a
%  very basic function, avoid calling LOGDIM.
%
nc = round(n/logold);
%
%  if debg=3, check whether n is an integer multiple of logold.
%
if debg == 3,
  if abs(n/logold - nc) > 10*eps,
    disp('EXPAND: Error - physical/logical column mismatch'),
    abort,
  end,
end
%
%  Now, the columns of zero will be added
%
p2 = p1(:,1:logold);
zc = zeros(m,ncol);
p2 = [p2,zc];
%
%  Add ZC to remaining logcol blocks and concatenate
%
for i=1:nc-1,
  bc = p1(:,i*logold+1:(i+1)*logold);
  bc = [bc,zc];
  p2 = [p2,bc];
end
% 
%  Create the final output data structure
%
p2(1,1) = ppack(mode,lognew,dtype,stype);
%
pull
return
