function [pm] = eyep(lrow,lcol,mode,logcol,stype,dom)
%
%  This routine creates a unity POLPAC data structure with LROW
%  logical rows and LCOL logical columns using storage mode MODE
%  assuming initially LOGCOL physical columns per one logical
%  column.  LOGCOL can later be modified by functions SQUEEZE
%  and EXPAND when needed.
%
%  This functions operates on storage modes 1 thru 6 only.
%  In modes 3 and 6, the domain information (DOM) needs to be provided.
%  In all other cases, this parameter can be set equal to zero,
%
%  Input Parameters:
%  -----------------
%
%    lrow   := Number of logical rows of new data structure
%    lcol   := Number of logical columns of new data structure
%    mode   := Storage mode of new data structure (1 thru 6)
%    logcol := Number of physical columns per one logical column
%    stype  := System type (0 or 1)
%    dom    := Vector containing the domain information
%
%  Output Parameters:
%  ------------------
%
%    pm     := Unity data structure created
% 
push('EYEP')
global debg
ddom = dom;
%
%  Check for consistency
%
if debg == 3,
  if mode < 1,
    disp('EYEP: Error - Operates on modes 1 thru 6 only'),
    mode,
    abort,
  end,
  if mode > 6,
    disp('EYEP: Error - Operates on modes 1 thru 6 only'),
    mode,
    abort,
  end,
  if abs(mode - 3*round(mode/3)) < 100*eps,
    [n,m] = size(dom);
    nm = n*m;
    if n ~= 1,
      ddom = conj(dom');
      if m ~= 1,
        disp('EYEP: Error - The domain must be a vector'),
        dom,
        abort,
      end,
    end,
    if nm ~= logcol,
      disp('EYEP: Error - Domain incompatible with LOGCOL'),
      abort,
    end,
  end,
  if lrow ~= lcol,
    disp('EYEP: Error - Operates on square matrices only'),
    abort,
  end,
end
%
% Pack storage information
%
dtype = 1;
nbr = ppack(mode,logcol,dtype,stype);
%
%  Compute the number of physical columns and rows of the new
%  data structure
%
%  Number of physical columns
%
pcol = lcol*logcol;
%
%  Number of physical rows
%
if mode == 1,
  prow = lrow + 1;
  else if mode == 2,
    prow = 2*lrow + 1;
    else if mode == 3,
      prow = lrow + 2;
    end,
  end,
end
if mode == 4,
  prow = 2*lrow + 1;
  else if mode == 5,
    prow = 4*lrow + 1;
    else if mode == 6,
      prow = 2*lrow + 2;
    end,
  end,
end
%
%  Build the unity matrix
%
pm = zeros(prow,pcol);
%
%  Add the first row (NBR)
%
pm(1,1) = nbr;
%
%  if mode == 3 or 6, add the second row (DOM)
%
if mode == 3,
  pm(2,1:logcol) = ddom;
  else if mode == 6,
    pm(2,1:logcol) = ddom;
  end,
end
%
%  Put the unity matrix into the numerator
%
for i=1:lrow,
  if mode == 1,
    pm(i+1,(i-1)*logcol+1) = 1;
    else if mode == 2,
      pm(2*i,(i-1)*logcol+1) = 1;
      else if mode == 3,
        pm(i+2,(i-1)*logcol+1:i*logcol) = ones(1,logcol);
      end,
    end,
  end,
  if mode == 4,
    pm(i+1,(i-1)*logcol+1) = 1;
    else if mode == 5,
      pm(2*i,(i-1)*logcol+1) = 1;
      else if mode == 6,
        pm(i+2,(i-1)*logcol+1:i*logcol) = ones(1,logcol);
      end,
    end,
  end,
end
%
%  if mode == 4, 5, or 6, make denominator 1 rather than 0
%
if mode == 4,
  pm(lrow+2:prow,1:logcol:pcol-logcol+1) = ones(lrow,lcol);
  else if mode == 5,
    pm(2*lrow+2:2:prow-1,1:logcol:pcol-logcol+1) = ones(lrow,lcol);
    else if mode == 6,
      pm(lrow+3:prow,1:pcol) = ones(lrow,pcol);
    end,
  end,
end
%
pull
return 
