function [T,S1,S2,Ge] = feedbk(G,H,sgn)
%
%  This function computes a feedback structure for a POLPAC system.  It
%  operates on modes 0, 4..6, and 7..9.
%
%  Input Parameters:
%  -----------------
%
%    G      :=  POLPAC system in forward path
%    H      :=  POLPAC system in feedback path
%    sgn    :=  The sign of the unit feedback
%               default: sign = -1
%
%  Output Parameters:
%  ------------------
%
%    T      :=  Transmission transfer function ( r -> y )
%    S1     :=  Sensitivity transfer function  ( r -> u )
%    S2     :=  Sensitivity transfer function of equivalent system ( r -> e )
%    Ge     :=  Equivalent open-loop transfer function
%
%    All outputs are returned in the same mode as G and H.
%    MIXMOD is employed to check for compatibility between G and H.
%
%  Explanation:
%  ------------
%
%        r   +    u    -------
%      ------->o------>|  G  |-------+------> y
%              ^       -------       |
%         sign |                     |
%              |       -------       |
%              --------|  H  |<-------
%                      -------
%
%  Equivalent System:
%  ------------------
%
%        r   +    e    -------
%      ------->o------>|  Ge |-------+------> y
%              ^       -------       |
%         sign |                     |
%              -----------------------
%
push('FEEDBK')
global debg
%
%  Check for mixed mode operation
%
[G,H] = mixmod(G,H);
%
%  Start by unpacking the structural information
%
[mode,logcolG,dtypG,stypG] = unpack(G(1,1));
[mode,logcolH,dtypH,stypH] = unpack(H(1,1));
[lrG,lcG] = logdim(G);
[lrH,lcH] = logdim(H);
dom = domain(G);
%
%  Check for missing sign
%
if nargin == 1,
  sgn = -1;
end
%
%  Check for consistency
%
if debg == 3,
  if lrG ~= lcH | lcG ~= lrH,
    disp('FEEDBK: Error - G and H are structurally incompatible'),
    abort,
    return,
  end,
  if abs(sgn) - 1 > 1000*eps,
    disp('FEEDBK: Error - Second argument must be either +1 or -1'),
    abort,
    return,
  end,
  if stypH ~= stypG,
    disp('FEEDBK: Error - Incompatible system types'),
    abort,
  end,
end
%
%  Branch depending on the mode of the system.
%
if mode == 0,
  %
  %  Is a state-space description
  %
  [Ag,Bg,Cg,Dg] = oldsyst(G);
  [Ah,Bh,Ch,Dh] = oldsyst(H);
  DhDg = Dh*Dg;
  DD = eye(size(DhDg)) - sgn*DhDg;
  %
  %  Check whether system is well-posed
  %
  if norm(DD) < 1000*eps,
    disp('FEEDBK: Error - System is ill-posed'),
    abort,
    return,
  end,
  %
  %  Compute closed-loop system
  %
  DDinv = inv(DD);
  Bcl1 = Bg*DDinv;
  Bcl2 = Bh*Dg*DDinv;
  Bcl = [ Bcl1 ; Bcl2 ];
  sBcl1 = sgn*Bcl1;
  sBcl2 = sgn*Bcl2;
  DhCg = Dh*Cg;
  Acl11 = Ag + sBcl1*DhCg;
  Acl12 = sBcl1*Ch;
  Acl21 = Bh*Cg + sBcl2*DhCg;
  Acl22 = Ah + sBcl2*Ch;
  Acl = [ Acl11 , Acl12 ; Acl21 , Acl22 ];
  D1cl = Dg*DDinv;
  sD1cl = sgn*D1cl;
  C1cl = [ (Cg + sD1cl*DhCg) , (sD1Cl*Ch) ];
  D2cl = DDinv;
  sD2cl = sgn*D2cl;
  C2cl = [ (sD2cl*DhCg) , (sD2cl*Ch) ];
  T = newsyst(Acl,Bcl,C1cl,D1cl,stypG);
  S1 = newsyst(Acl,Bcl,C2cl,D2cl,stypG);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = minreals(T);
  S1 = minreals(S1);
  %
  %  Now check, whether we are already done
  %
  if nargout > 2,
    %
    %  No such luck!
    %  We need to check for compatibility
    %
    [lrow,lcol] = logdim(T);
    if lrow ~= lcol,
      disp('FEEDBK: Error - Number of inputs and outputs must be same'),
      abort,
      return,
    end,
    C3cl = sgn*C1cl;
    D3cl = sgn*D1cl + eye(size(D1cl));
    S2 = newsyst(Acl,Bcl,C3cl,D3cl,stypG);
    Ge = unitfb(T,-sgn);
    %
    %  Get rid of uncontrollable/unobservable modes
    %
    S2 = minreals(S2);
    Ge = minreals(Ge);
  end,
  pull,
  return,
end
%
if mode == 4 | mode == 5 | mode == 6,
  %
  %  Is a transfer function matrix
  %
  ssgn = newmat(sgn,mode-3,stypG,dom);
  sHG = multg(ssgn,multg(H,G));
  [mode,logcol,dtype,stype] = unpack(sHG(1,1));
  [lrow,lcol] = logdim(sHG);
  I = eyep(lrow,lcol,mode,logcol,stype,dom);
  IHG = sub(I,sHG);
  %
  %  Check whether system is well-posed
  %
  [G2,P2] = strictprop(IHG);
  ord = order(P2);
  if ord == 0,
    gn = gain(P2);
    if gn == 0,
      disp('FEEDBK: Error - System is ill-posed'),
      abort,
      return,
    end,
  end,
  %
  %  Compute closed-loop system
  %
  S1 = invg(IHG);
  T = multg(G,S1);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  S1 = reduceg(S1);
  T = reduceg(T);
  %
  %  Now check, whether we are already done
  %
  if nargout > 2,
    %
    %  No such luck!
    %  We need to check for compatibility
    %
    [lrow,lcol] = logdim(T);
    if lrow ~= lcol,
      disp('FEEDBK: Error - Number of inputs and outputs must be same'),
      abort,
      return,
    end,
    [mode,logcol,dtype,stype] = unpack(T(1,1));
    I = eyep(lrow,lcol,mode,logcol,stype,dom);
    S2 = addg(I,multg(sgn,T));
    Ge = multg(T,invg(S2));
    %
    %  Get rid of uncontrollable/unobservable modes
    %
    S2 = reduceg(S2);
    Ge = reduceg(Ge);
  end,
  pull,
  return,
end
%
if mode == 7 | mode == 8 | mode == 9,
  %
  %  Is a polynomial matrix system description
  %
  [P,Q,R,W] = pms2plm(G);
  %
  %  Check whether Wh*Wg is a constant matrix.
  %
  WhWg = multp(Wh,Wg);
  ord = order(WhWg);
  if ord > 0,
    %
    %  Convert to transfer function matrix form, and try again.
    %
    G = pms2tfm(G);
    [T,S] = feedbk(G,sgn);
    T = tfm2pms(T);
    S = tfm2pms(S);
    else
    %
    %  Okay!  We can proceed directly.
    %
    [mode,logcol,dtype,stype] = unpack(WhWg(1,1));
    [lrow,lcol] = logdim(WhWg);
    I = eyep(lrow,lcol,mode,logcol,stype,dom);
    ssgn = newmat(sgn,mode,stype,dom);
    IWhWg = sub(I,multp(ssgn,WhWg));
    %
    %  Check whether system is well-posed
    %
    P2 = rdetp(IWhWg);
    ord = order(P2);
    if ord == 0,
      gn = gain(P2);
      if gn == 0,
        disp('FEEDBK: Error - System is ill-posed'),
        abort,
        return,
      end,
    end,
    %
    %  Compute closed-loop system
    %
    IWinv = invp(IWhWg);
    Qcl1 = multp(Qg,IWinv);
    Qcl2 = multp(Qh,multp(Wg,IWinv));
    Qcl = vconc(Qcl1,Qcl2);
    sQcl1 = minus(multp(ssgn,Qcl1));
    sQcl2 = minus(multp(ssgn,Qcl2));
    WhRg = multp(Wh,Rg);
    Pcl11 = addp(Pg,multp(sQcl1,WhRg));
    Pcl12 = multp(sQcl1,Rh);
    Pcl21 = sub(multp(sQcl2,WhRg),multp(Qh,Rg));
    Pcl22 = addp(Ph,multp(sQcl2,Rh));
    Pcl = vconc(hconc(Pcl11,Pcl12),hconc(Pcl21,Pcl22));
    W1cl = multp(Wg,IWinv);
    sW1cl = multp(ssgn,W1cl);
    R1cl1 = addp(Rg,multp(sW1cl,WhRg);
    R1cl2 = multp(sW1Cl,Rh);
    R1cl = hconc(R1cl1,R1cl2);
    W2cl = IWinv;
    sW2cl = multp(ssgn,W2cl);
    R2cl1 = multp(sW2cl,WhRg);
    R2cl2 = multp(sW2cl,Rh);
    R2cl = hconc(R2cl1,R2cl2);
    T = plm2pms(Pcl,Qcl,R1cl,W1cl);
    S1 = plm2pms(Pcl,Qcl,R2cl,W2cl);
  end,
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = minrealps(T);
  S1 = minrealps(S1);
  %
  %  Now check, whether we are already done
  %
  if nargout > 2,
    %
    %  No such luck!
    %  We need to check for compatibility
    %
    [lrow,lcol] = logdim(T);
    if lrow ~= lcol,
      disp('FEEDBK: Error - Number of inputs and outputs must be same'),
      abort,
      return,
    end,
    R3cl = multp(ssgn,R1cl);
    [mode,logcol,dtype,stype] = unpack(W1cl(1,1));
    [lrow,lcol] = logdim(W1cl);
    I = eyep(lrow,lcol,mode,logcol,stype,dom);
    W3cl = addp(I,multp(ssgn,W1cl));
    S2 = plm2pms(Pcl,Qcl,R3cl,W3cl);
    Ge = unitfb(T,-sgn);
    %
    %  Get rid of uncontrollable/unobservable modes
    %
    S2 = minrealps(S2);
    Ge = minrealps(Ge);
  end,
  pull,
  return,
end
%
return
