function [gm] = invp(pm)
%
%  This function inverts a polynomial matrix.  If the matrix is unimodular,
%  the result is again a polynomial matrix, otherwise, it is a transfer
%  function matrix.
%
%  Input Parameter:
%  ----------------
%
%    pm     :=  The polynomial matrix to be inverted
%               (modes 1, 2, or 3 only)
%
%  Output Parameter:
%  -----------------
%
%    gm     :=  The inverse polynomial/rational function matrix
%
push('INVP');
global debg
%
%  Start by unpacking the structural information.
%
[mode,logcol,dtype,stype] = unpack(pm(1,1));
%
%  Check for consistency.
%
if debg == 3,
  if mode < 1,
    disp('INVP: Error - Operates on modes 1 thru 3 only'),
    abort,
  end,
  if mode > 3,
    disp('INVP: Error - Operates on modes 1 thru 3 only'),
    abort,
  end,
end
%
%  Get the logical dimensions of PM.
%
[lrow,lcol] = logdim(pm);
if debg == 3,
  if lrow ~= lcol,
    disp('INVP: Error - matrix must be square'),
    abort,
  end,
end
%
%  Get the domain information.
%
dom = domain(pm);
%
%  Get the determinant and the adjugate matrix.
%
pdet = rdetp(pm);
padj = radjp(pm);
%
%  Check whether matrix is unimodular.
%
odet = order(pdet);
if odet == 0,
  %
  %  Input matrix is unimodular.
  %
  pdet = gain(pdet);
  %
  %  Check whether unimodular matrix is singular.
  %
  if pdet == 0,
    disp('INVP: Error - matrix is singular'),
    abort,
  end,
  pdet = 1/pdet;
  pdet = newpoly(pdet,stype);
  if mode == 2,
    pdet = c12(pdet);
    else if mode == 3,
      pdet = c13(pdet,dom);
    end,
  end,
  gm = multp(pdet,padj);
  else
  %
  %  Input matrix is not unimodular.
  %
  qm = multp(pdet,onesp(lrow,lcol,mode,logcol,stype,dom));
  gm = plm2tfm(padj,qm);
  gm = reduceg(gm);
end
%
pull
return
