function [flag] = isproper(pm)
%
%  This function determines whether ot not a polynomial matrix or
%  a transfer function matrix is proper.
%
%  Input Parameter:
%  ----------------
%
%    pm     := Any polynomial matrix or transfer function matrix
%              (modes 1 thru 6)
%
%  Output Parameter:
%  -----------------
%
%    flag   := A scalar flag
%              := 0 (false)     :  The data structure is not proper
%              := 1 (true)      :  The data structure is proper
%              := 2 (very true) :  The rational matrix is strictly proper
%
push('ISPROPER')
global debg
%
%  Start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(pm(1,1));
%
%  Check for consistency
%
if debg == 3,
  if mode == 0,
    disp('ISPROPER: Error - Operates only on modes 1 thru 6'),
    abort,
  end,
  if mode > 6,
    disp('ISPROPER: Error - Operates only on modes 1 thru 6'),
    abort,
  end,
end
%
%  Check whether this is a polynomial matrix (modes 1 thru 3) or a
%  transfer function matrix (modes 4 thru 6)
%
m3 = round((mode + 1)/3);
%
%  For polynomial matrices, M3=1
%
if m3 == 1,
  rflag = propcol(pm);
  cflag = proprow(pm);
  flag  = rflag*cflag;
  pull,
  return,
end
%
%  For transfer function matrices, M3=2
%
[p,q] = tfm2plm(pm);
if mode == 6,
  p = c32(p,0);
  q = c32(q,0);
end
%
%  Get the orders of the numerator polynomials, but catch the
%  special case where the polynomial is solidly zero ==> we'll
%  then make the order -1.
%
ordnum = order(p);
[n,m] = size(ordnum);
[mode,logcol,dtype,stype] = unpack(p(1,1));
for i=1:n,
  for k=1:m,
    if ordnum(i,k) == 0,
      if mode == 1,
        pp = p(i+1,(k-1)*logcol+1);
        else
        pp = p(2*i,(k-1)*logcol+1);
      end,
      if abs(pp) < 100*eps,
        ordnum(i,k) = -1;
      end,
    end,
  end,
end
%
%  Get now the orders of the denominator polynomial
%
ordden = order(q);
%
%  Compute the difference and get the maximum.  For strictly proper,
%  this maximum must be negative, for proper but not strictly proper,
%  it must be zero.
%
orddiff = ordnum - ordden;
maxdiff = max(max(orddiff));
if maxdiff > 100*eps,
  flag = 0;
  else if abs(maxdiff) < 100*eps,
    flag = 1;
    else
    flag = 2;
  end,
end
%
pull
return
