function [pr1,pr2,fac] = lcd1(p1,p2)
%
%  This function operates on single polynomials stored in modes 1 thru 3.
%  P3 is the largest common divider of P1 and P2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    p1     :=  Single polynomial in modes 1 thru 3
%    p2     :=  Single polynomial in modes 1 thru 3
%
%  Output Parameter:
%  -----------------
%
%    pr1    :=  Reduced p1 polynomial
%    pr2    :=  Reduced p2 polynomial
%    fac    :=  Common factor of p1 and p2
%               p1 = fac*pr1
%               p2 = fac*pr2
%
push('LCD1')
global debg
%
%  Check for necessary conversions first
%
[p1,p2] = mixmod(p1,p2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(p1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(p2(1,1));
[n1,m1] = logdim(p1);
[n2,m2] = logdim(p2);
dom = domain(p1);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 1,
    disp('LCD1: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if mod1 > 3,
    disp('LCD1: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('LCD1: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if n1*m1 ~= 1,
    disp('LCD1: Error - Operates on scalar polynomials only'),
    abort,
  end,
  if n2*m2 ~= 1,
    disp('LCD1: Error - Operates on scalar polynomials only'),
    abort,
  end,
end
%
o1 = order(p1);
o2 = order(p2);
if o2 > o1,
  g = plm2tfm(p1,p2);
  else
  g = plm2tfm(p2,p1);
end
gr = reduceg(g);
[mode,logcol,dtype,stype] = unpack(gr(1,1));
if mod1 == 3,
  fac = onesp(1,1,mod1,lcol1,styp1,dom);
  else
  fac = onesp(1,1,mod1,1,styp1,dom);
end
if o2 > o1,
  if mode > 3,
    [pr1,pr2] = tfm2plm(gr);
    else
    pr1 = gr;
    pr2 = fac;
  end,
  else
  if mode > 3,
    [pr2,pr1] = tfm2plm(gr);
    else
    pr2 = gr;
    pr1 = fac;
  end,
end
or1 = order(pr1);
if or1 < o1,
  %
  %  We got a common factor.
  %
  g = plm2tfm(p1,pr1);
  [g,fac] = strictprop(g);
end
%
pull
return
