function [TL,Q] = lcld(Pm,Rm)
%
%   This function lcld determine the largest common left divider
%   of the polynomial matrices Pm and Rm.
%
%   Input Parameters:
%   -----------------
% 
%   Pm,Rm :=  Two polynomial matrices (modes 1, 2, or 3 only)
%             from which the largest common left divider
%             is to be determined.
% 
%   Output Parameters:
%   ------------------
% 
%   TL   :=   The largest common left divider of Pm and Rm,
%             a polynomial matrix in lower triangular form.       
%   Q    :=   A unimodular polynomial matrix satifying
%             [Pm, Rm] = [TL,0]*Q.
%
%   Concatenates the polynomial matrix Rm horizontally from the right
%   to the polynomial matrix Pm.  Concatenation here is meant in a 
%   logical rather than physical sense.
%
push('LCLD')
global debg
%
%  Check for mixed mode operations
%
[Pm,Rm] = mixmod(Pm,Rm);
%
%  Get the structural information
%
[modP,lcolP,dtypP,stypP] = unpack(Pm(1,1));
[modR,lcolR,dtypR,stypR] = unpack(Rm(1,1));
[nP,mP] = logdim(Pm);
[nR,mR] = logdim(Rm);
%
%  Consistency checks
%
if debg == 3,
  if modP < 1,
    disp('LCLD: Error - Operates on modes 1 to 3 only'),
    abort,
    return,
  end,
  if modP > 3,
    disp('LCLD: Error - Operates on modes 1 to 3 only'),
    abort,
    return,
  end,
end
if debg > 1,
  if nP ~= nR,
    disp('LCLD: Error - P and R have incompatible dimensions'),
    abort,
    return,
  end,
end
%
%  Okay.  Concatenate the matrices now.
%
Cm = hconc(Pm,Rm);
Cmt = transpose(Cm);
[mode,logcol,dtype,stype] = unpack(Cmt(1,1));
dom = domain(Cmt);
%
%   Perform an upper triangularization of the polynomial matrix Cmt.
%
[Qt,Rt] = qrp(Cmt);
Q = transpose(Qt);
R = transpose(Rt);
%
%   The largest common left divider is extracted from the 
%   lower triangular polynomial matrix R
%
[n,m] = logdim(R);
m = min([n,m]);
TL = gget(R,1:m,1:m);
%
pull
return
