function [Tu,Q] = lcrd(Pm,Rm)
%
%   This function lcrd determine the largest common right divider
%   of the polynomial matrices Pm and Rm.
%
%   Input Parameters:
%   -----------------
% 
%   Pm,Rm :=  Two polynomial matrices (modes 1, 2, or 3 only)
%             from which the largest common right divider
%             is to be determined.
% 
%   Output Parameters:
%   ------------------
% 
%   Tu   :=   The largest common right divider of Pm and Rm,
%             a polynomial matrix in the uppertriangular form.       
%   Q    :=   A unimodular polynomial matrix satifying
%             [Pm; Rm] = Q*[Tu;0]
%
%   Concatenates the polynomial matrix Rm vertically from below to 
%   the polynomial matrix Pm.   Concatenation here is meant in a 
%   logical rather than physical sense.
%
push('LCRD')
global debg
%
%  Check for mixed mode operations
%
[Pm,Rm] = mixmod(Pm,Rm);
%
%  Get the structural information
%
[modP,lcolP,dtypP,stypP] = unpack(Pm(1,1));
[modR,lcolR,dtypR,stypR] = unpack(Rm(1,1));
[nP,mP] = logdim(Pm);
[nR,mR] = logdim(Rm);
%
%  Consistency checks
%
if debg == 3,
  if modP < 1,
    disp('LCRD: Error - Operates on modes 1 to 3 only'),
    abort,
    return,
  end,
  if modP > 3,
    disp('LCRD: Error - Operates on modes 1 to 3 only'),
    abort,
    return,
  end,
end
if debg > 1,
  if mP ~= mR,
    disp('LCRD: Error - P and R have incompatible dimensions'),
    abort,
    return,
  end,
end
%
%  Okay.  Concatenate the matrices now.
%
Cm = vconc(Pm,Rm);
[mode,logcol,dtype,stype] = unpack(Cm(1,1));
dom = domain(Cm);
%
%   Perform an upper triangularization of the polynomial matrix Cm.
%
[Q,R] = qrp(Cm);
%
%   The largest common right divider is extracted from the 
%   upper triangular polynomial matrix R
%
[n,m] = logdim(R);
m = min([n,m]);
Tu = gget(R,1:m,1:m);
%
pull
return
