function [g3] = ldivg(g1,g2)
%
%  This function operates on rational function matrices stored in
%  modes 4 thru 6.  G3 is the left division of G2 into G1.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    g1     := Rational function matrix in modes 4 thru 6
%    g2     := Rational function matrix in modes 4 thru 6
%
%  Output Parameter:
%  -----------------
%
%    g3     := Rational function matrix
%
push('LDIVG')
global debg
%
%  Check for necessary conversions first
%
[gg1,gg2] = mixmod(g1,g2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(gg1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(gg2(1,1));
%
[n1,m1] = logdim(gg1);
[n2,m2] = logdim(gg2);
nm1 = n1*m1;
nm2 = n2*m2;
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 4,
    disp('LDIVG: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if mod1 > 6,
    disp('LDIVG: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('LDIVG: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if nm1 == 1,
    disp('LDIVG: Error - Left division is only defined for matrices')
    abort,
  end,
  if nm1 == 2,
    disp('LDIVG: Error - Left division is only defined for matrices')
    abort,
  end,
  if m2 ~= n2,
    disp('LDIVG: Error - G2 must be square'),
    abort,
  end,
  if m1 ~= n2,
    disp('LDIVG: Error - Incompatible number of rows/columns'),
    abort,
  end,
end
%
%  Create a shell to store the result.
%
logcol = max([lcol1,lcol2]);
dom1 = domain(gg1);
g3 = zerop(n1,m1,mod1,logcol,styp1,dom1);
%
%  Perform the actual division now.
%
%  The operation must be of type: matrix divided by matrix
%
g3 = mult(invg(gg1),gg2);
g3 = squeeze(g3);
%
pull
return
