function [g3] = ldivp(p1,p2)
%
%  This function operates on polynomial matrices stored in modes 1 thru 3.
%  G3 is the left division of P2 into P1.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    p1     := Polynomial matrix in modes 1 thru 3
%    p2     := Polynomial matrix in modes 1 thru 3
%
%  Output Parameter:
%  -----------------
%
%    g3     := Rational function matrix in modes 4 thru 6
%
push('LDIVP')
global debg
%
%  Check for necessary conversions first
%
[pp1,pp2] = mixmod(p1,p2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(pp1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(pp2(1,1));
%
[n1,m1] = logdim(pp1);
[n2,m2] = logdim(pp2);
nm1 = n1*m1;
nm2 = n2*m2;
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 1,
    disp('LDIVP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if mod1 > 3,
    disp('LDIVP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('LDIVP: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if nm1 == 1,
    disp('LDIVP: Error - Left division is only defined for matrices')
    abort,
  end,
  if nm1 == 2,
    disp('LDIVP: Error - Left division is only defined for matrices')
    abort,
  end,
  if m1 ~= n1,
    disp('LDIVP: Error - P1 must be square'),
    abort,
  end,
  if m1 ~= n2,
    disp('LDIVP: Error - Incompatible number of rows/columns'),
    abort,
  end,
end
%
%  Create a shell to store the result.
%
logcol = max([lcol1,lcol2]);
dom1 = domain(pp1);
g3 = zerop(n2,m2,mod1+3,logcol,styp1,dom1);
%
%  Perform the actual division now.
%
%  The operation must be of type: matrix divided by matrix
%
g3 = mult(invp(pp1),pp2);
g3 = squeeze(g3);
%
pull
return
