function [T] = lftl(G,K)
%
%  This function operates on POLPAC system descriptions stored in modes 0,
%  4..6, and 7..9.  T is the lower linear fractional transformation (LFT)
%  of G and K.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    G      := System in forward path
%    K      := System in feedback path
%
%  Output Parameter:
%  -----------------
%
%    T      := Transmission system ( r -> y )
%
%  Explanation:
%  ------------
%
%               ---------------
%      r ------>|  G11   G12  |--------> y
%           --->|  G21   G22  |----
%           |   ---------------   |
%           |                     |
%           |       -------       |
%           --------|  K  |<-------
%                   -------
%
push('LFTL')
global debg
%
%  Check for necessary conversions first
%
[G,K] = mixmod(G,K);
%
%  Start by unpacking the structural information
%
[modG,lcolG,dtypG,stypG] = unpack(G(1,1));
[modK,lcolK,dtypK,stypK] = unpack(K(1,1));
logcol = max([lcolG,lcolK]);
[pG,mG] = logdim(G);
[pK,mK] = logdim(K);
domG = domain(G);
%
%  Check for consistency
%
if debg == 3,
  if modG > 0,
    if modG < 4,
      disp('LFTU: Error - Operates on system modes only'),
      abort,
    end,
  end,
  if stypK ~= stypG,
    disp('LFTL: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if mG <= pK,
    disp('LFTL: Error - Incompatible number of rows/columns'),
    abort,
  end,
  if pG <= mK,
    disp('LFTL: Error - Incompatible number of rows/columns'),
    abort,
  end,
end
%
%  Branch depending on the mode of the system.
%
if modG == 0,
  %
  %  Is a state-space description
  %
  [Ag,Bg,Cg,Dg] = oldsyst(G);
  [Ak,Bk,Ck,Dk] = oldsyst(K);
  %
  %  Extract the four subsystems.
  %
  Bg1 = Bg(:,1:mG-pK);
  Bg2 = Bg(:,mG-pK+1:mG);
  Cg1 = Cg(1:pG-mK,:);
  Cg2 = Cg(pG-mK+1:pG,:);
  Dg11 = Dg(1:pG-mK,1:mG-pK);
  Dg12 = Dg(1:pG-mK,mG-pK+1:mG);
  Dg21 = Dg(pG-mK+1:pG,1:mG-pK);
  Dg22 = Dg(pG-mK+1:pG,mG-pK+1:mG);
  DkDg22 = Dk*Dg22;
  DD = eye(size(DkDg22)) - DkDg22;
  %
  %  Check whether the problem is well-posed
  %
  if norm(DD) < 1000*eps,
    disp('LFTL: Error - System is ill-posed'),
    abort,
    return,
  end,
  %
  %  Now compute the feedback structure
  %
  DDinv = inv(DD);
  Bg2DDi = Bg2*DDinv;
  BkDg22DDi = Bk*Dg22*DDinv;
  DkCg2 = Dk*Cg2;
  DkDg21 = Dk*Dg21;
  Bcl1 = Bg1 + Bg2DDi*DkDg21;
  Bcl2 = Bk*Dg21 + BkDg22DDi*DkDg21;
  Bcl = [ Bcl1 ; Bcl2 ];
  Acl11 = Ag + Bg2DDi*DkCg2;
  Acl12 = Bg2DDi*Ck;
  Acl21 = Bk*Cg2 + BkDg22DDi*DkCg2;
  Acl22 = Ak + BkDg22DDi*Ck;
  Acl = [ Acl11 , Acl12 ; Acl21 , Acl22 ];
  Dg12DDi = Dg12*DDinv;
  Dcl = Dg11 + Dg12DDi*DkDg21;
  Ccl1 = Cg1 + Dg12DDi*DkCg2;
  Ccl2 = Dg12DDi*Ck;
  Ccl = [ Ccl1 , Ccl2 ];
  T = newsyst(Acl,Bcl,Ccl,Dcl,stypG);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = minreals(T);
  pull,
  return,
end
%
if modG == 4 | modG == 5 | modG == 6,
  %
  %  Is a transfer function matrix
  %  Extract the four submatrices.
  %
  G11 = gget(G,1:pG-mK,1:mG-pK);
  G12 = gget(G,1:pG-mK,mG-pK+1:mG);
  G21 = gget(G,pG-mK+1:pG,1:mG-pK);
  G22 = gget(G,pG-mK+1:pG,mG-pK+1:mG);
  %
  %  Now compute: T = G11 + G12*K*(I - G22*K)\G21
  %
  Aux = multg(G22,K);
  I = eyep(mK,mK,modG,logcol,stypG,domG);
  Aux = sub(I,Aux);
  %
  %  Check whether system is well-posed
  %
  [G2,P2] = strictprop(Aux);
  ord = order(P2);
  if ord == 0,
    gn = gain(P2);
    if gn == 0,
      disp('LFTL: Error - System is ill-posed'),
      abort,
      return,
    end,
  end,
  Aux = invg(Aux);
  Aux = multg(Aux,G21);
  Aux = multg(K,Aux);
  Aux = multg(G12,Aux);
  T = addg(G11,Aux);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = reduceg(T);
  pull,
  return,
end
%
if modG == 7 | modG == 8 | modG == 9,
  %
  %  Is a polynomial matrix system description
  %
  [Pg,Qg,Rg,Wg] = pms2plm(G);
  [Pk,Qk,Rk,Wk] = pms2plm(K);
  %
  %  Extract the four subsystems
  %
  [nQ,mQ] = logdim(Qg);
  [nR,mR] = logdim(Rg);
  Qg1 = gget(Qg,1:nQ,1:mG-pK);
  Qg2 = gget(Qg,1:nQ,mG-pK+1:mG);
  Rg1 = gget(Rg,1:pG-mK,1:mR);
  Rg2 = gget(Rg,pG-mK+1:pG,1:mR);
  Wg11 = gget(Wg,1:pG-mK,1:mG-pK);
  Wg12 = gget(Wg,1:pG-mK,mG-pK+1:mG);
  Wg21 = gget(Wg,pG-mK+1:pG,1:mG-pK);
  Wg22 = gget(Wg,pG-mK+1:pG,mG-pK+1:mG);
  %
  %  Plug closed loop polynomial matrices together
  %
  [qG,qG] = logdim(Pg);
  [qK,qK] = logdim(Pk);
  qT = qG + qK + mK + pK;
  pT = pG - mK;
  mT = mG - pK;
  Pcl = zerop(qT,qT,modG-6,logcol,stypG,domG);
  Im = eyep(mK,mK,modG-6,logcol,stypG,domG);
  Ip = eyep(pK,pK,modG-6,logcol,stypG,domG);
  Pcl = put(Pcl,Pg,1:qG,1:qG);
  Pcl = put(Pcl,Qg2,1:qG,qT-pK+1:qT);
  Pcl = put(Pcl,Pk,qG+1:qG+qK,qG+1:qG+qK);
  Pcl = put(Pcl,Qk,qG+1:qG+qK,qG+qK+1:qT-pK);
  Pcl = put(Pcl,minus(Rk),qG+qK+1:qT-mK,qG+1:qG+qK);
  Pcl = put(Pcl,Wk,qG+qK+1:qT-mK,qG+qK+1:qT-pK);
  Pcl = put(Pcl,minus(Ip),qG+qK+1:qT-mK,qT-pK+1:qT);
  Pcl = put(Pcl,minus(Rg2),qT-mK+1:qT,1:qG);
  Pcl = put(Pcl,minus(Im),qT-mK+1:qT,qG+qK+1:qT-pK);
  Pcl = put(Pcl,Wg22,qT-mK+1:qT,qT-pK+1:qT);
  Qcl = zerop(qT,mT,modG-6,logcol,stypG,domG);
  Qcl = put(Qcl,Qg1,1:qG,1:mT);
  Qcl = put(Qcl,Wg21,qT-mK+1:qT,1:mT);
  Rcl = zerop(pT,qT,modG-6,logcol,stypG,domG);
  Rcl = put(Rcl,Rg1,1:pT,1:qG);
  Rcl = put(Rcl,minus(Wg12),1:pT,qT-pK+1:qT);
  Wcl = Wg11;
  %
  %  Make a new polynomial matrix system
  %
  T = plm2pms(Pcl,Qcl,Rcl,Wcl);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = minrealps(T);
  pull,
  return,
end
%
return
