function [T] = lftu(G,K)
%
%  This function operates on POLPAC system descriptions stored in modes 0,
%  4..6, and 7..9.  T is the upper linear fractional transformation (LFT)
%  of G and K.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    G      := System in forward path
%    K      := System in feedback path
%
%  Output Parameter:
%  -----------------
%
%    T      := Transmission system ( r -> y )
%
%  Explanation:
%  ------------
%
%                   -------
%           --------|  K  |<-------
%           |       -------       |
%           |                     |
%           |   ---------------   |
%           --->|  G11   G12  |----
%      r ------>|  G21   G22  |--------> y
%               ---------------
%
push('LFTU')
global debg
%
%  Check for necessary conversions first
%
[G,K] = mixmod(G,K);
%
%  Start by unpacking the structural information
%
[modG,lcolG,dtypG,stypG] = unpack(G(1,1));
[modK,lcolK,dtypK,stypK] = unpack(K(1,1));
logcol = max([lcolG,lcolK]);
[pG,mG] = logdim(G);
[pK,mK] = logdim(K);
domG = domain(G);
%
%  Check for consistency
%
if debg == 3,
  if modG > 0,
    if modG < 4,
      disp('LFTU: Error - Operates on system modes only'),
      abort,
    end,
  end,
  if stypK ~= stypG,
    disp('LFTU: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if mG <= pK,
    disp('LFTU: Error - Incompatible number of rows/columns'),
    abort,
  end,
  if pG <= mK,
    disp('LFTU: Error - Incompatible number of rows/columns'),
    abort,
  end,
end
%
%  Branch depending on the mode of the system.
%
if modG == 0,
  %
  %  Is a state-space description
  %
  [Ag,Bg,Cg,Dg] = oldsyst(G);
  [Ak,Bk,Ck,Dk] = oldsyst(K);
  %
  %  Extract the four subsystems.
  %
  Bg1 = Bg(:,1:pK);
  Bg2 = Bg(:,pK+1:mG);
  Cg1 = Cg(1:mK,:);
  Cg2 = Cg(mK+1:pG,:);
  Dg11 = Dg(1:mK,1:pK);
  Dg12 = Dg(1:mK,pK+1:mG);
  Dg21 = Dg(mK+1:pG,1:pK);
  Dg22 = Dg(mK+1:pG,pK+1:mG);
  DkDg11 = Dk*Dg11;
  DD = eye(size(DkDg11)) - DkDg11;
  %
  %  Check whether the problem is well-posed
  %
  if norm(DD) < 1000*eps,
    disp('LFTU: Error - System is ill-posed'),
    abort,
    return,
  end,
  %
  %  Now compute the feedback structure
  %
  DDinv = inv(DD);
  Bg1DDi = Bg1*DDinv;
  BkDg11DDi = Bk*Dg11*DDinv;
  DkCg1 = Dk*Cg1;
  DkDg12 = Dk*Dg12;
  Bcl1 = Bg2 + Bg1DDi*DkDg12;
  Bcl2 = Bk*Dg12 + BkDg11DDi*DkDg12;
  Bcl = [ Bcl1 ; Bcl2 ];
  Acl11 = Ag + Bg1DDi*DkCg1;
  Acl12 = Bg1DDi*Ck;
  Acl21 = Bk*Cg1 + BkDg11DDi*DkCg1;
  Acl22 = Ak + BkDg11DDi*Ck;
  Acl = [ Acl11 , Acl12 ; Acl21 , Acl22 ];
  Dg21DDi = Dg21*DDinv;
  Dcl = Dg22 + Dg21DDi*DkDg12;
  Ccl1 = Cg2 + Dg21DDi*DkCg1;
  Ccl2 = Dg21DDi*Ck;
  Ccl = [ Ccl1 , Ccl2 ];
  T = newsyst(Acl,Bcl,Ccl,Dcl,stypG);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = minreals(T);
  pull,
  return,
end
%
if modG == 4 | modG == 5 | modG == 6,
  %
  %  Is a transfer function matrix
  %  Extract the four submatrices.
  %
  G11 = gget(G,1:mK,1:pK);
  G12 = gget(G,1:mK,pK+1:mG);
  G21 = gget(G,mK+1:pG,1:pK);
  G22 = gget(G,mK+1:pG,pK+1:mG);
  %
  %  Now compute: T = G22 + G21*K*(I - G11*K)\G12
  %
  Aux = multg(G11,K);
  I = eyep(mK,mK,modG,logcol,stypG,domG);
  Aux = sub(I,Aux);
  %
  %  Check whether system is well-posed
  %
  [G2,P2] = strictprop(Aux);
  ord = order(P2);
  if ord == 0,
    gn = gain(P2);
    if gn == 0,
      disp('LFTU: Error - System is ill-posed'),
      abort,
      return,
    end,
  end,
  Aux = invg(Aux);
  Aux = multg(Aux,G12);
  Aux = multg(K,Aux);
  Aux = multg(G21,Aux);
  T = addg(G22,Aux);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = reduceg(T);
  pull,
  return,
end
%
if modG == 7 | modG == 8 | modG == 9,
  %
  %  Is a polynomial matrix system description
  %
  [Pg,Qg,Rg,Wg] = pms2plm(G);
  [Pk,Qk,Rk,Wk] = pms2plm(K);
  %
  %  Extract the four subsystems.
  %
  [nQ,mQ] = logdim(Qg);
  [nR,mR] = logdim(Rg);
  Qg1 = gget(Qg,1:nQ,1:pK);
  Qg2 = gget(Qg,1:nQ,pK+1:mG);
  Rg1 = gget(Rg,1:mK,1:mR);
  Rg2 = gget(Rg,mK+1:pG,1:mR);
  Wg11 = gget(Wg,1:mK,1:pK);
  Wg12 = gget(Wg,1:mK,pK+1:mG);
  Wg21 = gget(Wg,mK+1:pG,1:pK);
  Wg22 = gget(Wg,mK+1:pG,pK+1:mG);
  %
  %  Plug closed loop polynomial matrices together
  %
  [qG,qG] = logdim(Pg);
  [qK,qK] = logdim(Pk);
  qT = qG + qK + mK + pK;
  pT = pG - mK;
  mT = mG - pK;
  Pcl = zerop(qT,qT,modG-6,logcol,stypG,domG);
  Im = eyep(mK,mK,modG-6,logcol,stypG,domG);
  Ip = eyep(pK,pK,modG-6,logcol,stypG,domG);
  Pcl = put(Pcl,Pg,1:qG,1:qG);
  Pcl = put(Pcl,Qg1,1:qG,qT-pK+1:qT);
  Pcl = put(Pcl,Pk,qG+1:qG+qK,qG+1:qG+qK);
  Pcl = put(Pcl,Qk,qG+1:qG+qK,qG+qK+1:qT-pK);
  Pcl = put(Pcl,minus(Rk),qG+qK+1:qT-mK,qG+1:qG+qK);
  Pcl = put(Pcl,Wk,qG+qK+1:qT-mK,qG+qK+1:qT-pK);
  Pcl = put(Pcl,minus(Ip),qG+qK+1:qT-mK,qT-pK+1:qT);
  Pcl = put(Pcl,minus(Rg1),qT-mK+1:qT,1:qG);
  Pcl = put(Pcl,minus(Im),qT-mK+1:qT,qG+qK+1:qT-pK);
  Pcl = put(Pcl,Wg11,qT-mK+1:qT,qT-pK+1:qT);
  Qcl = zerop(qT,mT,modG-6,logcol,stypG,domG);
  Qcl = put(Qcl,Qg2,1:qG,1:mT);
  Qcl = put(Qcl,Wg12,qT-mK+1:qT,1:mT);
  Rcl = zerop(pT,qT,modG-6,logcol,stypG,domG);
  Rcl = put(Rcl,Rg2,1:pT,1:qG);
  Rcl = put(Rcl,minus(Wg21),1:pT,qT-pK+1:qT);
  Wcl = Wg22;
  %
  %  Make a new polynomial matrix system
  %
  T = plm2pms(Pcl,Qcl,Rcl,Wcl);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = minrealps(T);
  pull,
  return,
end
%
return
