function [n,m] = logdim(pm)
%
%  The function LOGDIM returns the logical rather than the physical
%  dimensions of the POLPAC data structure PM.
%
%    Physical dimension ::= rows x columns of a matrix.
%    logcol (logrow)    ::= number of physical columns (rows) making up
%                           one logical column (row).
%
%  Input Parameter:
%  ----------------
%    pm     ::= POLPAC input data structure (stored in any mode)
% 
%  Output Parameter:
%  -----------------
%    for storage modes 1 thru 6:
%    n      ::= number of logical rows.
%    m      ::= number of logical columns.
%
%    for storage modes 7,8,9 and 0
%    n      ::= number of outputs.
%    m      ::= number of inputs.
%
push('LOGDIM')
%
[mode,logcol,dtype,stype] = unpack(pm(1,1));
[prow,pcol] = size(pm);
prow1 = prow-1;
m = pcol/logcol;
%
%=============================================================================
%  POLYNOMIAL MATRIX: 
%  ------------------
%
if mode == 1,
  %
  %  Nonfactorized polynomial matrix
  %
  n = prow1;
  else if mode == 2,
    %
    %  Factorized polynomial matrix
    %
    n = prow1/2;
    else if mode == 3,
      %
      %  Trajectory polynomial matrix
      %
      n = prow1-1;   
    end,
  end,
end
%
%=============================================================================
%  TRANSFER FUNCTION MATRIX:
%  -------------------------
%
if mode == 4,
  %
  %  Nonfactorized transfer function matrix
  %
  n = prow1/2;
  else if mode == 5, 
    %
    %  Factorized transfer function matrix
    %
    n = prow1/4;
    else if mode == 6,
      %
      %  Trajectory transfer function matrix
      %
      n = (prow1 - 1)/2;
    end,
  end,
end
%
%=============================================================================
%  POLYNOMIAL MATRIX SYSTEM:
%  -------------------------
%
if mode > 6,
  [ns,ms] = size(pm);
  if ms == 1,
    ordr = 0;
    else
    ordr = pm(1,2);
  end;
  m = m - ordr;
end
if mode == 7,
  %
  %  Nonfactorized polynomial matrix system
  %
  n = prow1 - ordr;
  else if mode == 8,
    %
    %  Factorized polynomial matrix system
    %
    n = prow1/2 - ordr;
    else if mode == 9,
      %
      %  Trajectory polynomial matrix system
      %
      n = prow1 - 1 - ordr;
    end,
  end,
end
%
%============================================================================
%  SYSTEM REPRESENTATION IN TIME DOMAIN:
%  -------------------------------------
%
if mode == 0,
  [ns,ms] = size(pm);
  if ms == 1,
    ordr = 0;
    else
    ordr = pm(1,2);
  end;
  n = prow1 - ordr;
  m = pcol - ordr;
end
%
pull
return
