function [pm2] = minp(pm)
%
%  This function computes the upper envelopes of trajectory matrices
%  stored in POLPAC mode 3.  The logic is as follows:
%
%  If PM is a scalar trajectory, MINP returns its maximum recorded
%  function value.  If all values are either REAL or INTEGER (i.e.,
%  DTYPE ~= 3), MINP returns the most positive value.  If some
%  function values are complex (DTYPE = 3), MINP returns the
%  absolute largest value.
%
%  If PM is a trajectory vector (either a logical column vector or
%  a logical row vector), MINP returns the upper envelope of all
%  trajectories in the trajectory vector.
%
%  If PM is a trajectory matrix, MINP returns a row trajectory
%  vector which consists of the upper column envelopes of all
%  trajectories in each column.
%
%  Therefore:
%
%    transpose(minp(transpose(pm))  returns a column trajectory vector
%                                   consisting of the upper row
%                                   envelopes of all trajectories in
%                                   each row.
%
%    minp(minp(pm))  returns the upper envelope of all trajectories
%                    in the trajectory matrix.
%
%    minp(minp(minp(pm)))  returns the largest recorded value which can
%                          e.g. be used for scaling purposes.
%
%  Input Parameter:
%  ----------------
%
%    pm     :=  Any POLPAC trajectory matrix (stored in mode 3)
%
%  Output Parameter:
%  -----------------
%
%    pm2    :=  The 'maximum' of PM according to the above definitions.
%
%        logdim(pm)=[1,1]  ::=  PM2 is the largest recorded value (a Matlab
%                               number)
%
%        logdim(pm)=[n,1]  ::=  PM2 is the upper envelope (a single POLPAC
%   or:  logdim(pm)=[1,m]  ::=  trajectory stored in mode 3)
%
%        logdim(pm)=[n,m]  ::=  PM2 is a row trajectory vector of logical
%                               length m (stored in mode 3)
%
push('MINP')
global debg
%
%  Start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(pm(1,1));
%
%  Check for consistency
%
if debg == 3,
  if mode ~= 3,
    disp('MINP: Error - Operates on mode 3 only'),
    abort,
  end,
end
%
%  Get the logical dimensions
%
[lr,lc] = logdim(pm);
%
%  Handle the scalar case first
%
if lr*lc == 1,
  val = pm(3,:);
  pm2 = min(val);
  pull,
  return,
end
%
%  Next comes the vector case
%
if lr == 1,
  val = zeros(lc,logcol);
  for i=1:lc,
    val(i,:) = pm(3,(i-1)*logcol+1:i*logcol);
  end,
end
if lc == 1,
  val = pm(3:lr+2,:);
end
if min([lr,lc]) == 1,
  val = min(val);
  dom = domain(pm);
  pm2 = zerop(1,1,mode,logcol,stype,dom);
  pm2(3,:) = val;
  dtype = datatype(pm2);
  if dtype > 1,
    pm2(1,1) = ppack(mode,logcol,dtype,stype);
  end,
  pull,
  return,
end
%
%  Finally, handle the matrix case recursively
%
dom = domain(pm);
pm2 = zerop(1,lc,mode,logcol,stype,dom);
for i=1:lc,
  pmi = gget(pm,1:lr,i);
  pmi = minp(pmi);
  pm2 = put(pm2,pmi,1,i);
end
%
pull
return
