function [Smin,Zid,Zoi] = minrealps(S)
%
%  This function determines the minimal polynomial matrix
%  realization of a system, which is represented in the form:
%
%           S(s) = [  P(s) , Q(s)
%                    -R(s) , W(s) ]
%
%  where the four polynomial matrices are 
%
%           P(s) :  dimension qxq
%           Q(s) :  dimension qxm
%           R(s) :  dimension pxq
%           W(s) :  dimension pxm
%
%  Input Parameter:
%  ----------------
%
%    S      :=  Polynomial matrix system (modes 7 .. 9)
%
%  Output Parameters:
%  ------------------
%
%    Smin   :=  Minimal realization (same mode as S)
%    Zid    :=  Input decoupling zeros (uncontrollable modes)
%    Zoi    :=  Output-input decoupling zeros (unobservable yet controllable
%               modes)
%
%  The minimal realization is denoted by
%
%          Smin(s) = [  Poi(s)  Qid(s)
%                      -Roi(s)  W(s)   ]
%
%  The transmission zeros (poles of the system) are the invariant dividers
%  of Smin(s).  The routine first applies input decoupling, and then
%  output decoupling, i.e., the invariant dividers of Zid(s) are ALL
%  uncontrollable modes, irrespective of whether they are observable or not.
%  Output decoupling is done to the controllable subsystems, i.e., the
%  invariant dividers of Zoi(s) contain only those unobservable modes that
%  are controllable.
%
%  The decomposition can be written as:
%
%   S(s) = [ P(s) Q(s)   = [ Zid(s) 0   * [ Poi(s) Qid(s)   * [ Zoi(s) 0
%           -R(s) W(s) ]       0    I ]    -Roi(s) W(s)   ]       0    I ]
%
push('MINREALPS')
global debg
%
%  Get the structural information
%
[mode,logcol,dtype,stype] = unpack(S(1,1));
%
%  Consistency checks
%
if debg == 3,
  if mode < 7,
    disp('MINREALPS: Error - Operates on modes 7 to 9 only'),
    abort,
    return,
  end,
end
%
%  Extract P,Q,R and W from S(s)
%
[P,Q,R,W] = pms2plm(S);
[iP,jP] = logdim(P);
[iQ,jQ] = logdim(Q);
[iR,jR] = logdim(R);
% 
%  Do input decoupling first.  Determine the largest common left divider
%  of P(s) and Q(s).
%
[TL,QUL] = lcld(P,Q);
[iTL,jTL] = logdim(TL);
Zid = TL;
Pid = gget(QUL,1:jTL,1:jP);
Qid = gget(QUL,1:jTL,jP+1:jP+jQ); 
%
%  Now do output decoupling.  Determine the largest common right divider
%  of Pid(s) and R(s); 
%
[TU,QUR] = lcrd(Pid,R);
[iTU,jTU] = logdim(TU);
Zoi = TU;
Poi = gget(QUR,1:jTL,1:jTU);
Roi = gget(QUR,jTL+1:jTL+iR,1:jTU);
%
%  Pack the individual matrices back together
%
Smin = plm2pms(Poi,Qid,Roi,W);
%
pull
return      
