function [s2] = minus(s1)
%
%  This function negates an arbitrary POLPAC data structure.  It
%  operates on all 10 modes.
%
%  The negation function is defined as follows:
%
%    mode = 0 :      s1 = [ a , b ; c , d ]
%                ==> s2 = [ a , b ; -c , -d ]
%    mode = 1 :      All coefficients are negated
%    mode = 2 :      All gain factors are negated
%    mode = 3 :      All trajectory values (but not the domain)
%                    are negated
%    mode = 4 :      All numerator coefficients are negated
%    mode = 5 :      All numerator gain factors are negated
%    mode = 6 :      All numerator trajectory values are negated
%    mode = 7 :      s1 = [ p , q ; -r , w ] (in the sense of mode=1)
%                ==> s2 = [ p , q ; r , -w ] (in the sense of mode=1)
%    mode = 8 :      same, but in the sense of mode=2
%    mode = 9 :      same, but in the sense of mode=3
%
push('MINUS')
%
%  Start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(s1(1,1));
[np,mp] = size(s1);
[nl,ml] = logdim(s1);
%
%  Branch depending on the mode to compute the physical
%  locations of those matrix elements that must be negated.
%
if mode == 0,
  [ns,ms] = size(s1);
  if ms == 1,
    ord = 0;
    else
    ord = s1(1,2);
  end,
  prow = ord+2 : np;
  pcol = 1 : mp;
end
%
if mode == 1,
  prow = 2 : np;
  pcol = 1 : mp;
  else if mode == 2,
    prow = 2 : 2 : np-1;
    pcol = 1 : logcol : mp-logcol+1;
    else if mode == 3,
      prow = 3 : np;
      pcol = 1 : mp;
    end,
  end,
end
%
if mode == 4,
  prow = 2 : nl+1;
  pcol = 1 : mp;
  else if mode == 5,
    prow = 2 : 2 : 2*nl;
    pcol = 1 : logcol : mp-logcol+1;
    else if mode == 6,
      prow = 3 : nl+2;
      pcol = 1 : mp;
    end,
  end,
end
%
if mode > 6,
  [ns,ms] = size(s1);
  if ms == 1,
    ord = 0;
    else
    ord = s1(1,2);
  end,
end
if mode == 7,
  prow = ord+2 : np;
  pcol = 1 : mp;
  else if mode == 8,
    prow = 2*ord+2 : 2 : np-1;
    pcol = 1 : logcol : mp-logcol+1;
    else if mode == 9,
      prow = ord+3 : np;
      pcol = 1 : mp;
    end,
  end,
end
%
%  Now negate the designated matrix elements
%
s2 = s1;
s2(prow,pcol) = -s2(prow,pcol);
%
pull
return
