function [p3,p4] = mixmod(p1,p2)
%
%  This function performs all kind of mixed mode conversions with
%  a flavor of preference for trajectory representations over
%  coefficient representations over root representations
%
%  Input Parameters:
%  -----------------
%
%    p1     := Any POLPAC data structure in modes 1 thru 9
%    p2     := Any POLPAC data structure in modes 1 thru 9
%
%  Output Parameters:
%  ------------------
%
%    p3,p4  := Converted data structures (p1->p3,p2->p4), such
%              that mode(p3)=mode(p4).
%
%
push('MIXMOD')
global debg
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(p1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(p2(1,1));
%
%  Initialize the output variables
%
p3 = p1;
p4 = p2;
%
%  Check for consistency
%
if debg == 3,
  if styp2 ~= styp1,
    disp('MIXMOD: Error - Incompatible system types'),
    abort,
  end,
end
%
%  Check whether a conversion between classes of modes must be done
%
mc1 = round((mod1 + 1)/3);
mc2 = round((mod2 + 1)/3);
if abs(mc1 - mc2) == 3,
  %
  %  One of the data structures is a state space description, whereas the
  %  other is a polynomial matrix system description.  Upgrade the state
  %  space description to the polynomial matrix form.
  %
  if mc1 == 0,
    p3 = ss2pms(p3,mod2,domain(p4));
    pull,
    return,
  end,
  if mc2 == 0,
    s2 = ss2pms(s2,mod1,domain(s1));
    pull,
    return
  end,
end
%
%  Check whether one of the data structures is a state-space description,
%  whereas the other is a transfer function matrix.  If so, upgrade the
%  state-space description to the status of a rational function matrix.
%
if mc1 == 0,
  if mc2 == 2,
    p3 = ss2tfm(p3,mod2,domain(p4));
    pull,
    return,
  end,
end
if mc1 == 2,
  if mc2 == 0,
    p4 = ss2tfm(p4,mod1,domain(p3));
    pull,
    return,
  end,
end
%
%  Check if one of the data structure is a polynomial matrix, the other a
%  rational function matrix.  If so, upgrade the polynomial matrix to the
%  status of a rational function matrix.
%
if mc1*mc2 == 2,
  if mc1 == 1,
    [lr1,lc1] = logdim(p3);
    dom1 = domain(p3);
    q = onesp(lr1,lc1,mod1,lcol1,styp1,dom1);
    p3 = plm2tfm(p3,q);
    mod1 = mod1 + 3;
  end,
  if mc2 == 1,
    [lr2,lc2] = logdim(p4);
    dom2 = domain(p4);
    q = onesp(lr2,lc2,mod2,lcol2,styp2,dom2);
    p4 = plm2tfm(p4,q);
    mod2 = mod2 + 3;
  end,
end
%
%  Check for domain conversions
%
if mod1 == mod2,
  m3 = mod1/3;
  if abs(m3 - round(m3)) > 100*eps,
    pull,
    return,
  end,
  dom1 = domain(p3);
  dom2 = domain(p4);
  if normcomp(dom1,dom2,100*eps) == 1,
    pull,
    return,
  end,
  if mod1 == 3,
    p4 = c33(p4,dom1);
    else if mod1 == 6,
      p4 = c66(p4,dom1);
      else
      p4 = c99(p4,dom1);
    end,
  end,
  pull,
  return,
end
%
%  Check for conversions within a single group of mode classes.
%  Give preference to trajectory representation over coefficient
%  representation over root representation.
%
%  Treat classes 1 through 3, 4 through 6, 7 through 9 separately
%
%  Classes 1 through 3
%
if mc1 == 1,
  if mod1 == 1,
    if mod2 == 2,
      p4 = c21(p2);
      else
      dom2 = domain(p2);
      p3 = c13(p1,dom2);
    end,
    else if mod1 == 2,
      if mod2 == 1,
        p3 = c21(p1);
        else
        dom2 = domain(p2);
        p3 = c23(p1,dom2);
      end,
      else
        dom1 = domain(p1);
        if mod2 == 1,
          p4 = c13(p2,dom1);
          else
          p4 = c23(p2,dom1);
        end,
      end,
    end,
  end,
  pull,
  return,
end
%
%  Classes 4 through 6
%
if mc1 == 2,
  if mod1 == 4,
    if mod2 == 5,
      p4 = c54(p2);
      else
      dom2 = domain(p2);
      p3 = c46(p1,dom2);
    end,
    else if mod1 == 5,
      if mod2 == 4,
        p3 = c54(p1);
        else
        dom2 = domain(p2);
        p3 = c56(p1,dom2);
      end,
      else
        dom1 = domain(p1);
        if mod2 == 4,
          p4 = c46(p2,dom1);
          else
          p4 = c56(p2,dom1);
        end,
      end,
    end,
  end,
  pull,
  return,
end
%
%  Classes 7 through 9
%
if mc1 == 2,
  if mod1 == 7,
    if mod2 == 8,
      p4 = c87(p2);
      else
      dom2 = domain(p2);
      p3 = c79(p1,dom2);
    end,
    else if mod1 == 8,
      if mod2 == 7,
        p3 = c87(p1);
        else
        dom2 = domain(p2);
        p3 = c89(p1,dom2);
      end,
      else
        dom1 = domain(p1);
        if mod2 == 7,
          p4 = c79(p1,dom1);
          else
          p4 = c89(p1,dom1);
        end,
      end,
    end,
  end,
end
%
pull
return
