function [v] = modal(a,l,m)
%
%  This function generates the generalized modal matrix for a square
%  matrix A with eigenvalues L and multiplicities M.
%
%  Input Parameters:
%  -----------------
%
%    a      := any square matrix
%    l      := column vector of distinct eigenvalues of A
%    m      := column vector of their multiplicities
%
%  Output Parameter:
%  -----------------
%
%    v      := the generalized modal matrix
%
push('MODAL')
global debg
%
[n,n1] = size(a);
[nl,n2] = size(l);
[n3,n4] = size(m);
%
%  Check for consistency
%
if debg == 3,
  if n ~= n1,
    disp('MODAL: Error - Matrix not square'),
    abort,
  end,
  if n2 ~= 1,
    disp('MODAL: Error - Eigenvalues must form a column vector'),
    abort,
  end,
  if n3 ~= nl,
    disp('MODAL: Error - Multiplicities incompatible with eigenvalues'),
    abort,
  end,
  if n4 ~= 1,
    disp('MODAL: Error - Multiplicities must form a column vector'),
    abort,
  end,
  if sum(m) ~= n,
    disp('MODAL: Error - Multiplicities incompatible with matrix size'),
    abort,
  end,
  if min(m) < 1,
    disp('MODAL: Error - Multiplicities must all be positive'),
    abort,
  end,
  if norm(m - round(m),'inf') > 10*eps,
    disp('MODAL: Error - Multiplicities must all be integer'),
    abort,
  end,
end
%
%  Treat the simple cases first.  if there are as many distinct eigenvalues
%  as the dimension of A, obviously all of them have a multiplicity of 1,
%  and the regular algorithm can be used.
%
if n == nl,
  [v,d] = eig(a);
  pull,
  return,
end
%
%  Too bad, we need to do more work.  Prepare the modal matrix (V) by a
%  dummy first column to which we are going to concatenate all the
%  eigenvectors as soon as we found them.  IND is used to store the
%  grade of the individual generalized eigenvectors.
%
v = zeros(n,1);
ind = 0;
%
%  Loop over all distinct eigenvalues
%
for i=1:nl,
  [n1,ni] = size(ind);
  aleph = a - l(i)*eye(n);
  %
  %  Check whether the multiplicity of this eigenvalue is 1.  If so,
  %  one simple SVD will do.
  %
  if m(i) == 1,
    [ui,si,vi] = svd(aleph);
    x = vi(:,n);
    v = [ v , x ];
    ind = [ ind , 1 ];
    else
    %
    %  Nope.  Compute the Rank (RHO) and Nullity (NU) of ALEPH
    %
    beth = aleph;
    k = 1;
    [ui,si,vi] = svd(beth);
    uu = ui;
    si = diag(si);
    ss = si;
    vv = vi;
    ii = n;
    while si(ii) < 100*eps,
      ii = ii - 1;
    end,
    rho(k) = ii;
    nu(k) = n - ii;
    %
    %  Check whether the Nullity equals the multiplicity.  If so,
    %  a simple SVD still suffices.
    %
    if nu(k) == m(i),
      nn = n - m(i) + 1;
      x = v(:,nn:n);
      v = [ v , x ];
      ii = ones(1,m(i));
      ind = [ ind , ii ];
      else
      %
      %  No luck.  This is the most complicated case.
      %  Compute the Ranks and Nullities of the higher grades
      %
      while nu(k) < m(i),
        k = k + 1;
        beth = aleph*beth;
        [ui,si,vi] = svd(beth);
        uu = [ uu , ui ];
        si = diag(si);
        ss = [ ss , si ];
        vv = [ vv , vi ];
        tol = eps^(1/k);
        ii = n;
        while si(ii) < 100*tol,
          ii = ii - 1;
        end,
        rho(k) = ii;
        nu(k) = n - ii;
      end,
      %
      %  Loop over all generalized eigenvectors, starting with
      %  the highest power.
      %
      for kk=k:-1:2,
        %
        %  Calculate the number of generalized eigenvectors of
        %  this grade (NV).
        %
        nv = nu(kk) - nu(kk-1);
        %
        %  Find out how many of those were already found (NK).
        %  There are NR remaining ones.
        %
        [n1,nj] = size(ind);
        nk = 0;
        if nj > ni,
          nn = ni + 1;
          for lk=nn:nj,
            if ind(lk) == kk,
              nk = nk + 1;
            end,
          end,
        end,
        nr = nv - nk;
        %
        %  Loop over the remaining generalized eigenvectors
        %  of grade KK, and generate their chains.
        %
        if nr > 0,
          %
          %  Retrieve the Image of ALEPH^KK
          %
          n1 = n*(kk - 1) + 1;
          n2 = n*kk;
          vi = vv(:,n1:n2);
          nn = rho(kk);
          x = vi(:,1:nn);
          %
          %  Retrieve the Nullspace of ALEPH^(KK-1), and concatenate
          %  it from the right.
          %
          n1 = n*(kk - 2) + 1;
          n2 = n*(kk - 1);
          vi = vv(:,n1:n2);
          nn = rho(kk-1) + 1;
          xx = vi(:,nn:n);
          x = [ x , xx ];
          %
          %  Add the generalized eigenvectors of the same grade for
          %  the same eigenvalue that were already found.
          %
          if nk > 0,
            nn = ni + 1;
            for lk=nn:nj,
              if ind(lk) == kk,
                x = [ x , v(:,lk) ];
              end,
            end,
          end,
          %
          %  Fill up with zeros to make X a square matrix.
          %
          [n,n1] = size(x);
          nn = n - n1;
          x = [ x , zeros(n,nn) ];
          %
          %  Now, get the SVD of X.
          %
          [ui,si,vi] = svd(x);
          %
          %  Okay, we just found NN new generalized eigenvectors
          %  of grade KK.  Create their chains.
          %
          nn = n1 + 1;
          for lk=nn:n,
            x = ui(:,lk);
            vvv = x;
            kkk = kk - 1;
            for ll=kkk:-1:1,
              x = aleph*x;
              vvv = [ x , vvv ];
            end,
            v = [ v , vvv ];
            ind = [ ind , 1:kk ];
          end,
        end,
      end,
    end,
  end,
end
%
%  Get rid of the dummy first column of V
%
v = v(:,2:n+1);
%
pull
return
