function [l2,m2,indx] = mroot0(l1,m1)
%
%  This function tests a set of (possibly complex) eigenvalues
%  (roots) for multiples.  It is taken for granted that the
%  eigenvalues (roots) are specified correctly, and are not the
%  result of a previous (possibly unprecise) numerical operation.
%
%  Multiplicities of the input vector need not be known.  It just
%  increases the speed of the algorithm if they are.  Thus, M1 can
%  always be specified as a ONES-vector.
%
%  The eigenvalues (roots) are simultaneously sorted in
%  increasing length.  if two different eigenvalues are of same
%  length, the one with smaller angle (between 0 and 360 degrees)
%  comes first.
%
%  if the characteristic polynomial corresponding to the sets of
%  roots is REAL (negligible imaginary parts), the roots will be
%  furthermore equalized to make pairs of complex poles truly
%  conjugate complex.
%
%  Input Parameters:
%  -----------------
%
%    l1     := A set of roots (column vector)
%    m1     := Column vector of their multiplicities
%
%  Output Parameters:
%  ------------------
%
%    l2     := Column vector of sorted distinct eigenvalues
%    m2     := Column vector of their multiplicities
%    indx   := Index column vector relating the input vector
%              to the output vector
%
push('MROOT')
global debg
[n,m] = size(l1);
%
%  Check for consistency
%
if debg == 3,
  [nn,mm] = size(m1);
  if m ~= 1,
    disp('MROOT0: Error - The input must be a column vector'),
    abort,
  end,
  if mm ~= 1,
    disp('MROOT0: Error - The input must be a column vector'),
    abort,
  end,
  if n ~= nn,
    disp('MROOT0: Error - The input vectors are incompatible'),
    abort,
  end,
end
%
indx = 100*ones(1,n);
%
%  Decide whether or not to equalize
%
r = l1;
for i=1:n,
  if m1(i) > 1,
    r = [ r ; l1(i)*ones(m1(i)-1,1) ];
  end
end
p = poly(r);
check = 0;
if norm(imag(p),'inf') < 1000*eps,
  check = 1;
end
%
%  Sort the numbers according to magnitude and phase.
%
[l2,index] = sortc(l1);
m2 = m1(index);
%
%  Now equalize conjugate complex pairs if necessary
%
if check == 1,
  for i=1:n-1,
    ll = l2(i);
    if abs(imag(ll)) < 1000*eps,
      l2(i) = real(ll);
     else
      lc = conj(ll);
      for k=i+1:n,
        if abs(l2(k) - lc) < 1000*eps,
          re = (real(ll) + real(lc))/2;
          im = (imag(ll) - imag(lc))/2;
          l2(i) = re + j*im;
          l2(k) = re - j*im;
        end,
      end,
    end,
  end,
  ll = l2(n);
  if abs(imag(ll)) < 1000*eps,
    l2(n) = real(ll);
  end,
end
%
%  Finally, compute the multiplicities
%
ll1 = l2;
mm1 = m2;
k = 1;
kk = 1;
l2 = ll1(1);
m2 = mm1(1);
indx(index(1)) = 1; 
for i=2:n,
  indx(index(kk)) = k;
  if abs(ll1(i) - l2(k)) < 1000*eps,
    m2(k) = m2(k) + mm1(i);
    indx(index(i)) = k;
    kk = kk + 1;
   else
    kk = kk + 1;
    k = k + 1;
    l2 = [ l2 ; ll1(i) ];
    m2 = [ m2 ; mm1(i) ];
  end,
  indx(index(kk)) = k;
end
indx = indx';
%
pull
return
