function [g3] = multg(g1,g2)
%
%  This function operates on transfer function matrices stored in modes 4
%  thru 6.  G3 is the logical product of G1 and G2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    g1     := Transfer function matrix in modes 4 thru 6
%    g2     := Transfer function matrix in modes 4 thru 6
%
%  Output Parameter:
%  -----------------
%
%    g3     := Transfer function matrix in modes 4 thru 6
%
push('MULTG')
global debg
%
%  Check for necessary conversions first
%
[gg1,gg2] = mixmod(g1,g2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(gg1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(gg2(1,1));
logcol = lcol1 + lcol2 - 1;
if mod1 == 6,
  logcol = lcol1;
end
%
[n1,m1] = logdim(gg1);
[n2,m2] = logdim(gg2);
nm1 = n1*m1;
nm2 = n2*m2;
%
dom1 = domain(gg1);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 4,
    disp('MULTG: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if mod1 > 6,
    disp('MULTG: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('MULTG: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if nm1 ~= 1,
    if nm2 ~= 1,
      if m1 ~= n2,
        disp('MULTG: Error - Incompatible number of rows/columns'),
        abort,
      end,
    end,
  end,
end
%
%  Perform the actual multiplication now.
%
%  Check first whether this is a scalar/scalar multiplication
%
if nm1 == 1,
  if nm2 == 1,
    g3 = multg1(gg1,gg2);
    g3 = squeeze(g3);
    pull,
    return,
  end,
end
%
%  Check first whether this is a matrix/scalar multiplication or
%  a matrix/matrix multiplication
%
if nm1 == 1,
  if nm2 ~= 1,
    %
    % The multiplication is of type scalar times matrix
    %
    g3 = zerop(n2,m2,mod1,logcol,styp1,dom1);
    for i=1:n2,
      for k=1:m2,
        g2ik = gget(gg2,i,k);
        g3ik = multg1(gg1,g2ik);
        g3 = put(g3,g3ik,i,k);
      end,
    end,
    g3 = squeeze(g3);
    pull,
    return,
  end,
end
%
if nm1 ~= 1,
  if nm2 == 1,
    %
    % The multiplication is of type matrix times scalar
    %
    g3 = zerop(n1,m1,mod1,logcol,styp1,dom1);
    for i=1:n1,
      for k=1:m1,
        g1ik = gget(gg1,i,k);
        g3ik = multg1(g1ik,gg2);
        g3 = put(g3,g3ik,i,k);
      end,
    end,
    g3 = squeeze(g3);
    pull,
    return,
  end,
end
%
%  The operation must be of type: matrix times matrix
%
g3 = zerop(n1,m2,mod1,logcol,styp1,dom1);
for i=1:n1,
  for k=1:m2,
    g3s = zerop(1,1,mod1,logcol,styp1,dom1);
    for kk=1:m1,
      g1ik = gget(gg1,i,kk);
      g2ik = gget(gg2,kk,k);
      g3ik = multg1(g1ik,g2ik);
      g3s = addg1(g3s,g3ik);
    end,
    g3 = put(g3,g3s,i,k);
  end,
end
g3 = squeeze(g3);
%
pull
return
