function [p3] = multp(p1,p2)
%
%  This function operates on polynomial matrices stored in modes 1 thru 3.
%  P3 is the logical product of P1 and P2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    p1     := Polynomial matrix in modes 1 thru 3
%    p2     := Polynomial matrix in modes 1 thru 3
%
%  Output Parameter:
%  -----------------
%
%    p3     := Polynomial matrix in modes 1 thru 3
%
push('MULTP')
global debg
%
%  Check for necessary conversions first
%
[pp1,pp2] = mixmod(p1,p2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(pp1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(pp2(1,1));
logcol = lcol1 + lcol2 - 1;
if mod1 == 3,
  logcol = lcol1;
end
%
[n1,m1] = logdim(pp1);
[n2,m2] = logdim(pp2);
nm1 = n1*m1;
nm2 = n2*m2;
%
dom1 = domain(pp1);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 1,
    disp('MULTP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if mod1 > 3,
    disp('MULTP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('MULTP: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if nm1 ~= 1,
    if nm2 ~= 1,
      if m1 ~= n2,
        disp('MULTP: Error - Incompatible number of rows/columns'),
        abort,
      end,
    end,
  end,
end
%
%  Perform the actual multiplication now.
%
%
%  Check first whether this is a scalar/scalar multiplication
%
if nm1 == 1,
  if nm2 == 1,
    p3 = multp1(pp1,pp2);
    p3 = squeeze(p3);
    pull,
    return,
  end,
end
%    
%  Check first whether this is a matrix/scalar multiplication or
%  a matrix/matrix multiplication
%
if nm1 == 1,
  if nm2 ~= 1,
    %
    % The multiplication is of type scalar times matrix
    %
    p3 = zerop(n2,m2,mod1,logcol,styp1,dom1);
    for i=1:n2,
      for k=1:m2,
        p2ik = gget(pp2,i,k);
        p3ik = multp1(pp1,p2ik);
        p3 = put(p3,p3ik,i,k);
      end,
    end,
    p3 = squeeze(p3);
    pull,
    return,
  end,
end
%
if nm1 ~= 1,
  if nm2 == 1,
    %
    % The multiplication is of type matrix times scalar
    %
    p3 = zerop(n1,m1,mod1,logcol,styp1,dom1);
    for i=1:n1,
      for k=1:m1,
        p1ik = gget(pp1,i,k);
        p3ik = multp1(p1ik,pp2);
        p3 = put(p3,p3ik,i,k);
      end,
    end,
    p3 = squeeze(p3);
    pull,
    return,
  end,
end
%
%  The operation must be of type: matrix times matrix
%
p3 = zerop(n1,m2,mod1,logcol,styp1,dom1);
for i=1:n1,
  for k=1:m2,
    p3s = zerop(1,1,mod1,logcol,styp1,dom1);
    for kk=1:m1,
      p1ik = gget(pp1,i,kk);
      p2ik = gget(pp2,kk,k);
      p3ik = multp1(p1ik,p2ik);
      p3s = addp1(p3s,p3ik);
    end,
    p3 = put(p3,p3s,i,k);
  end,
end
p3 = squeeze(p3);
%
pull
return
