function [p3] = multp1(p1,p2)
%
%  This function operates on single polynomials stored in modes 1 thru 3.
%  P3 is the logical product of P1 and P2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    p1     := Single polynomial in modes 1 thru 3
%    p2     := Single polynomial in modes 1 thru 3
%
%  Output Parameter:
%  -----------------
%
%    p3     := Single polynomial in modes 1 thru 3
%
push('MULTP1')
global debg
global mrtflag
%
%  Check for necessary conversions first
%
[pp1,pp2] = mixmod(p1,p2);
pp1 = squeeze(pp1);
pp2 = squeeze(pp2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(pp1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(pp2(1,1));
%
[n1,m1] = logdim(pp1);
[n2,m2] = logdim(pp2);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 1,
    disp('MULTP1: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if mod1 > 3,
    disp('MULTP1: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('MULTP1: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if n1*m1 ~= 1,
    disp('MULTP1: Error - Operates on scalar polynomials only'),
    abort,
  end,
  if n2*m2 ~= 1,
    disp('MULTP1: Error - Operates on scalar polynomials only'),
    abort,
  end,
end
%
%  Perform the actual multiplication now.
%  Treat each mode separately.
%
%  Storage mode = 1
%
if mod1 == 1,
  %
  %  Use convolution on polynomial coefficients
  %
  pp3 = conv(pp1(2,1:lcol1),pp2(2,1:lcol2));
  %
  %  Compute the new logcol and the physical destination
  %
  logcol = lcol1 + lcol2 - 1;
  prow = 2;
  pcol = 1 : logcol;
end
%
%  Storage mode = 2
%
if mod1 == 2,
  %
  %  Multiply the gain factors, and concatenate the poles.
  %  However, watch for multiples.
  %
  gain1 = pp1(2,1);
  gain2 = pp2(2,1);
  gain3 = gain1*gain2;
  if lcol1 == 1,
    if lcol2 == 1,
      %
      %  Only gain factors (both polynomials are constant)
      %
      pp3 = [ gain3 ; 0 ];
      else
      %
      % First polynomial is constant, second is not
      %
      pp3 = [ gain3 , pp2(2,2:lcol2) ; pp2(3,1:lcol2) ];
    end,
    else
    if lcol2 == 1,
      %
      % Second polynomial is constant, first is not
      %
      pp3 = [ gain3 , pp1(2,2:lcol1) ; pp1(3,1:lcol1) ];
      else
      %
      %  Concatenate both sets of roots
      %
      l1 = conj(pp1(2,2:lcol1)');
      m1 = conj(pp1(3,2:lcol1)');
      l2 = conj(pp2(2,2:lcol2)');
      m2 = conj(pp2(3,2:lcol2)');
      l3 = [ l1 ; l2 ];
      m3 = [ m1 ; m2 ];
      if mrtflag == 0,  [l3,m3] = mroot0(l3,m3);  end,
      if mrtflag == 1,  [l3,m3] = mroot1(l3,m3);  end,
      if mrtflag == 2,  [l3,m3] = mroot2(l3,m3);  end,
      pp3 = [ gain3 , conj(l3') ; pp1(3,1)+pp2(3,1) , conj(m3') ];
    end,
  end,
  %
  %  Compute the new logcol and the physical destination
  %
  [npp,logcol] = size(pp3);
  prow = 2 : 3;
  pcol = 1 : logcol;
end
%
%  Storage mode = 3
%
if mod1 == 3,
  %
  %  Multiply element-by-element all trajectory values
  %
  ppp1 = pp1(3,1:lcol1);
  ppp2 = pp2(3,1:lcol2);
  pp3 = ppp1 .* ppp2;
  %
  %  Compute the new logcol and the physical destination
  %
  logcol = lcol1;
  prow = 3;
  pcol = 1 : logcol;
end
%
%  Compute the new structural information.
%  Same for all three modes.
%
dtype = datatype(pp3);
%
%  Create the new data structure
%
dom1 = domain(pp1);
p3 = zerop(1,1,mod1,logcol,styp1,dom1);
%
%  Copy the convolved polynomial into the new data structure
%
p3(prow,pcol) = pp3;
%
%  Copy the structural information back into the new structure
%
nbr = ppack(mod1,logcol,dtype,styp1);
p3(1,1) = nbr;
%
%  The gain could be zero, therefore SQUEEZE
%
p3 = squeeze(p3);
%
pull
return
