function [s3] = multps(s1,s2)
%
%  This function connects two systems s1 and s2 specified through
%  polynomial system descriptions (modes 7 thru 9) in series.
%
%  Input Parameters:
%  -----------------
%
%    s1     := POLPAC polynomial matrix system description (modes 7 thru 9)
%    s2     := POLPAC polynomial matrix system description (modes 7 thru 9)
%
%  Output Parameters:
%  ------------------
%
%    s3     := POLPAC polynomial matrix system description (modes 7 thru 9)
%
push('MULTPS')
global debg
%
%  Check for necessary conversions first
%
[ss1,ss2] = mixmod(s1,s2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(ss1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(ss2(1,1));
%
[o1,i1] = logdim(ss1);
[o2,i2] = logdim(ss2);
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 7,
    disp('MULTPS: Error - Operates only on modes 7 thru 9'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('MULTPS: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if o2 ~= i1,
    disp('MULTPS: Error - Incompatible number of inputs/outputs'),
    abort,
  end,
end
%
%  Unpack the systems
%
[p1,q1,r1,w1] = pms2plm(ss1);
[p2,q2,r2,w2] = pms2plm(ss2);
[ns,ms] = size(ss1);
if ms == 1,
  n1 = 0;
  else
  n1 = ss1(1,2);
end
[ns,ms] = size(ss2);
if ms == 1,
  n2 = 0;
  else
  n2 = ss2(1,2);
end
%
%  Check for special cases
%
if n1 == 0,
  if n2 == 0,
    %
    %  Both systems are constant
    %
    p3 = [];
    q3 = [];
    r3 = [];
  end,
  %
  %  System 1 is constant, whereas system 2 is not
  %
  p3 = p2;
  q3 = q2;
  r3 = multp(w1,r2);
  else
  if n2 == 0,
    %
    %  System 2 is constant, whereas system 1 is not
    %
    p3 = p1;
    q3 = multp(q1,w2);
    r3 = r1;
    else
    %
    %  Series connect the subsystems
    %
    p3 = dconc(p1,p2);
    h1 = multp(q1,r2);
    h1 = minus(h1);
    p3 = put(p3,h1,1:n1,n1+1:n1+n2);
    h1 = multp(q1,w2);
    q3 = vconc(h1,q2);
    h1 = multp(w1,r2);
    r3 = hconc(r1,h1);
    w3 = multp(w1,w2);
  end,
end
%
%  Repack the system
%
s3 = plm2pms(p3,q3,r3,w3);
%
pull
return
