function [l,m,inndx] = neigal1(a,l1)
%
%  This is an experimental function to compute the eigenvalues
%  of a matrix together with their multiplicities.  The approach
%  is extremely conservative: eigenvalues are assumed multiple
%  as long as the cannot be proven to be distinct.
%
%  Input Parameter:
%  ----------------
%
%    a      := A square matrix
%    l1     := The eigenvalues as best known
%
%  Output parameters:
%  ------------------
%
%    l      := column vector of distinct eigenvalues
%    m      := column vector of their multiplicities
%    inndx  := index column vector relating the input vector
%              to the output vector
%
push('NEIGAL1')
global debg
[n,n1] = size(a);
nv = n;
%
%  We first check for dimensional compatibility
%
if debg == 3,
  if n ~= n1,
    disp('NEIGAL1: Error - The A matrix is not square'),
    abort,
  end,
end
%
%  Get the norm
%
s1 = norm(a);
inndx = zeros(n,1);
inddx2 = [ 1:n ]';
k = 0;
kk = 0;
%
%  Loop over all eigenvalues.  Whenever an eigenvalue has been
%  processed, it is added to l and m, and taken out of l1, thus:
%    l1  contains the remaining eigenvalues to be processed
%    k   denotes the number of already processed eigenvalues
%    kk  denotes the sum of their multiplicities
%
while kk < n,
  [n1,m1] = size(l1);
  %
  %  Compute the distances of all other eigenvalues to the first
  %  remaining one, and sort these distances in ascending order
  %
  d = abs(l1 - l1(1)*ones(n1,m1));
  [d,ind] = sort(d);
  %
  %  Assume all eigenvalues are the same, compute their
  %  potential resolution, and check for contradiction
  %
  i = n1;
  err = s1*eps^(1/n1);
  while err < d(i),
    i = i - 1;
    err = s1*eps^(1/i);
  end,
  %
  %  i is the multiplicity of the first remaining eigenvalue.
  %  Compute now the arithmetic mean value of all eigenvalues
  %  that are considered to be the same.
  s = l1(1);
  if i > 1,
    for ii = 2:i,
      s = s + l1(ind(ii));
    end,
    s = s/i;
  end,
  %
  %  s contains the improved eigenvalue.  Add it to the list
  %  of processed eigenvalues
  %
  k = k + 1;
  l(k) = s;
  m(k) = i;
  for ii=1:i, inndx(inddx2(ind(ii))) = k; end,
  kk = sum(m);
  %
  %  Take this eigenvalue and all its multiplicities out of
  %  the vector of remaining eigenvalues
  %
  if i < n1,
    i1 = i + 1;
    i2 = 0;
    l2 = l1;
    for ii = i1:n1,
      i2 = i2 + 1;
      l2(i2) = l1(ind(ii));
      inddx2(i2) = inddx2(ind(ii));
    end,
    l1 = l2(1:i2);
    inddx2 = inddx2(1:i2);
  end,
end
%
%  Get rid of numerical corpses
%
for i=1:k,
  if abs(round(imag(l(i))) - imag(l(i))) < 1000*eps,
    l(i) = real(l(i)) + j*round(imag(l(i)));
  end
  if abs(round(real(l(i))) - real(l(i))) < 1000*eps,
    l(i) = round(real(l(i))) + j*imag(l(i));
  end
  if abs(round(l(i)) - l(i)) < 1000*eps,
    l(i) = round(l(i));
  end,
end
l = conj(l');
m = m';
%
%  Sort the eigenvalues now
%
[l,indx] = sortc(l);
m = m(indx);
innd = inndx;
for i=1:nv,
  for i1=1:k,
    if innd(i) == indx(i1),
      inndx(i) = i1;
    end,
  end,
end
%
pull
return
