function [l,m,inndx] = neigal2(a,l1)
%
%  This is an experimental function similar to NEIGAL1 to compute the 
%  eigenvalues of a matrix together with their multiplicities.
%  The approach is different than the one of NEIGAL1.  We assume the
%  eigenvalues have a multiplicity m and compute their centroid c. 
%  We then check if (A-cI)^m has indeed a rank of n-m.  If so, the
%  eigenvalue is c with a multiplicity m.  If not, we throw away the
%  farest one from the centroid and start again with m-1 eigenvalues.
%
%  Input Parameter:
%  ----------------
%
%    a      := An at least triangular matrix
%              (Schur decomposition already performed by NEIG2)
%    l1     := The eigenvalues as best known
%              (Diagonal elements of A)
%
%  Output parameters:
%  ------------------
%
%    l      := column vector of distinct eigenvalues
%    m      := column vector of their multiplicities
%    inndx  := index column vector relating the input vector
%              to the output vector
%
push('NEIGAL2')
global debg
[n,n1] = size(a);
%
%  We first check for dimensional compatibility
%
if debg == 3,
  if n ~= n1,
    disp('NEIGAL2: Error - The A matrix is not square'),
    abort,
  end,
end
%
%  Get rid of numerical corpses
%
for i=1:n,
  if abs(round(imag(l1(i))) - imag(l1(i))) < 1000*eps,
    l1(i) = real(l1(i)) + j*round(imag(l1(i)));
  end
  if abs(round(real(l1(i))) - real(l1(i))) < 1000*eps,
    l1(i) = round(real(l1(i))) + j*imag(l1(i));
  end
  if abs(round(l1(i)) - l1(i)) < 1000*eps,
    l1(i) = round(l1(i));
  end,
  for k=1:m,
    if abs(round(imag(a(i,k))) - imag(a(i,k))) < 1000*eps,
      a(i,k) = real(a(i,k)) + j*round(imag(a(i,k)));
    end
    if abs(round(real(a(i,k))) - real(a(i,k))) < 1000*eps,
      a(i,k) = round(real(a(i,k))) + j*imag(a(i,k));
    end
    if abs(round(a(i,k)) - a(i,k)) < 1000*eps,
      a(i,k) = round(a(i,k));
    end,
  end,
end
%
%  Start the algorithm
%
inndx = zeros(n,1);
inddx2 = [ 1:n ]';
nv = n;
k = 0;
kk = 0;
%
%  Loop over all eigenvalues.  Whenever an eigenvalue has been
%  processed, it is added to l and m, and taken out of l1, thus:
%    l1  contains the remaining eigenvalues to be processed
%    k   denotes the number of already processed eigenvalues
%    kk  denotes the sum of their multiplicities
%
while kk < nv,
  [n,n1] = size(a);
  [n1,m1] = size(l1);
  %
  %  Compute the distances of all other eigenvalues to the first
  %  remaining one, and sort these distances in ascending order
  %
  d = abs(l1 - l1(1)*ones(n1,m1));
  [d,ind] = sort(d);
  %
  %  Assume all eigenvalues are the same, compute their
  %  centroid, and check for contradiction
  %
  i = n1;
  s = l1(1);
  while i > 1,
    %
    %  i is the multiplicity of the first remaining eigenvalue.
    %  Compute now the arithmetic mean value of all eigenvalues
    %  that are considered to be the same.
    %
    s = l1(1);
    for ii=2:i,
      s = s + l1(ind(ii));
    end,
    s = s/i;
    aleph = a - s*eye(n);
    %
    %  Check if the centroid is indeed an eigenvalue
    %  of multiplicity i
    %
    beth = aleph^i;
    tol = eps^(1/i);
    r = rank(beth,tol);
    flg = 0;
    tolfac = 1;
    if r < n - i,
      %
      %  This should never occur, adjust tolerance
      %
      tolsave = tol;
      svdbeth = svd(beth);
      tol = 0.5*(svdbeth(n-i) + svdbeth(n-i+1));
      tolfac = tol/tolsave;
      r = n - i;
    end,
    if r == n - i,
      %
      %  We got a potential match.  Check further for
      %  contradictions.
      %
      beth = aleph;
      kk = 1;
      r = rank(beth);
      rm1 = r;
      rdm1 = r;
      %
      %  There must be a Rank reduction, otherwise, this is not
      %  an eigenvalue at all.
      %
      if r < n,
        %
        %  Compute the Ranks and Nullities of increasing grades
        %
        while r < n - i,
          kk = kk + 1;
          beth = aleph*beth;
          tol = tolfac*eps^(1/kk);
          r = rank(beth,tol);
          rd = rm1 - r;
          %
          %  The additional Rank reduction since the last grade raise
          %  may not be larger than any previous incremental Rank
          %  reductions.
          %
          if rd > rdm1,
            flg = 1;
            else
            rm1 = r;
            rdm1 = rd;
          end,
        end,
        else
        flg = 1;
      end,
      else
      flg = 1;
    end,
    %
    %  if FLG is still 0, we really got a match (no more contradictions)
    %
    if flg == 0, break, end,
    s = l1(1);
    i = i - 1;
  end,
  %
  %  s contains the improved eigenvalue.  Add it to the list
  %  of processed eigenvalues
  %
  k = k + 1;
  l(k) = s;
  m(k) = i;
  for ii=1:i, inndx(inddx2(ind(ii))) = k; end,
  kk = sum(m);
  %
  %  Take this eigenvalue and all its multiplicities out of
  %  the vector of remaining eigenvalues as well as the matrix
  %
  if i < n1,
    t = zeros(n1);
    i1 = i+1;
    i2 = 0;
    l2 = l1;
    inddx3 = inddx2;
    for ii = i1:n1,
      i2 = i2+1;
      l2(i2) = l1(ind(ii));
      inddx3(i2) = inddx2(ind(ii));
      t(i2,ind(ii)) = 1;
    end,
    i3 = i2;
    for ii = 1:i,
      i3 = i3+1;
      t(i3,ind(ii)) = 1;
    end,
    l1 = l2(1:i2);
    inddx2 = inddx3(1:i2);
    a = t*a/t;
    a = a(1:n1-i,1:n1-i);
  end,
end
%
%  Get rid of numerical corpses
%
for i=1:k,
  if abs(round(imag(l(i))) - imag(l(i))) < 1000*eps,
    l(i) = real(l(i)) + j*round(imag(l(i)));
  end
  if abs(round(real(l(i))) - real(l(i))) < 1000*eps,
    l(i) = round(real(l(i))) + j*imag(l(i));
  end
  if abs(round(l(i)) - l(i)) < 1000*eps,
    l(i) = round(l(i));
  end,
end
l = conj(l');
m = m';
%
%  Sort the eigenvalues now
%
[l,indx] = sortc(l);
m = m(indx);
innd = inndx;
for i=1:nv, 
  for i1=1:k,
    if innd(i) == indx(i1),
      inndx(i) = i1;
    end,
  end,
end
%
pull
return
