function [pm] = newpoly(p,stype)
%
%  This function converts a single Matlab polynomial into a POLPAC
%  polynomial.
%
%  If the input vector is a row vector, POLPAC assumes that the
%  elements of this vector represent the coefficients of the
%  polynomial in descending order.
%
%  If the input vector is a column vector, POLPAC assumes that the
%  elements of this vector represent the roots of the polynomial
%  in arbitrary order.  In that case, the gain factor is arbitrarily
%  assigned a value of 1.
%
%  If the input is a scalar, we got an ambiguity.  In that case, the
%  scalar is assumed to represent a constant polynomial (no roots).
%  This implies that also first order polynomials MUST be entered
%  through their coefficients rather than through their roots.
%
%  Input Parameters:
%  -----------------
%
%    p      := Matlab polynomial
%                row vector:     coefficients in descending order
%                column vector:  roots in arbitrary order (more than 1)
%    stype  := System type
%
%  Output Parameter:
%  -----------------
%
%    pm     := Polynomial matrix (POLPAC mode = 1 or 2)
%                mode = 1 :  p was specified through coefficients
%                mode = 2 :  p was specified through roots (gain = 1)
%
push('NEWPOLY')
global debg
global mrtflag
%
%  We first check whether STYPE is legal.
%
if debg == 3,
  if stype ~= 0,
    if stype ~= 1,
      disp('NEWPOLY: Error - STYPE must be 0 or 1'),
      stype,
      abort,
    end,
  end,
end
%
%  Decide mode next
%
[n1,n2] = size(p);
mode = -1;
if n2 == 1,
  mode = 2;
end
if n1 == 1,
  mode = 1;
end
if mode == -1,
  disp('NEWPOLY: Error - Operates on vectors only'),
  p,
  abort,
end
%
%  Generate the structural information
%
logcol = n1*n2;
dtype = datatype(p);
nbr = ppack(mode,logcol,dtype,stype);
%
%  Treat each mode separately
%
%  First case: mode = 1
%
if mode == 1,
  pm = zeros(2,logcol);
  pm(1,1) = nbr;
  pm(2,:) = p(logcol:-1:1);
end
%
%  Second case:  mode = 2
%
if mode == 2,
  mlt1 = ones(logcol,1);
  if mrtflag == 0,  [lamb,mlt] = mroot0(p,mlt1);  end,
  if mrtflag == 1,  [lamb,mlt] = mroot1(p,mlt1);  end,
  if mrtflag == 2,  [lamb,mlt] = mroot2(p,mlt1);  end,
  lamb = conj(lamb');
  mlt = mlt';
  [n,m] = size(lamb);
  logcol = m + 1;
  pm = zeros(3,logcol);
  nbr = ppack(mode,logcol,dtype,stype);
  pm(1,1) = nbr;
  pm(2,1) = 1;
  pm(3,1) = sum(mlt);
  pm(2,2:logcol) = lamb;
  pm(3,2:logcol) = mlt;
end
%
%  Squeeze to minimal LOGCOL value
%
pm = squeeze(pm);
%
pull
return
