function [s] = newsyst(a,b,c,d,stype)
%
%  This function converts the four system matrices A, B, C, and D
%  into a POLPAC system in time domain representation (storage mode 0).
%
%  Given a continuous time system (STYPE=0):
%
%    x. = A*x + B*u
%    y  = C*x + D*u
%
%  or its discrete time equivalent (STYPE=1):
%
%    x(k+1) = A*x(k) + B*u(k)
%    y(k)   = C*x(k) + D*u(k)
%
%  The four system matrices A, B, C, and D can conveniently be concatenated
%  to one total system matrix:
%
%    S = [ A , B ; C , D ]
%
%  Input Parameters:
%  -----------------
%
%    a      := State matrix
%    b      := Input matrix
%    c      := Output matrix
%    d      := Input-output matrix
%    stype  := System type
%
%  Output Parameter:
%  -----------------
%
%    s      := System matrix (POLPAC mode = 0)
%
push('NEWSYST')
global debg
%
%  We first check whether STYPE is legal.
%
if debg == 3,
  if stype ~= 0,
    if stype ~= 1,
      disp('NEWSYST: Error - STYPE must be 0 or 1'),
      stype,
      abort,
    end,
  end,
end
%
%  We then check for dimensional compatability
%
if debg >= 2,
  [n,n1] = size(a);
  if n ~= n1,
    disp('NEWSYST: Error - The A matrix is not square'),
    abort,
  end,
  [n1,m] = size(b);
  if n ~= n1,
    disp('NEWSYST: Error - A and B matrices are incompatible'),
    abort,
  end,
  [p,n1] = size(c);
  if n ~= n1,
    disp('NEWSYST: Error - A and C matrices are incompatible'),
    abort,
  end,
  [p1,m1] = size(d);
  if n > 0,
    if m ~= m1,
      disp('NEWSYST: Error - B and D matrices are incompatible'),
      abort,
    end,
    if p ~= p1,
      disp('NEWSYST: Error - C and D matrices are incompatible'),
      abort,
    end,
  end,
end
%
%  Now, concatenate A, B, C and D into S.
%
[n,n1] = size(a);
if n > 0,
  s = [a,b;c,d];
  else
  s = d;
end
%
%  Retrieve the TYPE of this matrix
%
dtype = datatype(s);
%
%  Now pack the structural information
%
mode = 0;
logcol = 1;
nbr = ppack(mode,logcol,dtype,stype);
%
%  Concatenate the top row
%
[ns,ms] = size(s);
[n,m] = size(b);
s = [ zeros(1,ms) ; s ];
s(1,1) = nbr;
if n > 0,
  s(1,2) = n;
end
%
pull
return
