function [n] = nnorm(p,tp)
%
%  Find the norm of a POLPAC data structure.
%
%  Input Parameters:
%  -----------------
%
%    p     :=  Any POLPAC data structure (all modes)
%    tp    :=  The type of mode to be computed.
%              default:  tp = 2
%
%  Output Parameter:
%  -----------------
%
%    n     :=  The norm of p.
%
%  Explanations:
%  -------------
%
%  P can be either a signal (mode = 3, real-valued domain), or a system
%  modes 0, 4, 5, 6, 7, 8, 9.  If P is in modes 1, 2, or 3, and isn't a
%  signal, apply polynomial matrix norms.
%
%  If P is a signal, the following options are available:
%
%    tp = 1     :  Use l1-norm:  n = sum(abs(pi))        over the stored values
%    tp = 2     :  Use l2-norm:  n = sqrt(sum(abs(pi)^2))            " 
%    tp = p     :  Use lp-norm:  n = (sum(abs(pi)^p))^(1/p))         "
%    tp = inf   :  Use loo-norm: n = max(abs(pi))                    "
%    tp = 'inf' :  Same as above
%    tp = 'pow' :  nnorm(P,2)/(tmax - tmin)
%    tp = 'spc' :  Calculate the spectrum.
%                  Then compute: n = sqrt(||Suu||oo)
%
%  If P is a signal matrix:
%
%    tp = 2     :  Use l2-norm:  n = sqrt(trace(pi'*pi)) over the stored values
%    tp = inf   :  Use loo-norm: n = max(max(svd(p)))                "
%    tp = 'inf' :  For each domain value, t0, the matrix of p(t0) is extracted.
%                  max(svd(p)) is the largest singular value of p(t0).
%                  The outer max operator is then taken over all time values.
%    tp = 'pow' :  norm( [ nnorm(pij,'pow') ] , 1)
%    tp = 'spc' :  norm( [ nnorm(pij,'inf') ] , inf)
%
%  If P is a stable system, all system modes except mode = 6 with strictly
%  imaginary domain:
%
%    tp = 2     :  Use L2-norm:
%                  Convert system to state-space description.
%                  Get minimal realization.
%                  Compute the controllability Gramian, Gc.
%                  Then compute: n = sqrt(trace(C*Gc*C')
%    tp = inf   :  Use Hoo-norm:
%    tp = 'inf' :  Solve a series of Riccati equations, until H no longer
%                  has eigenvalues on the imaginary axis.
%    tp = 'hnk' :  Use Hankel-norm:
%                  n = sqrt(max(eig(Gc*Go)))
%                  
%  If P is a stable system with mode = 6 and a strictly imaginary domain:
%
%    tp = inf   :  Use Hoo-norm:
%    tp = 'inf' :  Calculate series of matrices for each domain value.
%                  Then compute: n = max(max(svd(abs(p(jw)))))
%                  where max(svd(.)) is the largest singular value, and the
%                  outer max operator is taken over all stored frequency values.
%    tp = 'mu'  :  Use mu-norm:
%                  n = inf(||D*P/D||oo)         over all nonsingular D matrices
%                  
%  If P is a polynomial matrix:
%
%    There is no physical interpretation of the norm of a polynomial.
%    However, it is useful to define a meaning, because this can be used to
%    assess the "magnitude" of the data values.  For this reason, the norm
%    of a polynomial is defined as follows:
%
%      mode = 1 :  Norm of vector of coefficients.
%      mode = 2 :  Norm of vector consisting of [ gain , l1 (m1 times) ,
%                  l2 (m2 times) , ... ].
%      mode = 3 :  Norm of vector of polynomial values at given domain values.
%
%    The absolute value of a polynomial matrix is the matrix of the absolute
%    values of the individual polynomials.
%
%    tp = 1     :  Use 1-vector/matrix norm.
%    tp = 2     :  Use 2-vector/matrix norm.
%    tp = p     :  Use p-vector/matrix norm.
%    tp = inf   :  Use oo-vector/matrix norm.
%    tp = 'inf' :  Same as above.
%
push('NNORM');
global debg
%
[mod,logcol,dtyp,styp] = unpack(p(1,1));
[lrow,lcol] = logdim(p);
%
%  Treat signals first
%
if mod == 3 & abs(dom - real(dom)) < 1000*eps,
  if ~isstr(tp),
    if tp == 1,
      if debg == 3,
        if lrow*lcol ~= 1,
          disp('NNORM: Error - 1-norm only implemented for single signals'),
          abort,
        end,
      end,
      n = norm(p(3,1:logcol),1);
      pull,
      return,
    end,
    if tp == 2,
      if lrow*lcol == 1,
        n = norm(p(3,1:logcol),2);
        else
        pp = tracep(multp(transpose(p),p));
        n = norm(pp(3,1:logcol),2);
      end,
      pull,
      return,
    end,
    if tp == inf,
      if lrow*lcol == 1,
        n = norm(p(3,1:logcol),inf);
        else
        pp = zeros(1,logcol);
        for i=1:logcol,
          a = p(3:lrow+2,i:logcol:(lcol-1)*logcol+i);
          s = svd(a);
          pp(i) = s(1);
        end,
        n = norm(pp,inf);
      end,
      pull,
      return,
    end,
  end,
  if isstr(tp),
    if strcmp(tp,'inf'),
      if lrow*lcol == 1,
        n = norm(p(3,1:logcol),inf);
        else
        pp = zeros(1,logcol);
        for i=1:logcol,
          a = p(3:lrow+2,i:logcol:(lcol-1)*logcol+i);
          s = svd(a);
          pp(i) = s(1);
        end,
        n = norm(pp,inf);
      end,
      pull,
      return,
    end,
    if strcmp(tp,'pow'),
      if lrow*lcol == 1,
        ruu = xcorr(p(3,1:logcol));
        n = sqrt(ruu(round(logcol)));
        else
        n = 0;
        for i=1:min([lrow,lcol]),
          ruu = xcorr(p(i+2,(i-1)*logcol+1:i*logcol));
          n = n + ruu(round(logcol));
        end,
        n = sqrt(n);
      pull,
      return,
    end,
    if strcmp(tp,'spc'),
      if lrow*lcol == 1,
        suu = psd(p(3,1:logcol));
        n = sqrt(max(suu));
        else
        suu = psd(p(3,1:logcol));
        [nn,m] = size(suu);
        logcol2 = nn*m;
        dom2 = [ 1:logcol2 ];
        suu = zerop(lrow,lcol,3,logcol2,styp,dom2);
        for i=1:lrow,
          for k=1:lcol,
            suuik = psd(p(i+2,(i-1)*logcol+1:i*logcol));
            suu(i+2,(i-1)*logcol2+1:i*logcol2) = suuik;
          end,
        end,
        n = nnorm(suu,inf);
        n = sqrt(n);
      pull,
      return,
    end,
    %
    %  If we still have a string, and none of the above, then there must
    %  be an error.
    %
    disp('NNORM: Error - Specified class not compatible with signal norms'),
    abort,
    return,
  end,
  %
  %  Must be a p-norm.
  %
  if lrow*lcol == 1,
    n = norm(p(3,1:logcol),tp);
    pull,
    return,
    else
    disp('NNORM: Error - p-norm undefined for signal matrices'),
    abort,
    return,
  end,
end
%
%  Now check polynomials
%
if mod > 0,
  if mod < 4,
    %
    %  Loop over each polynomial
    %
    a = zeros(lrow,lcol);
    for i=1:lrow,
      for k=1:lcol,
        pik = gget(p,i,k);
        [mod,lcolik,dtyp,styp] = unpack(pik(1,1));
        if mod == 1,
          %
          %  Mode = 1
          %
          v = pik(2,1:lcolik);
          else if mod == 2,
            %
            %  Mode = 2
            %
            n = pik(3,1);
            v = zeros(1,n+1);
            v(1) = pik(2,1);
            ll = 1;
            if n > 0,
              for kk=2:lcolik,
                mi = pik(3,kk);
                for l = 1:mi,
                  ll = ll + 1;
                  v(ll) = pik(2,kk);
                end,
              end,
            end,
            else
            %
            %  Mode = 3
            %
            v = pik(3,1:lcolik);
          end,
        end,
        a(i,k) = norm(v,tp);
      end,
    end,
    n = norm(a,tp);
    pull,
    return,
  end,
end
%
%  Okay.  Must be a system!
%  Check first for special case of mode = 6 with imaginary domain.
%
if ~isstr(tp),
  if tp == inf,
    if mod == 6,
      if abs(real(dom)) < 1000*eps,
        pp = zeros(1:logcol);
        if lrow*lcol == 1,
          for i=1:logcol,
            pp(i) = abs(p(3,i))/abs(p(4,i));
          end,
          else
          for i=1:logcol,
            a = p(3:lrow+2,i:logcol:(lcol-1)*logcol+i);
            b = p(lrow+3:2*lrow+2,i:logcol:(lcol-1)*logcol+i);
            g = a ./ b;
            s = svd(g);
            pp(i) = s(1);
          end,
        end,
        n = max(pp);
        pull,
        return,
      end,
    end,
  end,
end
if isstr(tp),
  if strcmp(tp,'inf'),
    if mod == 6,
      if abs(real(dom)) < 1000*eps,
        pp = zeros(1:logcol);
        if lrow*lcol == 1,
          for i=1:logcol,
            pp(i) = abs(p(3,i))/abs(p(4,i));
          end,
          else
          for i=1:logcol,
            a = p(3:lrow+2,i:logcol:(lcol-1)*logcol+i);
            b = p(lrow+3:2*lrow+2,i:logcol:(lcol-1)*logcol+i);
            g = a ./ b;
            s = svd(g);
            pp(i) = s(1);
          end,
        end,
        n = max(pp);
        pull,
        return,
      end,
    end,
  end,
  if strcmp(tp,'mu'),
    if mod == 6,
      if abs(real(dom)) < 1000*eps,
        pp = zeros(1:logcol);
        if lrow*lcol == 1,
          for i=1:logcol,
            pp(i) = abs(p(3,i))/abs(p(4,i));
          end,
          else
          for i=1:logcol,
            a = p(3:lrow+2,i:logcol:(lcol-1)*logcol+i);
            b = p(lrow+3:2*lrow+2,i:logcol:(lcol-1)*logcol+i);
            g = a ./ b;
            pp(i) = munorm(g);
          end,
        end,
        n = max(pp);
        pull,
        return,
      end,
    end,
  end,
end
%
%  Now check for necessary conversions
%
mcl = round((mod + 1)/3);
if mcl == 2,
  s = tfm2ss(p);
  else if mcl == 3,
  s = pms2plm(p);
end
[mod2,logcol2,dtyp2,styp2] = unpack(s(1,1));
[a,b,c,d] = oldsyst(s);
[a,b,c,d] = minreal(a,b,c,d);
%
%  Check whether system is stable
%
lmax = max(real(eig(a)));
if lmax >= 0,
  disp('NNORM: Error - System norms are only defined for stable systems'),
  abort,
  return,
end
if ~isstr(tp),
  if tp == 2,
    if norm(d,inf) > 1000*eps,
      n = inf;
      else
      Gc = gram(a,b);
      n = sqrt(trace(c*Gc*c'));
    end,
    pull,
    return,
  end,
  if tp == inf,
    %
    %  Get Hankel singular values to estimate gamma
    %
    Gc = gram(a,b);
    Go = gram(a',c');
    sH = sqrt(eig(Gc*Go));
    s = svd(d);
    gmin = max([sH(1),s(1)]);
    gmax = s(1) + 2*sum(sH);
    while gmax - gmin > 1.0e-6,
      gamma = 0.5*(gmin + gmax);
      dd = d'*d;
      [nn,m] = size(dd);
      R = gamma*gamma*eye(m) - dd;
      br = b/R;
      h11 = a + br*d'*c;
      h12 = br*b';
      drd = d/R*d';
      [nn,m] = size(drd);
      h21 = -c'*(eye(m) + drd)*c;
      h22 = -h11';
      H = [ h11 , h12 ; h21 , h22 ];
      l = eig(H);
      l = abs(real(l));
      if min(l) > 1000*eps,
        gmax = gamma;
        else
        gmin = gamma;
      end,
    end,
    n = gamma;
    pull,
    return,
  end,
  %
  %  Must be an error.
  %
  disp('NNORM: Error - p-norms are undefined for systems'),
  abort,
  return,
end
if isstr(tp),
  if strcmp(tp,'inf'),
    %
    %  Get Hankel singular values to estimate gamma
    %
    Gc = gram(a,b);
    Go = gram(a',c');
    sH = sqrt(eig(Gc*Go));
    s = svd(d);
    gmin = max([sH(1),s(1)]);
    gmax = s(1) + 2*sum(sH);
    while gmax - gmin > 1.0e-6,
      gamma = 0.5*(gmin + gmax);
      dd = d'*d;
      [nn,m] = size(dd);
      R = gamma*gamma*eye(m) - dd;
      br = b/R;
      h11 = a + br*d'*c;
      h12 = br*b';
      drd = d/R*d';
      [nn,m] = size(drd);
      h21 = -c'*(eye(m) + drd)*c;
      h22 = -h11';
      H = [ h11 , h12 ; h21 , h22 ];
      l = eig(H);
      l = abs(real(l));
      if min(l) > 1000*eps,
        gmax = gamma;
        else
        gmin = gamma;
      end,
    end,
    n = gamma;
    pull,
    return,
  end,
  if strcmp(tp,'hnk'),
    Gc = gram(a,b);
    Go = gram(a',c');
    sH = sqrt(eig(Gc*Go));
    n = sH(1);
    pull,
    return,
  end,
  %
  %  Must be an error.
  %
  disp('NNORM: Error - 2nd argument is an unrecognized string'),
  abort,
  return,
end
%
return
