function [ps] = plm2pms(p,q,r,w)
%
%  This function converts four polynomial matrices (stored in
%  storage modes 1 thru 3) into a single polynomial matrix
%  system description (stored in the corresponding storage
%  mode 7 thru 9).
%
%  Input Parameters:
%  -----------------
%
%    p      := System polynomial matrix (mode = 1 thru 3)
%    q      := Input polynomial matrix (mode = 1 thru 3)
%    r      := Output polynomial matrix (mode = 1 thru 3)
%    w      := Direct coupling polynomial matrix (mode = 1 thru 3)
%
%  Output Parameter:
%  -----------------
%
%    ps     := Polynomial matrix system description (mode = 7 thru 9)
%
push('PLM2PMS')
global debg
%
%  Start by using mixmod
%
[p,q] = mixmod(p,q);
[p,r] = mixmod(p,r);
[p,w] = mixmod(p,w);
%
%  Unpack the structural information
%
[mm,ll,dd,ss] = unpack(p(1,1));
m(1) = mm; l(1) = ll; d(1) = dd; s(1) = ss;
[mm,ll,dd,ss] = unpack(q(1,1));
m(2) = mm; l(2) = ll; d(2) = dd; s(2) = ss;
[mm,ll,dd,ss] = unpack(r(1,1));
m(3) = mm; l(3) = ll; d(3) = dd; s(3) = ss;
[mm,ll,dd,ss] = unpack(w(1,1));
m(4) = mm; l(4) = ll; d(4) = dd; s(4) = ss;
%
%  Check for MODE and STYPE consistency
%
if debg == 3,
  for i=1:4,
    if m(i) < 1 | m(i) > 3,
      disp('PLM2PMS: Error - Inputs must be polynomial matrices.'),
      abort,
    end,
  end,
  for i=2:4,
    if s(i) ~= s(1),
      t1 = 'PLM2PMS: Error - The STYPE of each polynomial matrix';
      t2 = '                 must be the same                   ';
      txt = [ t1 ; t2 ];
      disp(txt),
      abort,
    end,
  end,
end
%
%  Check for correct number of logical rows and columns
%
if debg >= 2,
  [q1,q2] = logdim(p);
  if q1 ~= q2,
    disp('PLM2PMS: Error - The P polynomial matrix is not logically square'),
    abort,
  end,
  [q2,mx] = logdim(q);
  if q2 ~= q1,
    t1 = 'PLM2PMS: Error - P and Q polynomial matrices have a';
    t2 = '                 different number of logical rows. ';
    txt = [ t1 ; t2 ];
    disp(txt),
    abort,
  end,
  [p1,q2] = logdim(r);
  if q2 ~= q1,
    t1 = 'PLM2PMS: Error - P and R polynomial matrices have a  ';
    t2 = '                 different number of logical columns.';
    txt = [ t1 ; t2 ];
    disp(txt),
    abort,
  end,
  [p2,m1x] = logdim(w);
  if mx ~= m1x,
    t1 = 'PLM2PMS: Error - Q and W polynomial matrices have a  ';
    t2 = '                 different number of logical columns.';
    txt = [ t1 ; t2 ];
    disp(txt),
    abort,
  end,
  if p2 ~= p1,
    t1 = 'PLM2PMS: Error - R and W polynomial matrices have a';
    t2 = '                 different number of logical rows. ';
    txt = [ t1 ; t2 ];
    disp(txt),
    abort,
  end,
end
%
%  Do the conversion now
%
if mm == 1,
  ps = c17(p,q,r,w);
  else if mm == 2,
    ps = c28(p,q,r,w);
    else
    ps = c39(p,q,r,w);
  end,
end
%
pull
return
