function [prow,pcol] = pmap(pm,lrow,lcol)
%
%  This routine maps the logical row- and columnvectors LROW and LCOL
%  into physical row- and columnvectors PROW and PCOL.
%
%  Input Parameters:
%  -----------------
%
%    pm     : Given POLPAC matrix (in storage modes 1 thru 6).
%    lrow   : Logical row vector.
%    lcol   : Logical column vector.
%
%  Output Parameters:
%  ------------------
%
%    prow   : Physical row vector.
%    pcol   : Physical column vector.
%
%  Supported modes = 1,2,3,4,5,6
%   
push('PMAP')
global debg
%
%  Unpack the storage quantities
%
[mode,logcol,dtype,stype] = unpack(pm(1,1));
[nlrow,nlcol] = logdim(pm);
[nrow,mrow] = size(lrow);
nmrow = nrow*mrow;
[ncol,mcol] = size(lcol);
nmcol = ncol*mcol;
lsrow = lrow;
lscol = lcol;
%
%  Check for consistency
%
if debg >= 2,
  if mode < 1,
    disp('PMAP: Error - Operates on modes 1 thru 6 only'),
    mode,
    abort,
  end,
  if mode > 6,
    disp('PMAP: Error - Operates on modes 1 thru 6 only'),
    mode,
    abort,
  end,
  if nrow~=1,
    lsrow = lrow';
    if mrow~=1,
      disp('PMAP: Error - LROW must be either a scalar or a vector'),
      abort,
    end,
  end,
  if ncol~=1,
    lscol = lcol';
    if mcol~=1,
      disp('PMAP: Error - LCOL must be either a scalar or a vector'),
      abort,
    end,
  end,
  [ls,indx] = sort(lsrow);
  if norm(ls-lsrow,'inf') > 10*eps,
    disp('PMAP: Error - LROW elements must be in ascending order'),
    lrow,
    abort,
  end,
  if ls(1) < 1,
    disp('PMAP: Error - LROW element is outside logical boundaries'),
    ls(1),
    abort,
  end,
  if ls(nmrow) > nlrow,
    disp('PMAP: Error - LROW element is outside logical boundaries'),
    ls(nrow),
    abort,
  end,
  [ls,indx] = sort(lscol);
  if norm(ls-lscol,'inf') > 10*eps,
    disp('PMAP: Error - LCOL elements must be in ascending order'),
    lcol,
    abort,
  end,
  if ls(1) < 1,
    disp('PMAP: Error - LCOL element is outside logical boundaries'),
    ls(1),
    abort,
  end,
  if ls(nmcol) > nlcol,
    disp('PMAP: Error - LCOL element is outside logical boundaries'),
    ls(ncol),
    abort,
  end,
end
%
%  Do the actual mapping now
%
%  Column mapping
%
for i=1:nmcol,
  pcol(1,(i-1)*logcol+1:i*logcol) = (lscol(i)-1)*logcol+1:lscol(i)*logcol;
end
%
%  Row mapping
%
i1 = ones(1,nmrow);
if mode == 1,
  prow = lsrow + i1;
  pull,
  return,
end
if mode == 2,
  prow(1,1:2:2*nmrow-1) = 2*lsrow; 
  prow(1,2:2:2*nmrow) = 2*lsrow + i1;
  pull,
  return,
end
if mode == 3,
  prow = lsrow + 2*i1;
  pull,
  return,
end
if mode == 4,
  prow = lsrow + i1;
  prow(1,nmrow+1:2*nmrow) = lsrow + (nlrow + 1)*i1;
  pull,
  return,
end
if mode == 5,
  prow(1,1:2:2*nmrow-1) = 2*lsrow;
  prow(1,2:2:2*nmrow) = 2*lsrow + i1;
  prow(1,2*nmrow+1:2:4*nmrow-1) = 2*lsrow + 2*nlrow*i1;
  prow(1,2*nmrow+2:2:4*nmrow) = 2*lsrow + (2*nlrow + 1)*i1;
  pull,
  return,
end
if mode == 6,
  prow = lsrow + 2*i1;
  prow(nmrow+1:2*nmrow) = lsrow + (nlrow + 2)*i1;
end
%
pull
return
