function [nbr] = ppack(mode,logcol,dtype,stype)
%
%  This function PACKs a number of data structure identifiers into
%  one integer number.
%
%  Input Parameters:
%  -----------------
%
%    mode   ::= storage mode of the data structure with the following
%               meaning:
%                 mode = 0 :  System representation in time domain
%                 mode = 1 :  Unfactorized polynomial matrix (coefficients)
%                 mode = 2 :  Factorized polynomial matrix (roots)
%                 mode = 3 :  Trajectory polynomial matrix (supporting values)
%                 mode = 4 :  Unfactorized transfer function matrix
%                 mode = 5 :  Factorized transfer function matrix
%                 mode = 6 :  Trajectory transfer function matrix
%                 mode = 7 :  Unfactorized polynomial matrix system
%                 mode = 8 :  Factorized polynomial matrix system
%                 mode = 9 :  Trajectory polynomial matrix system
%    logcol ::= number of physical column making up one logical
%               column (positive integer)
%    dtype  ::= type of data matrix:
%                 dtype = 1 :  All elements are integer
%                 dtype = 2 :  Some elements are real, but none are complex
%                 dtype = 3 :  Some elements are complex
%    stype  ::= type of system:
%                 stype = 0 :  Continuous time system
%                 stype = 1 :  Discrete time system
%
%  Output Parameters:
%  ------------------
%
%    nbr    ::= coded positive integer containing the information of all
%               input parameters plus the version number (see below)
%
%  POLPAC operates on one additional (internal) parameter called version.
%  This parameter will guarantee upwards compatibility of data bases
%  between any two subsequent versions of POLPAC.
%
push('PPACK')
global version
global debg
%
%  Set data base version number to 1.0
%
version = 1;
%
%  if debg=3, check for consistency
%
if debg == 3,
  if logcol < 1,
    disp('PACK: Error - logcol must be a positive integer'),
    logcol,
    abort,
  end,
  if abs(logcol-round(logcol)) > 10*eps,
    disp('PACK: Error - logcol must be integer'),
    logcol,
    abort,
  end,
  if mode < 0,
    disp('PACK: Error - mode may not be negative'),
    mode,
    abort,
  end,
  if abs(mode-round(mode)) > 10*eps,
    disp('PACK: Error - mode must be integer'),
    mode,
    abort,
  end,
  if mode > 9,
    disp('PACK: Error - mode must be smaller than 10'),
    mode,
    abort,
  end,
  if dtype < 1,
    disp('PACK: Error - dtype must be a positive integer'),
    dtype,
    abort,
  end,
  if abs(dtype-round(dtype)) > 10*eps,
    disp('PACK: Error - dtype must be integer'),
    dtype,
    abort,
  end,
  if dtype > 3,
    disp('PACK: Error - dtype must be smaller than 4'),
    dtype,
    abort,
  end,
  if stype < 0,
    disp('PACK: Error - stype must not be negative'),
    stype,
    abort,
  end,
  if abs(stype-round(stype)) > 10*eps,
    disp('PACK: Error - stype must be integer'),
    stype,
    abort,
  end,
  if stype > 1,
    disp('PACK: Error - stype must be smaller than 2'),
    stype,
    abort,
  end,
end
%
%  Do the actual PACKing now
%
nbr = 1000*logcol + 100*mode + 30*stype + 10*dtype + version;
%
pull
return
