function [pm3] = put(pm1,pm2,lrow,lcol)
%
%  This routines first copies data structure PM1 into data structure PM3,
%  then copies the substructure PM2 into the logical positions LROW and
%  LCOL of PM3.
%
%  LROW and LCOL contain vectors of numbers of logical rows(columns)
%  into which PM2 is to be copied.
%
%  The size of PM2 must coinside with the lengths of vectors LROW and
%  LCOL.
%
%  However, it is possible to specify logical elements in LROW
%  and LCOL that are logically outside PM1.  In that case, PM1
%  is not simply copied into PM2, but first, PM2 is created as a
%  zero data structure of required size (by use of function ZEROP), and
%  only then, PM1 is copied into PM2.
%
%  Also LOGCOL will be automatically adjusted, and a SQUEEZE operation
%  is automatically performed in the end.
%
%  PM1 and PM2 must be of same mode.  PM3 will be generated again using
%  the same storage mode.
%
%  In storage modes 3 and 6, the domains must be the same.
%
%  PUT operates on modes 1 thru 6 only.
%
%  Input Parameters:
%  -----------------
%
%    pm1    : Given POLPAC base data structure (modes 1 thru 6)
%    pm2    : Given POLPAC substructure (same mode as pm1)
%    lrow   : vector containing the numbers of those rows that
%             are to be extracted (in ascending order)
%    lcol   : vector containing the numbers of those columns that
%             are to be extracted (in ascending order)
%
%  Output Parameter:
%  -----------------
%
%    pm3    : Merged POLPAC data structure (same mode as PM1 and PM2)
%
push('PUT')
global debg
%
%  Call MIXMOD first
%
[pp1,pp2] = mixmod(pm1,pm2);
%
%  Start by unpacking the structural information
%
[mod1,logc1,dtyp1,styp1] = unpack(pp1(1,1));
[mod2,logc2,dtyp2,styp2] = unpack(pp2(1,1));
logcol = max([logc1,logc2]);
%
%  Check for consistency first
%
if debg >= 2,
  if styp1 ~= styp2,
    disp('PUT: Error - Incompatible system types'),
    abort,
  end,
end
%
%  Get the logical dimensions
%
[nrow1,ncol1] = logdim(pp1);
[nrow2,ncol2] = logdim(pp2);
[nr,mr] = size(lrow);
nr = nr*mr;
[nc,mc] = size(lcol);
nc = nc*mc;
if debg >=2,
  if nr ~= nrow2,
    disp('PUT: Error - LROW incompatible with PM2'),
    abort,
  end,
  if nc ~= ncol2,
    disp('PUT: Error - LCOL incompatible with PM2'),
    abort,
  end,
end
%
%  Expand either PM1 or PM2 if necessary (incompatible LOGCOL)
%
pm1n = pp1;
if logcol > logc1,
  pm1n = expand(pp1,logcol);
end
pm2n = pp2;
if logcol > logc2,
  pm2n = expand(pp2,logcol);
end
%
%  Check whether the logical dimensions need to be expanded
%
check = 0;
nrow3 = nrow1;
if lrow(nrow2) > nrow1,
  check = 1;
  nrow3 = lrow(nrow2);
end
ncol3 = ncol1;
if lcol(ncol2) > ncol1,
  check = 1;
  ncol3 = lcol(ncol2);
end
if check == 0,
  %
  %  New size equals old size.  Simply copy PM1 into PM3
  %
  pm3 = pm1n;
end
if check == 1,
  %
  %  New size larger than old size.  Create first a new
  %  data structure (pm3), then copy pm1 into it.
  %
  dom = domain(pp1);
  pm3 = zerop(nrow3,ncol3,mod1,logcol,styp1,dom);
  [prow1,pcol1] = pmap(pm1n,1:nrow1,1:ncol1);
  [prow3,pcol3] = pmap(pm3,1:nrow1,1:ncol1);
  pm3(prow3,pcol3) = pm1n(prow1,pcol1);
end
%
%  Now, copy PM2 into PM3
%
[prow2,pcol2] = pmap(pm2n,1:nrow2,1:ncol2);
[prow3,pcol3] = pmap(pm3,lrow,lcol);
pm3(prow3,pcol3) = pm2n(prow2,pcol2);
%
%  Check whether the data type is still okay
%
dtype = datatype(pm3);
if dtype > 1,
  [mode,logcol,dtyp3,stype] = unpack(pm3(1,1));
  nbr = ppack(mode,logcol,dtype,stype);
  pm3(1,1) = nbr;
end
%
%  SQUEEZE for the sake of it
%
pm3 = squeeze(pm3);
%
pull
return
