function [q,r] = qrp(pm)
%
%  This function performs an upper triangularization of a polynomial
%  matrix: PM = Q*R.
%
%  Input Parameter:
%  ----------------
%
%    pm     :=  The polynomial matrix to be triangularized
%               (modes 1, 2, or 3 only)
%
%  Output Parameters:
%  ------------------
%
%    q      :=  A unimodular polynomial matrix
%    r      :=  An uppertriangular polynomial matrix
%
push('QRP');
global debg
%
%  Start by unpacking the structural information.
%
[mode,logcol,dtype,stype] = unpack(pm(1,1));
%
%  Check for consistency.
%
if debg == 3,
  if mode < 1,
    disp('QRP: Error - Operates on modes 1 thru 3 only'),
    abort,
  end,
  if mode > 3,
    disp('QRP: Error - Operates on modes 1 thru 3 only'),
    abort,
  end,
end
%
%  Get the logical dimensions of PM.
%
[lrow,lcol] = logdim(pm);
%
%  Get the domain information.
%
dom = domain(pm);
%
%  Initialize Q and R.
%
q = eyep(lrow,lrow,mode,logcol,stype,dom);
r = pm;
%
%  Loop over all significant columns.
%
imax = min([lrow,lcol]);
for i=1:imax,
  %
  %  Make all elements below diagonal of column i zero.
  %
  if i < lrow,
    [q,r] = qrpi(q,r,i);
  end,
  %
  %  Check whether highest order polynomial is now on the diagonal.
  %
  pri = gget(r,1:lrow,i);
  ori = order(pri);
  [omax,indx] = max(ori);
  if indx < i,
    %
    %  Apply division algorithm to elements above diagonal.
    %
    pi = gget(r,i,i);
    pri = gget(r,1:i-1,i);
    qri = multp(pi,onesp(i-1,1,mode,logcol,stype,dom));
    gri = plm2tfm(pri,qri);
    [gri,pri] = strictprop(gri);
    %
    %  Now perform next transformation.
    %
    pri = minus(pri);
    ui = eyep(lrow,lrow,mode,logcol,stype,dom);
    ui = put(ui,pri,1:i-1,i);
    r = multp(ui,r);
    q = multp(ui,q);
  end,
end
%
%  We still need to invert Q.
%
qdet = rdetp(q);
qadj = radjp(q);
qdet = gain(qdet);
qdet = 1/qdet;
qdet = newpoly(qdet,stype);
if mode == 2,
  qdet = c12(qdet);
  else if mode == 3,
    qdet = c13(qdet,dom);
  end,
end
q = multp(qdet,qadj);
%
pull
return
