function [q2,r2] = qrpi(q1,r1,i)
%
%  This function is called by QRP and SVDP.  It uses row operations to
%  make all elements below the diagonal of column i zero.
%
%  Input Parameter:
%  ----------------
%
%    q1     :=  A unimodular polynomial matrix
%    r1     :=  A polynomial matrix
%               (modes 1, 2, or 3 only)
%
%  Output Parameters:
%  ------------------
%
%    q2     :=  The consolidated q1 polynomial matrix
%    r2     :=  The modified r1 polynomial matrix
%
push('QRPI');
global debg
%
%  Start by unpacking the structural information.
%
[mode,logcol,dtype,stype] = unpack(r1(1,1));
%
%  Get the logical dimensions of R1.
%
[lrow,lcol] = logdim(r1);
%
%  Get the domain information.
%
dom = domain(r1);
%
%  Initialize R2 and Q2.
%
q2 = q1;
r2 = r1;
%
%  Check whether anything below diagonal is not equal zero.
%
pri = gget(r2,i+1:lrow,i);
ori = order(pri);
omax = max(ori);
if omax > 0,
  pmax = 1;
  else
  p = oldmat(pri);
  pmax = max(abs(p));
end
while pmax > 0,
  %
  %  Find lowest order non-zero polynomial.
  %
  pri = gget(r2,i:lrow,i);
  ori = order(pri);
  [sri,indx] = sort(ori);
  if sri(1) > 0,
    k = i + indx(1) - 1;
    else
    p = 0;
    k = 1;
    while p == 0,
      pi = gget(pri,indx(k),1);
      oi = order(pi);
      if oi > 0,
        p = 1;
        k = i + indx(k) - 1;
        else
        p = gain(pi);
        if p ~= 0,
          k = i + indx(k) - 1;
          else
          k = k + 1;
        end,
      end,
    end,
  end,
  %
  %  Lowest order non-zero polynomial is in row k.
  %  Exchange rows i and k.
  %
  [r2,e1] = elopr1(r2,i,k);
  q2 = multp(e1,q2);
  %
  %  Now apply division algorithm.
  %
  pi = gget(r2,i,i);
  pri = gget(r2,i+1:lrow,i);
  qri = multp(pi,onesp(lrow-i,1,mode,logcol,stype,dom));
  gri = plm2tfm(pri,qri);
  [gri,pri] = strictprop(gri);
  %
  %  Now perform next transformation.
  %
  pri = minus(pri);
  ui = eyep(lrow,lrow,mode,logcol,stype,dom);
  ui = put(ui,pri,i+1:lrow,i);
  r2 = multp(ui,r2);
  q2 = multp(ui,q2);
  %
  %  Check whether there is still anything non-zero below diagonal.
  %
  pri = gget(r2,i+1:lrow,i);
  ori = order(pri);
  omax = max(ori);
  if omax > 0,
    pmax = 1;
    else
    p = oldmat(pri);
    pmax = max(abs(p));
  end,
end
%
pull
return
