function [padj] = radjp(p1)
%
%  Find the adjugate of a POLPAC polynomial matrix stored in modes
%  1 thru 3.  This is another polynomial matrix of the same logical
%  dimensions stored in the same mode as the input data structure.
%
%  Input Parameter:
%  ----------------
%
%    p1     :=  Any square POLPAC polynomial matrix in modes 1 thru 3
%
%  Output Parameter:
%  -----------------
%
%    padj   :=  The adjugate polynomial matrix of p1
%
push('RADJP');
global debg
%
[mod1,lcol1,dtyp1,styp1] = unpack(p1(1,1));
[s1,s2] = logdim(p1);
er = 100*eps;
%
%  Check out some error conditions
%
if debg == 3,
  if s1 ~= s2,
    disp('RADJP: Error - Works on square polpac structures only'),
    abort,
  end,
  if mod1 < 1,
    disp('RADJP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if mod1 > 3,
    disp('RADJP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
end
%
%  Create a zero POLPAC structure
%
dom = domain(p1);
zrop = zerop(1,1,mod1,lcol1,styp1,dom);
padj = zerop(s1,s1,mod1,lcol1,styp1,dom);
%
%  First extract a matrix A that has an element for each polynomial
%  in P1.  If the polynomial in P1(K,L) = 0, then A(K,L) = 0,
%  otherwise A(K,L) = 1
%
a = zeros(s1,s1);
xbase = [2 3 3];
xbase = xbase(mod1);
offset = [1 2 1];
offset = offset(mod1);
%
%  Take care of the degenerate case - return one!
%
if s1 == 1,
  if mod1 ~= 3,
    padj(2,1) = 1;
  end,
  if mod1 == 3,
    padj(3,:) = ones(1,lcol1);
  end,
  padj = squeeze(padj);
  pull,
  return,
end
%
%  Create an A matrix for STRUCTM (helps RDETP1() to speed the evaluation
%  in many cases).  K, KK denote row indices, while L, LL indicate column
%  indices.
%
for k = 1:s1,
  for l = 1:s1,
    kk = xbase + (k - 1)*offset;
    ll = (l - 1)*lcol1;
    for m=1:lcol1,
      if abs(p1(kk,ll+m)) > er,
        a(k,l) = 1;
      end,
    end,
    if mod1 == 2,
      if abs(p1(kk-1,ll+1)) > er,
        a(k,l) = 1;
      end,
    end,
  end,
end
%
if abs(norm(a,'inf')) < er,
  pull,
  return,
end
%
%  Calculate the adjugate
%
for k=1:s1,
  for l=1:s1,
    sign = 1;
    kl = (k + l)/2;
    if abs(kl - round(kl + 0.25)) > er,
      sign = -1;
    end,
    if k == 1,
      kk = 2:s1;
      else if k == s1,
        kk = 1:s1-1;
        else
        kk = [1:k-1,k+1:s1];
      end,
    end,
    if l == 1,
      ll = 2:s1;
      else if l == s1,
        ll = 1:s1-1;
        else
        ll = [1:l-1,l+1:s1];
      end,
    end,
    an = a(kk,ll);
    if abs(norm(an,'inf')) > er,
      copart = gget(p1,kk,ll);
      copart = rdetp1(copart,an,zrop,s1-1);
      if sign < 0,
        copart = minus(copart);
      end,
      padj = put(padj,copart,l,k);
    end,
  end,
end
%
%  Get rid of numerical corpses
%
[n,m] = size(padj);
for i=1:n,
  for k=1:m,
    if abs(padj(i,k)) < 1e-10,
      padj(i,k) = 0;
    end,
  end,
end
%
%  Squeeze just for the sake of it
%
padj = squeeze(padj);
%
pull
return
