function [N,M,U,V,K] = rcf(G,PcQ,PoRin,Rout)
%
%  This function operates on POLPAC system descriptions stored in modes 4..6.
%  It computes the right coprime factorization (RCF) of G.  It also computes
%  the optimal feedback matrix K.
%
%  Input Parameter:
%  ----------------
%
%    G      := Transfer function matrix (modes 4..6)
%    Pc/Q   := Pc is a column vector of controller pole locations
%              Q is a state weighting matrix for LQR
%    Po/Rin := Po is a column vector of observer pole locations
%              Rin is an input weighting matrix for LQR
%    Rout   := Rout is an output weighting matrix for LQR
%
%  Output Parameter:
%  -----------------
%
%    N, M   := Right coprime factorization of G
%    U, V   := Right coprime factorization of K
%    K      := The optimal feedback system
%
%  Explanation:
%  ------------
%
%  Given a minimal transfer function matrix.  G is first converted to the
%  time domain.  Optimal feedback matrices F and L are computed using LQR
%  with Q = I, Rin = I, and Rout = I..  System representations are then found
%  for M, N, U, V.  These are converted back to the frequency domain (same
%  mode as G).  Then:  G(s) = N(s)*inv(M(s)), and
%                      K(s) = U(s)*inv(V(s)).
%
%  Defaults:
%  ---------
%
%  Only the first input parameter is mandated.  By default, the stabilizing
%  output feedback controller will be computed by LQR using identity matrices
%  for Q, Rin, and Rout.
%
%  If three input parameters are provided, the second should be the pole
%  locations of the controller poles, and the third should be the pole
%  locations of the observer poles.  The PLACE routine will be used to
%  compute the stabilizing output feedback controller.
%
%  If four input parameters are provided, the second should be the state
%  weighting matrix, the third the input wieghting matrix, and the fourth
%  the output weighting matrix.  LQR will be used to compute the stabilizing
%  output feedback controller.
%
push('RCF')
global debg
%
%  Start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(G(1,1));
[p,m] = logdim(G);
dom = domain(G);
%
%  Check for consistency
%
if debg == 3,
  if mode < 4,
    disp('RCF: Error - Operates on modes 4..6 only'),
    abort,
  end,
  if mode > 6,
    disp('RCF: Error - Operates on modes 4..6 only'),
    abort,
  end,
end
%
%  Convert to time domain
%
Sg = tfm2ss(G);
Sg = minreals(Sg);
%
%  Extract system matrices
%
[A,B,C,D] = oldsyst(Sg);
[n,n] = size(A);
%
%  Compute optimal control
%
if nargin == 3,
  F = -place(A,B,PcQ);
  L = -place(A',C',PoRin)';
  else
  if nargin == 1,
    PcQ = eye(n);
    PcRin = eye(m);
    Rout = eye(p);
  end,
  F = -lqr(A,B,PcQ,PcRin);
  L = -lqr(A',C',PcQ,Rout)';
end
%
%  Now compute the four systems.
%
Am = A + B*F;
Bm = B;
Cm = F;
Dm = eye(m);
%
An = Am;
Bn = B;
Cn = C + D*F;
Dn = D;
%
Au = Am;
Bu = -L;
Cu = F;
Du = zeros(m,p);
%
Av = Am;
Bv = Bu;
Cv = Cn;
Dv = eye(p);
%
%  Make into systems
%
Sm = newsyst(Am,Bm,Cm,Dm,stype);
Sn = newsyst(An,Bn,Cn,Dn,stype);
Su = newsyst(Au,Bu,Cu,Du,stype);
Sv = newsyst(Av,Bv,Cv,Dv,stype);
%
%  Convert back to the frequency domain
%
M = ss2tfm(Sm,mode,dom);
N = ss2tfm(Sn,mode,dom);
U = ss2tfm(Su,mode,dom);
V = ss2tfm(Sv,mode,dom);
M = reduceg(M);
N = reduceg(N);
U = reduceg(U);
V = reduceg(V);
%
%  Check whether we are done
%
if nargout == 5,
  %
  %  We still need to compute K
  %
  Ak = A + B*F + L*C + L*D*F;
  Bk = -L;
  Ck = F;
  Dk = zeros(m,p);
  Sk = newsyst(Ak,Bk,Ck,Dk,stype);
  K = ss2tfm(Sk,mode,dom);
  K = reduceg(K);
end
%
pull
return
