function [detval] = rdet(a)
%
%  Find the determinant of an input matrix A  (by the definition)
%
%  This is the prototype for a POLPAC polynomial determinant function.
%  Therefore, any method to short-circuit processing is taken even if it 
%  does not improve the speed of this version very much.
%
%  Input Parameter:
%  ----------------
%
%    a      :=  Any square Matlab matrix
%
%  Output Parameter:
%  -----------------
%    detval :=  The determinant of the square input matrix
%
push('RDET');
global debg
%
errorbar = 1000*eps;
[s1,s2] = size(a);
detval = 0;
%
%  Check out some error conditions
%
if debg == 3,
  if s1 ~= s2,
    disp('RDET: Error - Works on square matrices only'),
    abort,
  end,
end
%
%  Compute the simple cases n = 1, 2, and 3 
%
if s1 == 1,
  detval = a;
  pull,
  return,
end
%
if s1 == 2,
  detval = a(1,1)*a(2,2) - a(1,2)*a(2,1);
  pull,
  return,
end
%
if s1 == 3,
  detval = a(1,1)*a(2,2)*a(3,3) + a(1,2)*a(2,3)*a(3,1) + ...
           a(1,3)*a(2,1)*a(3,2) - a(1,3)*a(2,2)*a(3,1) - ...
           a(1,1)*a(2,3)*a(3,2) - a(1,2)*a(2,1)*a(3,3);
  pull,
  return,
end
%
%  Look at the matrix structure for simplifications
%
%  diagonal mtype = 1
%  block-diagonal mtype = 2
%  upper triangular  mtype = 3
%  lower triangular mtype = 4
%
[struct,blocks] = strucm(a);
d = [0 1 0 1 1 0 0 0 0 0];
d = d(struct+1);
%
%  If the matrix is diagonal or triangular
%  compute the DET as the product of its diagonal elements
%
if d == 1,
  detval = prod(diag(a));
  pull,
  return,
end
%
%   The block-diagonal case
%
if struct == 2,
  l = 0;
  [m,n] = size(blocks);
  detval = 1;
  for k = 1:n;
    subpart = [(l+1):(l+blocks(k))];
    l = l+blocks(k);
    an = a(subpart,subpart);
    if blocks(k) ~= 1,
      detval = detval * rdet(an);
      else
      detval = detval * an;
    end,
  end,
  pull,
  return,
end   
%
%
%  Find the column/row with the most zeros
%
maxrowz = 0;
bestrow = 1;
for k=1:s1,
  rowz = 0;
  for l=1:s1,
    if abs(a(k,l)) < errorbar,
      rowz = rowz + 1;
    end,
  end,
  if rowz == s1,
    detval = 0;
    pull,
    return,
  end,
  if rowz > maxrowz,
    maxrowz = rowz;
    bestrow = k;
  end,
end
%
%  columns next
%
maxcolz = 0;
bestcol = 1;
for k=1:s1,
  colz = 0;
  for l=1:s1,
    if abs(a(l,k)) < errorbar,
      colz = colz + 1;
    end,
  end,
  if colz == s1,
    detval = 0;
    pull,
    return,
  end,
  if colz > maxcolz,
    maxcolz = colz;
    bestcol = k;
  end,
end
%
if maxcolz > maxrowz,
  k1 = 1; k2 = s1; l1 = bestcol; l2 = bestcol;
  else
  k1 = bestrow; k2 = bestrow; l1 = 1; l2 = s1;
end
%
%  Calculate the determinants of large systems recursively
%
for k = k1:k2,
  for l = l1:l2,
    if abs(a(k,l)) > errorbar,
      sign = 1;
      kl = (k + l)/2;
      if abs(kl - round(kl + 0.25)) > errorbar,
        sign = -1;
      end,
      if k == 1,
        kk = 2:s1;
        else if k == s1,
          kk = 1:s1-1;
          else
          kk = [1:k-1,k+1:s1];
        end,
      end,
      if l == 1,
        ll = 2:s1;
        else if l == s1,
          ll = 1:s1-1;
          else
          ll = [1:l-1,l+1:s1];
        end,
      end,
      an = a(kk,ll);
      detval = detval + sign*a(k,l)*rdet(an);
    end,
  end,
end
%
pull
return
