function [detval] = rdetp1(p1,a,zrop,s1)
%
%  Find the determinant of a square POLPAC polynomial matrix.
%  This is the routine that does all the work.  Note:  the routine is
%  NOT user called so there is little error checking.
%
%  Input Parameters:
%  -----------------
%
%    p1     :=  Square POLPAC polynomial matrix in mode 1, 2, or 3
%    a      :=  Simple Matlab matrix that describes the structure of P1
%    zrop   :=  POLPAC zero-matrix stored in the same mode as P1
%    s1     :=  Logical size of the polynomial matrix
%
%  Output Parameter:
%  -----------------
%
%    detval :=  POLPAC polynomial that is the determinant of P1,
%               stored in the same mode as P1
%
push('RDETP1');
%
er = 1000*eps;
detval = zrop;
if s1 < 1,
  disp('RDETP1: Internal recursion error, size less than one'),
  abort,
end
%
%  Return a zero for the determinant if zeros were input
%
if abs(norm(a,'inf')) < er,
  pull,
  return,
end
%
%  Compute the simple case n = 1
%
if s1 == 1,
  detval = p1;
  pull,
  return,
end
%
%  Compute the simple case n = 2
%
if s1 == 2,
  if a(1,1)*a(2,2) > er,
    t1 = gget(p1,1,1);
    t2 = gget(p1,2,2);
    detval = multp1(t1,t2);
  end,
  if a(1,2)*a(2,1) > er,
    t1 = gget(p1,1,2);
    t2 = gget(p1,2,1);
    t3 = multp1(t1,t2);
    t3 = minus(t3);
    detval = addp1(detval,t3);
  end,
  pull,
  return,
end
%
%  Solve general 3x3 case.  Use A matrix to determine if calls to
%  MULTP1 are really needed.  Note: this only calls gget() or computes
%  terms if needed.  All the attempts allow fewer calls to gget() and
%  reduce the user stack depth needed to execute.
%
if s1 == 3,
  if a(1,1)*a(2,2)*a(3,3) + a(1,1)*a(2,3)*a(3,2) > er,
    t11 = gget(p1,1,1);
  end,
  if a(1,2)*a(2,3)*a(3,1) + a(1,2)*a(2,1)*a(3,3) > er,
    t12 = gget(p1,1,2);
  end,
  if a(1,3)*a(2,1)*a(3,2) + a(1,3)*a(2,2)*a(3,1) > er,
    t13 = gget(p1,1,3);
  end,
  if a(1,3)*a(2,1)*a(3,2) + a(1,2)*a(2,1)*a(3,3) > er,
    t21 = gget(p1,2,1);
  end,
  if a(1,1)*a(2,2)*a(3,3) + a(1,3)*a(2,2)*a(3,1) > er,
    t22 = gget(p1,2,2);
  end,
  if a(1,1)*a(2,3)*a(3,2) + a(1,2)*a(2,3)*a(3,1) > er,
    t23 = gget(p1,2,3);
  end,
  if a(1,2)*a(2,3)*a(3,1) + a(1,3)*a(2,2)*a(3,1) > er,
    t31 = gget(p1,3,1);
  end,
  if a(1,1)*a(2,3)*a(3,2) + a(1,3)*a(2,1)*a(3,2) > er,
    t32 = gget(p1,3,2);
  end,
  if a(1,1)*a(2,2)*a(3,3) + a(1,2)*a(2,1)*a(3,3) > er,
    t33 = gget(p1,3,3);
  end,
  if a(1,1)*a(2,2)*a(3,3) > er,
    detval = multp1(t11,t22);
    detval = multp1(detval,t33);
  end,
  if a(1,2)*a(2,3)*a(3,1) > er,
    t4 = multp1(t12,t23);
    t4 = multp1(t4,t31);
    detval = addp1(detval,t4);
  end,
  if a(1,3)*a(2,1)*a(3,2) > er,
    t4 = multp1(t13,t21);
    t4 = multp1(t4,t32);
    detval = addp1(detval,t4);
  end,
  if a(1,3)*a(2,2)*a(3,1) > er,
    t4 = multp1(t13,t22);
    t4 = multp1(t4,t31);
    t4 = minus(t4);
    detval = addp1(detval,t4);
  end,
  if a(1,1)*a(2,3)*a(3,2) > er,
    t4 = multp1(t11,t23);
    t4 = multp1(t4,t32);
    t4 = minus(t4);
    detval = addp1(detval,t4);
  end,
  if a(1,2)*a(2,1)*a(3,3) > er,
    t4 = multp1(t12,t21);
    t4 = multp1(t4,t33);
    t4 = minus(t4);
    detval = addp1(detval,t4);
  end,
  pull,
  return,
end
%
%  Look at the matrix structure for simplifications
%
%  diagonal mtype = 1
%  block-diagonal mtype = 2
%  upper triangular  mtype = 3
%  lower triangular mtype = 4
%  None of the above, this is a general matrix mtype = 0
%
[struct,blocks] = strucm(a);
d = [0 1 0 1 1 0 0 0 0 0];
d = d(struct+1);
%
%  If the matrix is diagonal or triangular
%  compute the DETP by using the diag elements
%
if d == 1,
  if prod(diag(a)) > er,
    detval = gget(p1,1,1);
    for k=2:s1,
      copart = gget(p1,k,k);
      detval = multp1(detval,copart);
    end,
  end,
  pull,
  return,
end
%
%   The block-diagonal case
%
if struct == 2,
  [m,n] = size(blocks);
  l = 0;
  for k = 1:n;
    subpart = [(l+1):(l+blocks(k))];
    l = l+blocks(k);
    if sum(a(subpart,subpart)) == 0,
      pull,
      return,
    end,
  end,
  l = 0;
  for k = 1:n;
    subpart = [(l+1):(l+blocks(k))];
    l = l+blocks(k);
    part = gget(p1, subpart, subpart);
    if blocks(k) ~= 1,
      an = a(subpart, subpart);
      part = rdetp1(part, an, zrop, blocks(k));
    end,
    if k == 1,
      detval = part;
      else
      detval = multp1(detval, part);
    end,
  end,
  pull,
  return,
end   
%
%  Find the column/row with the most zeros
%
maxrowz = s1;
bestrow = 1;
for k=1:s1,
  rowz = sum(a(k,:));
  if rowz == 0,
    pull,
    return,
  end,
  if rowz < maxrowz,
    maxrowz = rowz;
    bestrow = k;
  end,
end
%
%  Columns next
%
maxcolz = s1;
bestcol = 1;
for k=1:s1,
  colz = sum(a(:,k));
  if colz == 0,
    pull,
    return,
  end,
  if colz < maxcolz,
    maxcolz = colz;
    bestcol = k;
  end,
end
%
if maxcolz < maxrowz,
  k1 = 1; k2 = s1; l1 = bestcol; l2 = bestcol;
  else
  k1 = bestrow; k2 = bestrow; l1 = 1; l2 = s1;
end
%
%  Calculate the determinants of large systems recursively
%
for k=k1:k2,
  for l=l1:l2,
    if a(k,l) == 1,
      sign = 1;
      kl = (k + l)/2;
      if abs(kl - round(kl + 0.25)) > er,
        sign = -1;
      end,
      if k == 1,
        kk = 2:s1;
        else if k == s1,
          kk = 1:s1-1;
          else
          kk = [1:k-1,k+1:s1];
        end,
      end,
      if l == 1,
        ll = 2:s1;
        else if l == s1,
          ll = 1:s1-1;
          else
          ll = [1:l-1,l+1:s1];
        end,
      end,
      an = a(kk,ll);
      if norm(an,'inf') > er,
        cofact = gget(p1,kk,ll);
        codetm = rdetp1(cofact,an,zrop,s1-1);
        comult = gget(p1,k,l);
        copart = multp1(comult,codetm);
        if sign < 0,
          copart = minus(copart);
        end,
        detval = addp1(detval,copart);
      end,
    end,
  end,
end
%
pull
return
