function [g3] = rdivg(g1,g2)
%
%  This function operates on rational function matrices stored in
%  modes 4 thru 6.  G3 is the right division of G1 into G2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    g1     := Rational function matrix in modes 4 thru 6
%    g2     := Rational function matrix in modes 4 thru 6
%
%  Output Parameter:
%  -----------------
%
%    g3     := Rational function matrix
%
push('RDIVG')
global debg
%
%  Check for necessary conversions first
%
[gg1,gg2] = mixmod(g1,g2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(gg1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(gg2(1,1));
%
[n1,m1] = logdim(gg1);
[n2,m2] = logdim(gg2);
nm1 = n1*m1;
nm2 = n2*m2;
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 4,
    disp('RDIVG: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if mod1 > 6,
    disp('RDIVG: Error - Operates only on modes 4 thru 6'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('RDIVG: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if m2 ~= n2,
    disp('RDIVG: Error - G2 must be square'),
    abort,
  end,
  if nm1 ~= 1,
    if nm2 ~= 1,
      if m1 ~= n2,
        disp('RDIVG: Error - Incompatible number of rows/columns'),
        abort,
      end,
    end,
  end,
end
%
%  Perform the actual division now.
%
%  Check first whether this is a scalar/scalar division.
%
if nm1 == 1,
  if nm2 == 1,
    g3 = divg1(gg1,gg2);
    g3 = squeeze(g3);
    pull,
    return,
  end,
end
%
%  Check first whether this is a matrix/scalar division or
%  a matrix/matrix division
%
if nm1 ~= 1,
  if nm2 == 1,
    %
    % The division is of type matrix divided by scalar
    %
    logcol = max([lcol1,lcol2]);
    dom1 = domain(gg1);
    g3 = zerop(n1,m1,mod1,logcol,styp1,dom1);
    for i=1:n1,
      for k=1:m1,
        g1ik = gget(gg1,i,k);
        g3ik = divg1(g1ik,gg2);
        g3 = put(g3,g3ik,i,k);
      end,
    end,
    g3 = squeeze(g3);
    pull,
    return,
  end,
end
%
%  The operation must be of type: matrix divided by matrix
%
g3 = mult(gg1,invg(gg2));
g3 = squeeze(g3);
%
pull
return
