function [g3] = rdivp(p1,p2)
%
%  This function operates on polynomial matrices stored in modes 1 thru 3.
%  G3 is the right division of P1 into P2.
%
%  Mixed mode operations are handled with a preference of trajectories
%  over coefficients over roots.
%
%  Input Parameters:
%  -----------------
%
%    p1     := Polynomial matrix in modes 1 thru 3
%    p2     := Polynomial matrix in modes 1 thru 3
%
%  Output Parameter:
%  -----------------
%
%    g3     := Rational function matrix in modes 4 thru 6
%
push('RDIVP')
global debg
%
%  Check for necessary conversions first
%
[pp1,pp2] = mixmod(p1,p2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(pp1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(pp2(1,1));
%
[n1,m1] = logdim(pp1);
[n2,m2] = logdim(pp2);
nm1 = n1*m1;
nm2 = n2*m2;
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 1,
    disp('RDIVP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if mod1 > 3,
    disp('RDIVP: Error - Operates only on modes 1 thru 3'),
    mod1,
    abort,
  end,
  if styp2 ~= styp1,
    disp('RDIVP: Error - Incompatible system types'),
    abort,
  end,
end
if debg >= 2,
  if m2 ~= n2,
    disp('RDIVP: Error - P2 must be square'),
    abort,
  end,
  if nm1 ~= 1,
    if nm2 ~= 1,
      if m1 ~= n2,
        disp('RDIVP: Error - Incompatible number of rows/columns'),
        abort,
      end,
    end,
  end,
end
%
%  Perform the actual division now.
%
%  Check first whether this is a scalar/scalar division.
%
if nm1 == 1,
  if nm2 == 1,
    g3 = divp1(pp1,pp2);
    g3 = squeeze(g3);
    pull,
    return,
  end,
end
%
%  Check first whether this is a matrix/scalar division or
%  a matrix/matrix division
%
%
if nm1 ~= 1,
  if nm2 == 1,
    %
    % The division is of type matrix divided by scalar
    %
    logcol = max([lcol1,lcol2]);
    dom1 = domain(pp1);
    g3 = zerop(n1,m1,mod1+3,logcol,styp1,dom1);
    for i=1:n1,
      for k=1:m1,
        p1ik = gget(pp1,i,k);
        g3ik = divp1(p1ik,pp2);
        g3 = put(g3,g3ik,i,k);
      end,
    end,
    g3 = squeeze(g3);
    pull,
    return,
  end,
end
%
%  The operation must be of type: matrix divided by matrix
%
g3 = mult(pp1,invp(pp2));
g3 = squeeze(g3);
%
pull
return
