function [gm] = reduceg(gm1)
%
%  This function preforms pole/zero cancellations on each transfer
%  function in GM1.  The numerator and denominator will be extracted
%  and clusters of roots will be replaced with one root using MROOT
%  before pole/zero cancellations take place.
%
%  Input Parameter:
%  ----------------
%
%    gm1    :=  A polynomial transfer function matrix 
%               (modes 4, 5, or 6 only)
%
%  Output Parameter:
%  -----------------
%
%    gm     :=  The same polynomial matrix with common
%               poles and zeroes cancelled.
%
push('REDUCEG')
global debg
%
%  SQUEEZE GM.
%
gm = squeeze(gm1);
%
%  Start by unpacking the structural information.
%
[mode,logcol,dtype,stype] = unpack(gm(1,1));
[lrow,lcol] = logdim(gm);
dom = domain(gm);
%
%  Check for consistency
%
if debg == 3,
  if mode < 4,
    disp('REDUCEG: Error - Operates on modes 4 thru 6 only'),
    abort,
  end,
  if mode > 6,
    disp('REDUCEG: Error - Operates on modes 4 thru 6 only'),
    abort,
  end,
end
%
%  If in mode 6, convert to mode 4 to proceed.
%
if mode == 6,
  gm = c64(gm,0);
end
%
%  Okay, we are ready to perform the pole/zero cancellation in the
%  current mode (either 4 or 5).  Check whether the gains of the
%  denominator polynomials are one.  If not, make them one.
%
[pm,qm] = tfm2plm(gm);
gn = gain(qm);
if norm(gn - ones(size(gn)),inf) > 1000*eps,
  %
  %  No such luck!  We need to move the gains.
  %
  [modp,lcolp,dtypp,stypp] = unpack(pm(1,1));
  [modq,lcolq,dtypq,stypq] = unpack(qm(1,1));
  for i=1:lrow,
    for k=1:lcol,
      if modp == 1,
        pm(i+1,(k-1)*lcolp+1:k*lcolp) = pm(i+1,(k-1)*lcolp+1:k*lcolp)/gn(i,k);
        qm(i+1,(k-1)*lcolq+1:k*lcolq) = qm(i+1,(k-1)*lcolq+1:k*lcolq)/gn(i,k);
        else
        pm(2*i,(k-1)*lcolp+1) = pm(2*i,(k-1)*lcolp+1)/gn(i,k);
        pm(2*i,(k-1)*lcolq+1) = qm(2*i,(k-1)*lcolq+1)/gn(i,k);
      end,
    end,
  end,
end
%
%  Reduce each individual transfer function separately.
%
for i=1:lrow,
  for k=1:lcol,
    pik = gget(pm,i,k);
    qik = gget(qm,i,k);
    gik = plm2tfm(pik,qik);
    [modp,lcolp,dtypp,stypp] = unpack(pik(1,1));
    [modq,lcolq,dtypq,stypq] = unpack(qik(1,1));
    [mod2,lcol2,dtyp2,styp2] = unpack(gik(1,1));
    %
    %  Check whether the numerator gain is zero.  If so, delete
    %  all the poles.
    %
    gainpik = gain(pik);
    if abs(gainpik) < sqrt(eps),
      pik = newpoly(0,stype);
      qik = newpoly(1,stype);
      if mod2 == 5,
        pik = c12(pik);
        qik = c12(qik);
      end,
      gik = plm2tfm(pik,qik);
      else
      %
      %  If nonproper, convert to mode 5 to proceed.
      %
      flg = 0;
      if mod2 == 4,
        if ~isproper(gik),
          flg = 1;
          gik = c45(gik);
          [pik,qik] = c52(gik);
          [modp,lcolp,dtypp,stypp] = unpack(pik(1,1));
          [modq,lcolq,dtypq,stypq] = unpack(qik(1,1));
          [mod2,lcol2,dtyp2,styp2] = unpack(gik(1,1));
        end,
      end
      %
      %  Check whether the numerator polynomial is constant.  If so, we are
      %  already done.
      %
      op = order(pik);
      if max(max(op)) > 0,
        %
        %  I didn't think so.  We need to continue.
        %  Check whether the denominator polynomial is constant.  If so,
        %  all we need to do is downgrade the rational function matrix to the
        %  status of a polynomial matrix.
        %
        oq = order(qm);
        if max(max(oq)) > 0,
          %
          %  Nope!  We really need to get to work!
          %  We no longer have ANY excuse for not getting to work.
          %
          if mod2 == 4,
            %
            %  Convert to the time domain, then use MINREALS,
            %  finally convert back to the original mode.
            %
            sik = c40(gik);
            sik = minreals(sik);
            gik = c04(sik);
            else
            %
            %  Match poles with zeros directly.
            %
            for l=2:lcolp,
              pl = pik(2,l);
              for ll=2:lcolq,
                qll = qik(2,ll);
                if abs(pl - qll) < sqrt(eps),
                  %
                  %  We got a match.  Reduce multiplicities.
                  %
                  mp = pik(3,l);
                  mq = qik(3,ll);
                  if mp < mq,
                    pik(3,l) = 0;
                    qik(3,ll) = mq - mp;
                    else
                    pik(3,l) = mp - mq;
                    qik(3,ll) = 0;
                  end,
                end,
              end,
            end,
            %
            %  Compute the new orders after reduction.
            %
            pik(3,1) = sum(pik(3,2:lcolp));
            qik(3,1) = sum(qik(3,2:lcolq));
            %
            %  Now get rid of poles with zero multiplicities.
            %
            ll = 1;
            for l=2:lcolp,
              mp = pik(3,l);
              if mp > 0,
                ll = ll + 1;
                pik(2:3,ll) = pik(2:3,l);
              end,
            end,
            if ll < lcolp,
              pik(2:3,ll+1:lcolp) = zeros(2,lcolp-ll);
            end,
            ll = 1;
            for l=2:lcolq,
              mq = qik(3,l);
              if mq > 0,
                ll = ll + 1;
                qik(2:3,ll) = qik(2:3,l);
              end,
            end,
            if ll < lcolq,
              qik(2:3,ll+1:lcolq) = zeros(2,lcolq-ll);
            end,
            %
            %  Finally squeeze.
            %
            pik = squeeze(pik);
            qik = squeeze(qik);
            gik = plm2tfm(pik,qik);
          end,
        end,
      end,
      %
      %  Undo the conversion.
      %
      if flg == 1,
        gik = c54(gik);
      end,
    end,
    gm = put(gm,gik,i,k);
  end,
end
%
%  Check whether the denominator polynomial is now constant.
%
[pm,qm] = tfm2plm(gm);
o = order(qm);
if max(max(o)) == 0,
  %
  %  Downgrade to polynomial matrix status.
  %
  gm = pm;
end
%
%  We need to undo the original conversion.
%
if mode == 6,
  [mod2,lcol2,dtyp2,styp2] = unpack(gm(1,1));
  if mod2 == 4,
    gm = c46(gm,dom);
    else
    gm = c13(gm,dom);
  end,
end
%
gm = squeeze(gm);
%
pull
return
