function [p2] = squeeze(p1)
% 
%  Input Parameters:
%  -----------------
%
%     p1     := The input file to be squeezed
%
%  Output Parameters:
%  ------------------
%
%     p2     := The squeezed output file 
%     
%  The SQUEEZE function removes excessive LOGCOL columns
%  full of zeros.  For example, if LOGCOL is 3 in
%  a 12x12 matrix, then squeeze will check if the columns
%  3,6,9, and 12 are zero.  if yes, SQUEEZE will remove these
%  columns, change LOGCOL to 2 and check again.
%
push('SQUEEZE')
global debg
%
%  Compute a decent tolerance value
%
tol = sqrt(eps);
%
%  Start by unpacking the MODE, LOGCOL, DTYPE, and STYPE parameters
%  of the data structure to be squeezed.
%
p2 = p1;
[mode,logcol,dtype,stype] = unpack(p2(1,1));
if logcol == 1,  pull,  return,  end
%
%  Check whether mode is appropriate for requested operation
%  (must be 1, 2, 4, 5, 7, or 8)
%
if mode == 0,
  pull,
  return,
end
m3 = mode/3;
if abs(m3-round(m3)) < 10*eps,
  pull,
  return,
end
%
%  if mode is 2, 5, or 8, we must check first for the gain values.
%  if any gain value is zero, we must remove all the roots related
%  with it.
%
m2 = (mode + 1)/3;
if abs(m2 - round(m2)) < tol,
  [np,mp] = size(p2);
  gain = abs(p2(2:2:np-1,1:logcol:mp-logcol+1));
  [rg,lg] = size(gain);
  if rg == 1,
    gc = gain;
    else 
    gc = min(gain);
  end,
  if min(gc) < tol,
    for k=1:(mp/logcol),
      if gc(k) < tol,
        kp1 = (k - 1)*logcol + 1;
        kp2 = k*logcol;
        for i=1:(np-1)/2,
          if gain(i,k) < tol,
            ip1 = 2*i;
            ip2 = ip1 + 1;
            if p2(ip2,kp1) > 0,
              p2(ip1:ip2,kp1:kp2) = zeros(2,logcol);
            end,
          end,
        end,
      end,
    end,
  end,
end
%
%  Put all rightmost physical columns into one long column vector,
%  determine through a NORM operation whether this vector is all
%  zero.  if so, throw it out, and repeat the test, until a set of
%  rightmost columns is found that can no longer be considered zero
%
check = 0;
while check <= tol,
  %
  %  Compute the number of logical columns.  Since SQUEEZE is
  %  a very low level function, avoid calling LOGDIM.
  %
  [m,n] = size(p2);
  nc = round(n/logcol);
  %
  %  if debg=3, check whether the number of physical columns
  %  is an integer multiple of the number of logical columns
  %
  if debg == 3,
    if abs(n/logcol - nc) > 10*eps,
      disp('SQUEEZE: Error - Physical/logical dimension mismatch'),
      abort,
    end,  
  end,
  %
  %  Now, concatenate the rightmost columns into the AC vector
  %
  ac = 0;
  for i=1:nc,
    bc = p2(:,i*logcol);
    ac = [ac;bc];
  end,
  %
  %  Now, check whether this is a ZERO vector
  %
  check = norm(ac,'inf');
  %
  %  if so, get rid of all these columns, and try again
  %
  if check <= tol,
    ac = p2(:,1:logcol-1);
    for i=1:nc-1,
      bc = p2(:,i*logcol+1:(i+1)*logcol-1);
      ac = [ac,bc];
    end,
    p2 = ac;
    logcol = logcol-1;
  end,
  %
  %  while CHECK is sufficiently small, stay in the loop, and
  %  try your luck once more.
  %
end
%
%  Set all remaining elements to zero that are negligibly small
%
[n,m] = size(p2);
for i=1:n,
  for k=1:m,
    if abs(round(imag(p2(i,k))) - imag(p2(i,k))) < tol,
      p2(i,k) = real(p2(i,k)) + j*round(imag(p2(i,k)));
    end
    if abs(round(real(p2(i,k))) - real(p2(i,k))) < tol,
      p2(i,k) = round(real(p2(i,k))) + j*imag(p2(i,k));
    end
    if abs(round(p2(i,k)) - p2(i,k)) < tol,
      p2(i,k) = round(p2(i,k));
    end,
  end,
end
%
%  Reevaluate the datatype
%
dtype = datatype(p2);
%
%  Create the final output data structure by encoding the (1,1) element
%  correctly.
%
p2(1,1) = ppack(mode,logcol,dtype,stype);
%
pull
return
