function [txt,indx] = strg(gm)
%
%  This function maps the transfer function matrix GM (storage modes 4 thru 6)
%  into a text string for display.
%
%  Input Parameter:
%  ----------------
%
%    gm     := POLPAC transfer function matrix (modes 4 thru 6)
%
%  Output Parameters:
%  ------------------
%
%    txt    := Text string with GM encoded
%    indx   := Row vector containing the column number of TXT where
%              each logical field ends
%
push('STRG')
global debg
%
%  Start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(gm(1,1));
[n,m] = logdim(gm);
%
%  Check for consistency
%
if debg == 3,
  if mode < 4,
    disp('STRG: Error - Operates on modes 4 thru 6 only');
    mode,
    abort,
  end,
  if mode > 6,
    disp('STRG: Error - Operates on modes 4 thru 6 only');
    mode,
    abort,
  end,
end
%
%  Treat first the case: mode=6
%
if mode == 6,
  disp('STRG: Error - This function is not yet implemented');
  abort,
end
%
%  The mode is secured to be either 4 or 5.
%  Double loop over all logical elements in GM
%
for i=1:n,
  for k=1:m,
    g = gget(gm,i,k);
    t = [ strg1(g) , ' '*ones(3,3) ];
    [nn,mm] = size(t);
    lpnt(1,k) = mm;
    if k == 1,
      tt = t;
    end,
    if k > 1,
      tt = [ tt , t ];
    end,
  end,
  %
  %  Place one row of blanks between transfer functions
  %
  [nn,mm] = size(tt);
  tt = [ tt ; ' '*ones(1,mm) ];
  %
  %  Concatenate new logical row from below
  %
  if i == 1,
    pnt = lpnt;
    ttt = tt;
  end,
  if i > 1,
    pnt = [ pnt ; lpnt ];
    [nn,mm] = size(tt);
    [nnn,mmm] = size(ttt);
    %
    %  if new text is longer, pad old text with blanks
    %
    if mm > mmm,
      ttt = [ ttt , ' '*ones(nnn,mm-mmm) ];
    end,
    %
    %  if new text is shorter, pad new text with blanks
    %
    if mm < mmm,
      tt = [ tt , ' '*ones(nn,mmm-mm) ];
    end,
    %
    %  Concatenate old text with new text from below
    %
    ttt = [ ttt ; tt ];
  end,
end
%
%  Now, equalize the text
%
%  First, compute the required column sizes
%
if n > 1,
  lpnt = max(pnt);
 else
  lpnt = pnt;
end
[ln,lm] = size(lpnt);
%
%  Now, calculate integral pointer tables
%
ipnt = zeros(ln,lm);
ipnt(1) = 1;
jpnt = ones(n,1);
indx = zeros(ln,lm);
indx(1) = lpnt(1);
if m > 1,
  for k=2:m,
    ipnt(k) = ipnt(k-1) + lpnt(k-1);
    indx(k) = indx(k-1) + lpnt(k);
    for i=1:n,
      jpnt(i,k) = jpnt(i,k-1) + pnt(i,k-1);
    end,
  end,
end
%
%  Now, compute the physical number of columns required
%  and create a blank text matrix
%
mm = sum(lpnt);
txt = ' '*ones(4*n,mm);
%
%  Center the old text strings into the new text matrix
%
for i=1:n,
  for k=1:m,
    dff = lpnt(k) - pnt(i,k);
    nbegin = ipnt(k) + round(dff/2);
    nend = nbegin + pnt(i,k) - 1;
    obegin = jpnt(i,k);
    oend = obegin + pnt(i,k) - 1;
    i1 = 4*(i - 1) + 1;
    i2 = 4*i;
    txt(i1:i2,nbegin:nend) = ttt(i1:i2,obegin:oend);
  end,
end
%
pull
return
