function [gm2,pm2] = strictprop(gm1)
%
%  This function does the following descomposition on GM1:
%
%          gm1(s) = gm2(s) + pm2(s)
%
%  If GM1 is strictly proper, PM2 will be a zero polynomial matrix.
%  If GM1 is proper but not strictly proper, PM2 will be a constant
%  polynomial matrix.  If GM1 is not proper, PM2 will be a polynomial
%  matrix of order higher than zero.  This function works intrinsically
%  on polynumial matrices in mode 4.  If the input mode of GM1 is 5 or 6,
%  conversions to mode 4 will take place.  The resulting PM2 will be
%  stored in the input mode minus 3.
%
%  Input Parameter:
%  ----------------
%
%    gm1    := transfer function matrix in modes 4 thru 6
%
%  Output Parameters:
%  ------------------
%
%    gm2    := strictly proper part of GM1 stored in the same mode as GM1
%    pm2    := a polynomial matrix stored in MODE = MODE{GM1(s)} - 3
%
push('STRICTPROP')
global debg
%
%  Unpack structural information.
%
[mode,logcol,dtype,stype] = unpack(gm1(1,1));
[lrow,lcol] = logdim(gm1);
dom = domain(gm1);
%
%  Check for consistency
%
if debg == 3,
  if mode < 4,
    disp('STRICTPROP: Error - Operates on modes 4 thru 6 only'),
    abort,
  end,
  if mode > 6,
    disp('STRICTPROP: Error - Operates on modes 4 thru 6 only'),
    abort,
  end,
end
%
%  Check if GM1 is strictly proper by calling function ISPROPER.
%  If GM1 is strictly proper, create a zero PM2, set GM2 equal GM1 and return.
%
flagp = isproper(gm1);
if flagp == 2,
  gm2 = gm1;
  pm2 = zerop(lrow,lcol,mode-3,1,stype,dom);
  pull,
  return,
end
%
%  Check that GM1 is in mode 4. Make conversions to mode 4 if necessary.
%  If conversions are made, need to unpack again.
%
gm = gm1;
if mode ~= 4,
  if mode == 5,
    gm = c54(gm1);
    else
    gm = c64(gm1);
  end,
  [mode4,logcol,dtype,stype] = unpack(gm(1,1));
end
%
%  Define the tolerance for a 'zero' coefficient.
%
tol = 100*eps;
%
%  Find the physical dimensions of GM1.
%
[gr,gc] = size(gm);
%
fr = (gr+1)/2;
%
%  Create empty shells for the outputs.
%
pm2 = zerop(lrow,lcol,1,logcol,stype,dom);
gm2 = zerop(lrow,lcol,4,logcol,stype,dom);
%
%  for every logical column loop over all logical rows.
%
for cc=1:lcol,
  ll = logcol*cc;
  la = ll - logcol + 1;
  for rr=1:lrow,
    %
    %  NU contains the coefficients of the numerator.
    %  DN contains the coefficients of the denominator.
    %  Reverse the coefficients and eliminate leading zeros.
    %
    nu = gm(rr+1,ll:-1:la);
    dn = gm(rr+fr,ll:-1:la);
    ii = 1;
    numc = abs(nu(ii));
    while numc < tol,
      ii = ii + 1;
      if ii >= logcol,
        ii = logcol;
        numc = 1;
        else
        numc = abs(nu(ii));
      end,
    end,
    nu = nu(:,ii:logcol);
    ii = 1;
    while abs(dn(ii)) < tol,
      ii = ii + 1;
    end,
    dn = dn(:,ii:logcol);
    %
    %  Now divide numerator by denominator, then reverse the
    %  the polynomial coefficients again and, finally
    %  assign results into GM2 and PM2.
    %
    [pc,nu] = deconv(nu,dn);
    [rpc,cpc] = size(pc);
    [rnu,cnu] = size(nu);
    pm2(rr+1,la:la+cpc-1) = pc(cpc:-1:1);
    gm2(rr+1,la:la+cnu-1) = nu(cnu:-1:1);
  end,
end
%
%  Now insert denominator polynomials into GM2.
%
gm2(fr+1:gr,:) = gm(fr+1:gr,:);
%
%  Now pack structure information correctly.
%
dtype = datatype(pm2);
pm2(1,1) = ppack(1,logcol,dtype,stype);
dtype = datatype(gm2);
gm2(1,1) = ppack(4,logcol,dtype,stype);
%
%  SQUEEZE outputs.
%
gm2 = squeeze(gm2);
pm2 = squeeze(pm2);
%
%  Make conversions to original representation if necessary.
%
if mode ~= 4,
  if mode == 5,
    gm2 = c45(gm2);
    pm2 = c12(pm2);      
    else
    gm2 = c46(gm2,dom);
    pm2 = c13(pm2,dom);
  end,
end
%
pull
return
