function [txt,indx] = strp(pm)
%
%  This function maps the polynomial matrix PM (in storage modes 1 thru 3)
%  into a text string for display.
%
%  Input Parameter:
%  ----------------
%
%    pm     := POLPAC polynomial matrix (modes 1 thru 3)
%
%  Output Parameters:
%  ------------------
%
%    txt    := Text string with PM encoded
%    indx   := Row vector containing the column number of TXT where
%              each logical field ends
%
push('STRP')
global debg
%
%  Start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(pm(1,1));
[n,m] = logdim(pm);
%
%  Check for consistency
%
if debg == 3,
  if mode == 0,
    disp('STRP: Error - Operates on modes 1 thru 3 only');
    mode,
    abort,
  end,
  if mode > 3,
    disp('STRP: Error - Operates on modes 1 thru 3 only');
    mode,
    abort,
  end,
end
%
%  Treat first the case: mode=3
%
if mode == 3,
  disp('STRP: Error - This function is not yet implemented');
  abort,
end
%
%  The mode is secured to be either 1 or 2.
%  Double loop over all logical elements in PM
%
for i=1:n,
  for k=1:m,
    p = gget(pm,i,k);
    t = [ strp1(p) , '   ' ];
    [nn,mm] = size(t);
    lpnt(1,k) = mm;
    if k == 1,
      tt = t;
    end,
    if k > 1,
      tt = [ tt , t ];
    end,
  end,
  if i == 1,
    pnt = lpnt;
    ttt = tt;
  end,
  if i > 1,
    pnt = [ pnt ; lpnt ];
    [nn,mm] = size(tt);
    [nnn,mmm] = size(ttt);
    %
    %  if new text is longer, pad old text with blanks
    %
    if mm > mmm,
      ttt = [ ttt , ' '*ones(nnn,mm-mmm) ];
    end,
    %
    %  if new text is shorter, pad new text with blanks
    %
    if mm < mmm,
      tt = [ tt , ' '*ones(nn,mmm-mm) ];
    end,
    %
    %  Concatenate old text with new text from below
    %
    ttt = [ ttt ; tt ];
  end,
end
%
%  Now, equalize the text
%
%  First, compute the required column sizes
%
if n > 1,
  lpnt = max(pnt);
  else
  lpnt = pnt;
end
[ln,lm] = size(lpnt);
%
%  Now, calculate integral pointer tables
%
ipnt = zeros(ln,lm);
ipnt(1) = 1;
jpnt = ones(n,1);
indx = zeros(ln,lm);
indx(1) = lpnt(1);
for k=2:m,
  ipnt(k) = ipnt(k-1) + lpnt(k-1);
  indx(k) = indx(k-1) + lpnt(k);
  for i=1:n,
    jpnt(i,k) = jpnt(i,k-1) + pnt(i,k-1);
  end,
end
%
%  Now, compute the physical number of columns required
%  and create a blank text matrix
%
mm = sum(lpnt);
txt = ' '*ones(n,mm);
%
%  Center the old text strings into the new text matrix
%
for i=1:n,
  for k=1:m,
    dff = lpnt(k) - pnt(i,k);
    nbegin = ipnt(k) + round(dff/2);
    nend = nbegin + pnt(i,k) - 1;
    obegin = jpnt(i,k);
    oend = obegin + pnt(i,k) - 1;
    txt(i,nbegin:nend) = ttt(i,obegin:oend);
  end,
end
%
pull
return
