function [mtype,bsize] = strucm(a)
%
%  This function evaluates properties of a square matrix
%
%  Input Parameter:
%  ----------------
%
%       a       :=  a square matrix
%
%  Output Parameters:
%  ------------------
%
%       mtype   :=  matrix type
%                     mtype = 0 :  general matrix
%                     mtype = 1 :  diagonal matrix
%                     mtype = 2 :  block-diagonal matrix
%                     mtype = 3 :  upper-triangular matrix
%                     mtype = 4 :  lower-triangular matrix
%                     mtype = 5 :  banded matrix
%                     mtype = 6 :  Hermitian matrix
%                     mtype = 7 :  skew-Hermitian matrix
%                     mtype = 8 :  symmetric and complex
%                     mtype = 9 :  skew-symmetric and complex
%       bsize   :=  matrix block/band size
%
push('STRUCM');
global debg
%
[n,m] = size(a);
%
%  Check for consistencies
%
if debg == 3,
  if n ~= m,
    disp('STRUCM: Error - Operates on square matrices only'),
    abort,
  end,
end
%
%  Set MTYPE and BSIZE to zero to start with
%
mtype = 0;
bsize = 0;
%
%  Check whether the matrix is diagonal
%
if norm(a - diag(diag(a)),'inf') < 100*eps,
  mtype = 1;
  pull,
  return,
end
%
%  Check whether the matrix is block-diagonal
%
k = 1;
for i=2:n,
  if abs(a(i,k)) < 100*eps,
    if abs(a(k,i)) < 100*eps,
      tstmat = [ a(i:n,k:(i-1)) , a(k:(i-1),i:n)' ];
      if norm(tstmat,'inf') < 100*eps,
        bsize = [ bsize , (i-k) ];
        k = i;
      end,
    end,
  end,
end
[nb,mb] = size(bsize);
if mb > 1,
  mtype = 2;
  bsize = bsize(2:mb);
  lastel = n - sum(bsize);
  bsize = [ bsize , lastel ];
  pull,
  return,
end
%
%  Check whether the matrix is upper triangular
%
tstmat = 0;
for i=1:(n-1),
  tstmat = [ tstmat ; a((i+1):n,i) ];
end
if norm(tstmat,'inf') < 100*eps,
  mtype = 3;
  pull,
  return,
end
%
%  Check whether the matrix is lower triangular
%
tstmat = 0;
for i=1:(n-1),
  tstmat = [ tstmat , a(i,(i+1):n) ];
end
if norm(tstmat,'inf') < 100*eps,
  mtype = 4;
  pull,
  return,
end
%
%  Check whether the matrix is banded
%
tstmat = 0;
for i=(n-1):-1:1,
  tstmat = [ tstmat ; diag(a,-i) ; diag(a,i) ];
  if norm(tstmat,'inf') < 100*eps,
    bsize = 2*i - 1;
    else break,
  end,
end
if bsize > 0,
  mtype = 5;
  pull,
  return,
end
%
%  Check whether the matrix is Hermitian
%
if norm(a - a','inf') < 100*eps,
  mtype = 6;
  pull,
  return,
end
%
%  Check whether the matrix is skew-Hermitian
%
if norm(a + a' - 2*diag(diag(a)),'inf') < 100*eps,
  mtype = 7;
  pull,
  return,
end
%
%  Check whether the matrix is symmetric and complex
%
if norm(a - conj(a'),'inf') < 100*eps,
  mtype = 8;
  pull,
  return,
end
%
%  Check whether the matrix is skew-symmetric and complex
%
if norm(a + conj(a') - 2*diag(diag(a)),'inf') < 100*eps,
  mtype = 9;
  pull,
  return,
end
%
%  None of the above, this is a general matrix
%
pull
return
