function [u,s,v] = svdp(pm)
%
%  This function performs a Smith diagonalization of a polynomial
%  matrix: PM = U*S*V'.
%
%  Input Parameter:
%  ----------------
%
%    pm     :=  The polynomial matrix to be diagonalized
%               (modes 1, 2, or 3 only)
%
%  Output Parameters:
%  ------------------
%
%    u      :=  A unimodular polynomial matrix
%    s      :=  A diagonal polynomial matrix
%    v      :=  A unimodular polynomial matrix
%
push('SVDP');
global debg
%
%  Start by unpacking the structural information.
%
[mode,logcol,dtype,stype] = unpack(pm(1,1));
%
%  Check for consistency.
%
if debg == 3,
  if mode < 1,
    disp('SVDP: Error - Operates on modes 1 thru 3 only'),
    abort,
  end,
  if mode > 3,
    disp('SVDP: Error - Operates on modes 1 thru 3 only'),
    abort,
  end,
end
%
%  Get the logical dimensions of PM.
%
[lrow,lcol] = logdim(pm);
%
%  Get the domain information.
%
dom = domain(pm);
%
%  Initialize U, S, and V.
%
u = eyep(lrow,lrow,mode,logcol,stype,dom);
s = pm;
v = eyep(lcol,lcol,mode,logcol,stype,dom);
%
%  Loop over all significant columns.
%
i = 1;
imax = min([lrow,lcol]);
while i <= imax,
  %
  %  Make all elements below diagonal of column i zero.
  %
  if i < lrow,
    [u,s] = qrpi(u,s,i);
  end,
  %
  %  Make all elements to the right of diagonal of row i zero.
  %
  s = transpose(s);
  if i < lcol,
    [v,s] = qrpi(v,s,i);
  end,
  s = transpose(s);
  %
  %  Check whether new non-zero elements were introduced below the diagonal.
  %
  if i < imax,
    pri = gget(s,i+1:lrow,i);
    ori = order(pri);
    omax = max(ori);
    if omax == 0,
      p = oldmat(pri);
      pmax = max(p);
      if p == 0,
        %
        %  Everything is fine.  Proceed with next column/row pair.
        %
        i = i + 1;
      end,
    end,
    else
    i = i + 1;
  end,
end
%
%  We still need to invert U and V.
%
udet = rdetp(u);
uadj = radjp(u);
udet = gain(udet);
udet = 1/udet;
udet = newpoly(udet,stype);
if mode == 2,
  udet = c12(udet);
  else if mode == 3,
    udet = c13(udet,dom);
  end,
end
u = multp(udet,uadj);
%
vdet = rdetp(v);
vadj = radjp(v);
vdet = gain(vdet);
vdet = 1/vdet;
vdet = newpoly(vdet,stype);
if mode == 2,
  vdet = c12(vdet);
  else if mode == 3,
    vdet = c13(vdet,dom);
  end,
end
v = multp(vdet,vadj);
%
pull
return
