function [pm2] = transpose(pm1)
%
%  This function takes the transpose of a polynomial
%  matrix.  In modes 1 thru 6, the output polynomial
%  matrix is the Hermitian transpose of the input.  For
%  modes 0 and 7 thru 9 the output represents the 
%  dual system of the input.
%
%  Input Parameter:
%  ----------------
%
%    pm1    := Polynomial matrix to be transposed (any mode)
%
%  Output Parameter:
%  -----------------
%
%    pm2    := Transposed polynomial matrix (same mode as input)
%
push('transpose')
%
%  Start by unpacking the structural information
% 
[mode,logcol,dtype,stype] = unpack(pm1(1,1));
%
%  if the mode equals 0 create the dual system and return
%
if mode == 0,
  [a,b,c,d] = oldsyst(pm1);
  pm2 = newsyst(a',c',b',d',stype);
  pull,
  return,
end
%
%  if the mode is 1 thru 6, compute the transpose of the 
%  polynomial matrix by using the ZEROP, GET, and PUT routines
%  and exchanging the ik polynomial with the ki polynomial.
%
if mode < 7,
  % 
  %  Create a shell for the transposed matrix that is (lcol*lrow)
  %
  [lrow,lcol] = logdim(pm1);
  dom = domain(pm1);
  pm2 = zerop(lcol,lrow,mode,logcol,stype,dom);
  %
  %  Exchange the logical rows and columns.
  %
  for i=1:lrow,
    for k=1:lcol,
      pik = gget(pm1,i,k);
      pm2 = put(pm2,pik,k,i);
    end,
  end,
  %
  %  Take the conjugate of the output matrix to produce a Hermitian
  %  transpose.  The regular Matlab function "CONJ" works fine in
  %  all these cases.
  %
  pm2 = conj(pm2);
  pull,
  return,
end
%
%  Now find the dual system for the modes 7, 8, and 9 using
%  the convert routines and calling transpose and then 
%  converting back to the proper mode.
%
[pm3,qm3,rm3,wm3] = pms2plm(pm1);
%
%  Perform the transpose using transpose recursively
%
if ~isempty(pm3),
  pm = transpose(pm3);
  qm = transpose(rm3);
  rm = transpose(qm3);
  else
  pm = [];
  qm = [];
  rm = [];
end
wm = transpose(wm3);
%
% Convert back to the desired mode
%
pm2 = plm2pms(pm,qm,rm,wm);
%
pull
return
