function [T,S] = unitfb(G,sgn)
%
%  This function computes unit feedback for a "square" POLPAC system.  It
%  operates on modes 0, 4..6, and 7..9.
%
%  Input Parameters:
%  -----------------
%
%    G      :=  A POLPAC system with equal numbers of inputs and outputs
%    sgn    :=  The sign of the unit feedback
%               default: sign = -1
%
%  Output Parameters:
%  ------------------
%
%    T      :=  Transmission transfer function ( r -> y )
%    S      :=  Sensitivity transfer function  ( r -> u )
%
%    T, S are returned in the same mode as G.
%
%  Explanation:
%  ------------
%
%        r   +    u    -------
%      ------->o------>|  G  |-------+------> y
%              ^       -------       |
%         sign |                     |
%              -----------------------
%
push('UNITFB')
global debg
%
%  Start by unpacking the structural information
%
[mode,logcol,dtype,stype] = unpack(G(1,1));
[lrow,lcol] = logdim(G);
dom = domain(G);
%
%  Check for missing sign
%
if nargin == 1,
  sgn = -1;
end
%
%  Check for consistency
%
if debg == 3,
  if lrow ~= lcol,
    disp('UNITFB: Error - System must have equal number of inputs and outputs'),
    abort,
    return,
  end,
  if abs(sgn) - 1 > 1000*eps,
    disp('UNITFB: Error - Second argument must be either +1 or -1'),
    abort,
    return,
  end,
end
%
%  Branch depending on the mode of the system.
%
if mode == 0,
  %
  %  Is a state-space description
  %
  [A,B,C,D] = oldsyst(G);
  DD = eye(size(D)) - sgn*D;
  %
  %  Check whether system is well-posed
  %
  if norm(DD) < 1000*eps,
    disp('UNITFB: Error - System is ill-posed'),
    abort,
    return,
  end,
  %
  %  Compute closed-loop system
  %
  DDinv = inv(DD);
  Bcl = B*DDinv;
  Acl = A + sgn*Bcl*C;
  C1cl = DDinv*C;
  D1cl = DDinv*D;
  C2cl = sgn*C1cl;
  D2cl = DDinv;
  T = newsyst(Acl,Bcl,C1cl,D1cl,stype);
  S = newsyst(Acl,Bcl,C2cl,D2cl,stype);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = minreals(T);
  S = minreals(S);
  pull,
  return,
end
%
if mode == 4 | mode == 5 | mode == 6,
  %
  %  Is a transfer function matrix
  %
  I = eyep(lrow,lcol,mode,logcol,stype,dom);
  ssgn = newmat(sgn,mode-3,stype,dom);
  IG = sub(I,multg(ssgn,G));
  %
  %  Check whether system is well-posed
  %
  [G2,P2] = strictprop(IG);
  ord = order(P2);
  if ord == 0,
    gn = gain(P2);
    if gn == 0,
      disp('UNITFB: Error - System is ill-posed'),
      abort,
      return,
    end,
  end,
  %
  %  Compute closed-loop system
  %
  S = invg(IG);
  T = multg(S,G);
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  S = reduceg(S);
  T = reduceg(T);
  pull,
  return,
end
%
if mode == 7 | mode == 8 | mode == 9,
  %
  %  Is a polynomial matrix system description
  %
  [P,Q,R,W] = pms2plm(G);
  %
  %  Check whether W is a constant matrix.
  %
  ord = order(W);
  if ord > 0,
    %
    %  Convert to transfer function matrix form, and try again.
    %
    G = pms2tfm(G);
    [T,S] = unitfb(G,sgn);
    T = tfm2pms(T);
    S = tfm2pms(S);
    else
    %
    %  Okay!  We can proceed directly.
    %
    I = eyep(lrow,lcol,mode-6,logcol,stype,dom);
    ssgn = newmat(sgn,mode-6,stype,dom);
    IW = sub(I,multp(ssgn,W));
    %
    %  Check whether system is well-posed
    %
    P2 = rdetp(IW);
    ord = order(P2);
    if ord == 0,
      gn = gain(P2);
      if gn == 0,
        disp('UNITFB: Error - System is ill-posed'),
        abort,
        return,
      end,
    end,
    %
    %  Compute closed-loop system
    %
    IWinv = invp(IW);
    Qcl = multp(Q,IWinv);
    Pcl = sub(P,multp(ssgn,multp(Qcl,R)));
    R1cl = multp(IWinv,R);
    W1cl = multp(IWinv,W);
    R2cl = multp(ssgn,R1cl);
    W2cl = IWinv;
    T = plm2pms(Pcl,Qcl,R1cl,W1cl);
    S = plm2pms(Pcl,Qcl,R2cl,W2cl);
  end,
  %
  %  Get rid of uncontrollable/unobservable modes
  %
  T = minrealps(T);
  S = minrealps(S);
  pull,
  return,
end
%
return
