function [p3] = vconc(p1,p2)
%  VCONC:  Vertically concatenate two POLPAC data structures
%
%  Usage:  p3 = vconc(p1,p2)
%
%  This function concatenates a POLPAC data structure P2 vertically
%  from below to another POLPAC data structure P1 of same type.
%  Concatenation here is meant in a logical rather than physical sense.
%  
%  Input Parameters:
%  -----------------
%
%    p1     :=  POLPAC data structure in mode 1 thru 6
%    p2     :=  Another POLPAC data structure in same mode as P1
%               The number of logical columns must be the same.
%
%  Output Parameter:
%  -----------------
%
%    p3     :=  The logical concatenation of [P1;P2]
%
push('VCONC')
global debg
%
%  Call MIXMOD fist
%
[pp1,pp2] = mixmod(p1,p2);
%
%  Start by unpacking the structural information
%
[mod1,lcol1,dtyp1,styp1] = unpack(pp1(1,1));
[mod2,lcol2,dtyp2,styp2] = unpack(pp2(1,1));
%
%  Compute the structural information of the resulting
%  data structure
%
mode = mod1;
logcol = max([lcol1,lcol2]);
dtype = max([dtyp1,dtyp2]);
stype = styp1;
%
%  Check for consistency
%
if debg == 3,
  if mod1 < 1,
    disp('VCONC: Error - Operates only on modes 1 thru 6'),
    mode,
    abort,
  end,
  if mod1 > 6,
    disp('VCONC: Error - Operates only on modes 1 thru 6'),
    mode,
    abort,
  end,
  if styp1 ~= styp2,
    disp('VCONC: Error - Incompatible system types'),
    abort,
  end,
end
%
%  Check for column compatibility
%
[n1,m1] = logdim(pp1);
[n2,m2] = logdim(pp2);
if debg >= 2,
  if m1 ~= m2,
    disp('VCONC: Error - Incompatible column sizes'),
    abort,
  end,
end
%
%  Get the domain information
%
%
%  Create the new data structure now
%
dom = domain(pp1);
p3 = zerop(n1+n2,m1,mode,logcol,stype,dom);
%
%  Copy the two input data structures into the new one
%
p3 = put(p3,pp1,1:n1,1:m1);
p3 = put(p3,pp2,n1+1:n1+n2,1:m2);
%
pull
return
